;----------------------------------------------------------------------------------------
;COMMUNICATION LAYER 
;-----------------------------------------------------------------------------------------
;serCOM_V1.2.asm -> arbeitet mit getrennten RX/TX-Pufferspeichern
;_________________________________________________________________________________________
;
;KURZBESCHREIBUNG:
;
;Aa)Von SERif_V1.2.asm gelieferte/vernderte Daten und Flag's:
;		- RXframe = 1 dann aktuelles daten-frame im puffer (SRAM)zur Verarbeitung 
;		- SEerror = 1 geliefertes daten-frame fehlerhaft
;		- SEbusy  = 1 datenbertragung aktiv
;
;Ab)An SERif_V1.2.asm zu liefernde Daten und Flag's:
;		- RXframe = nach abarbeitung 0
;		- SEerror =     "            0
;		- TXframe = zur weitern bearbeitung 1 und frame in den puffer (SRAM)
		


;--------------------------------------------------------------------------------
;->HEADER BYTES
;--------------------------------------------------------------------------------
;		           _______________________________________________________________
;mode-identifier: 	|mi7    |mi6    |mi5    |mi4    |mi3    |mi2    |mi1    |mi0    |
;	              	|_______|_______|_______|_______|_______|_______|_______|_______|
;		          |broadc.|setup  |request|adress |trans1 |trans0 |requ1  |requ0  |
;		          |_______|_______|_______|_______|_______|_______|_______|_______|
;	            	|1=adr. |1=setup|1=req  |1=seq. |data to|data to|request|request|
;                 	|0=broa.|0=komm.|0=n.req|0=gezi.|hostTAR|snetTAR|hostSRC|snetSRC|
;		          |_______|_______|_______|_______|_______|_______|_______|_______| 
;
;
;
;.equ	matrixCNT		=		;matrix step counter
;.equ	repeatCNT		=		;tx-repeat counter
;.equ	COPYtempSOURCEl	=		;kopier-adressvariable low-byte
;.equ	COPYtempSOURCEh	=		;kopier-adressvariable high-byte
;.equ	bufCNT			=		;adress-counter (buffer)
;.equ	tempADRA		=		;temp.adress variable
;----------------------------------------------------------------------------------
;Flags:
;.equ	b_cast		=	7
;.equ	RFsetup		=	6
;.equ	request		=	5
;.equ	adr_mod		=	4
;.equ	trans1		=	3
;.equ	trans0		=	2
;.equ	requ1		=	1
;.equ	requ0		=	0
;
;----------------------------------------------------------------------------------
;->MATRIX
;----------------------------------------------------------------------------------
;aufgabe: header-bytes decodieren & nutzdaten entsprechend verarbeiten
;
_serxHOMErun:
	clr	r16
	rjmp	_serxHOME
	;=================

_serial_COM_entry:
	lds	r16,matrixCNT		;lade matrix counter variable
	lds	r19,SEflagVARa		;serial flag variable A laden
	lds	r22,SEflagVARb		; "             "     B   "
	lds	r17,SEflagVARc		;  "            "     C
	;zuerst prfen ob serielle if enabled (...via program - mode )
	sbrs	r19,SEenabled
	rjmp		_serxHOMErun	
	;prfe ob framebuilder gerade rxbuffer belegt !
	sbrc	r17,FRbuild
	rjmp		_serxHOMEnewTXck
	;serif noch beschftigt
	sbrc	r19,SEbusy		
	rjmp		_serxHOMErun	;... bye bye
	;da luft noch ein job der den inhalt von rxbuffer
	;bentigt...
	sbrc	r19,TXframe
	rjmp		_serxHOMErun
	rjmp		_serx001_COM
_serxHOMEnewTXck:
	sbrc	r22,newTX
	rjmp		_serx001_COM	;newTX gesetzt == weiter
	rjmp		_serxHOMErun



;------------------------------
;Startstatus =0
;------------------------------
_serx001_COM:	
	cpi	r16,0
	brne	_serx002_COMJ		;da luft was ab....
	;jetzt prfen ob ein rx-frame im puffer wartet !
	sbrs	r19,RXframe		;
	rjmp	_ser001_TXini		;...kein frame im rx puffer, aber... 
	;empfangenes mi-byte		
	ldi	zl, low(savAREArx)	;Add low byte ( x -(-y)) = x + y
	ldi	zh, high(savAREArx)	;Add high byte with carry
	ld	r17,z
	;.... sollte aber fehlerfrei sein !
	sbrc	r19,SEerror
	rjmp	_serx001_Error
	ldi	r16,2				;MATRIX-Counter=2
	rjmp	_serxHOMEactiv
_serx002_COMJ:
	rjmp	_serx002_COM



;-----------------------------
;empfangenes frame defekt
;-----------------------------
;...keine weiterverarbeitung da error-flag gesetzt !
_serx001_Error:
	cbr	r19,1<<RXframe 
	cbr	r19,1<<SEerror
	rjmp	_serxHOME




;----------------------------------------------------------------------------------
;BUILD TX-FRAME ....
;----------------------------------------------------------------------------------
	;unterscheide zwischen request-frame und new-frame !!!
	;bedeutet: a) ein request-frame ist ein auf anforderung eines 
	;	          anderen hosts produziertes frame 
	;	       b) ein new-frame wurde auf veranlassung eines (ex)internen
	;	          ereignisses von diesem host produziert
    ;          c) ein twi-job muss vorher noch erledigt werden!
_ser001_TXini:
 	sbrc	r22,reqTX		;signalisiert dass der pufferinhalt gesendet werden mu
	rjmp	_serx001_TXreq		;
	sbrc	r22,newTX
	rjmp	_serx001_TXnew
	;ruhezustand
	ldi	r16,0			;MATRIX-Counter =0
	rjmp	_serxHOME		;da trifft nichts zu!!!
	;
;-----------------------------------------
;matrix counter =35...build request frame
;-----------------------------------------
_serx001_TXreq:
	ldi	r16,35
	rjmp	_serx035_COM
;-----------------------------------------
;matrix counter =40...new frame
;-----------------------------------------
_serx001_TXnew:
	ldi	r16,40
	rjmp	_serx040_COM






;----------------------------------------------------------------------------------
;DECODIERE MI....
;DYNAMISCHES MODUL
;----------------------------------------------------------------------------------
;mode-identifier: 
;		  	|mi7    |mi6    |mi5    |mi4    |mi3    |mi2     |mi1    |mi0    |
;		  	|_______|_______|_______|_______|_______|________|_______|_______|
;		  	|broadc.|RFsetup|request|adr_mod|trans1 |trans0  |requ1  |requ0  |
;		  	|_______|_______|_______|_______|_______|________|_______|_______|
;         	|1=adr. |1=set  |1=req  |1=inkr.|data to|data to |request|request|
;         	|0=broa.|       |0=n.req|0=gezi.|hostTAR|snetTAR |hostSRC|snetSRC|
;         	|       |       |       |       |_______|________|_______|_______|
;		  	|	    |       |       |       |trans1&0=1 SAB  |requ1&0=1 Core |
;		  	|_______|_______|_______|_______|________________|_______________| 


_serx005_COMv:
	jmp	_serx005_COM	

	;MATRIX-COUNTER =2 -> decodiere mode identifier & typisiere
_serx002_COM:
	cpi	r16,2
	brne	_serx005_COMv
	; buffer =0
	clr	zh
	sts	bufCNT,zh

	;*****************************************************************************************************
	;newTX=1...bedeutet da nun die lokal generierten bufferdaten fr extern oder twi aufbereitet werden
	;newTX=0...datenframe stammt von extern
	sbrc	r22,newTX		
	rjmp	_TOtargetSUBZorNOT
	;*****************************************************************************************************


	;
	;**** NEU ***********************************************************
    ;von extern empfangene daten vorhanden !
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;	  trans1(1)= von extern empfangene daten im core aufbereiten  ~
	;	  trans0(1)= von extern empfangene daten direkt zum SAB       ~
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	ldi	zl, low(RXbufferST)	
	ldi	zh, high(RXbufferST)
	ld	r18,z			;hole byte
	sts	teMIstore,r18	;mi-puffern (...wg manipulation)
	;mi nun in r18...
	;********************************************************

	sbrc	r18,trans1		
	rjmp	_TOtargetHO		;...externe daten f. das system oder request bestimmt ?
	sbrc	r18,trans0
	rjmp	_TOtargetSUBX	;...externe daten f. subnet bestimmt ? 
    rjmp    _SERdoNOTHING
    ;
_SERdoNOTHING:
	;keine bestimmung der daten erkennbar!
	cbr	r19,1<<RXframe		;... nicht plausibel 
	cbr	r19,1<<TXframe
	clr	r16			;matrix-counter =0
	rjmp	_serxHOME
	;
	;
	;
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;VON EXTERN direct to (S)ub (A)rea (B)us
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;tja..anforderung kam von extern 
    ;evtl. request wird spter bearbeitet
    ;
	;nun bereits fr TXack vorbereiten
	;...TXframe wird erst gesetzt wenn's soweit ist
_TOtargetSUBX:
;	sbr	r22,1<<TXack ;
	cbr	r19,1<<RXframe
	;lade sub-area-target adresse
	clr	zh	
	ldi	zl,2			;empfangene sub-area-adressbyte		
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	ld	r17,z
	sts	twiADR,r17		;speichere twi-adresse
	;weiter zum TWI-Modul...
	rjmp	_TOtargetSUB
    ;
	;
	;
	;
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;vom internen DMA/OBJECT-Profilframer 
    ;REQUESTANFORDERUNG an INTERN TWI oder EXTERN LAB/SAB?
	;frame im TXbuffer gesichert
    ;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    ;
_TOtargetSUBZorNOT:
		;*** NEU ****************************************************
		ldi	zl, low(TXbufferST)	
		ldi	zh, high(TXbufferST)
		ld	r18,z			;hole byte
		sts	teMIstore,r18	;mi-puffern (...wg manipulation)
		;*************************************************************
    ;ist als request definiert ?
        sbrs    r18,request
        rjmp    _TOtargetHOX  	;newTX & kein request

        sbrs	r18,trans0		
        rjmp	_TOtargetLAB	 ;...frame f. das externes LHA/SAB bestimmt !
	    sbrs	r18,trans1
        rjmp    _TOtargetSAB     ;...frame f. das externes SAB bestimmt !
                    ;
					;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
					;vom internen DMA-Profilframer 
					;REQUESTANFORDERUNG VON CORE AN SAB
					;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                    ;trans0=1 & trans1=1
                    ;frame fr's interne SAB !
                    cbr	r19,1<<RXframe
	                ;lade sub-area-source adresse aus frame
	                clr	zh	
	                ldi	zl,4			;empfangene sub-area-adressbyte		
	                subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	                sbci	zh, high(-TXbufferST)	;Add high byte with carry
                	ld	r17,z
	                sts	twiADR,r17		;speichere twi-adresse
	                ;weiter zum TWI-Modul...
	                rjmp	_TOtargetSUB
   

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;vom internen DMA-Profilframer 
;REQUESTANFORDERUNG TO EXTERN SAB/LAB)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

_TOtargetLAB:
    sbrs    r18,trans1
    rjmp    _SERdoNOTHINGb
    ;ja..fr's externe lab !
    ;== frame so wie es ist senden !!!!
_TOtargetSAB:
    cbr	r22,1<<reqTX
	sbr	r19,1<<TXframe
	cbr	r22,1<<newTX
	clr	r16
	rjmp	_serxHOME

;nicht plausibel !
_SERdoNOTHINGb:
        cbr r22,1<<newTX
        jmp _SERdoNOTHING



								;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
								;vom internen DMA-Profilframer 
								;wenn's ein datenobjekt ist, dann direkt verschicken
								;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
								;
							_TOtargetHOX:
								sbrc	r18,RFsetup			;
								rjmp	_TOrfmSETUP			;setup instruction to interface 
								;-> TYPE AUS PUFFER LADEN
								clr	zh
								ldi	zl,6
								subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
								sbci	zh, high(-TXbufferST)	;Add high byte with carry
								ld	r20,z			;hole typebyte
					
								;1)= 1:1 SRAM Kopie
								cpi	r20,1
								brne	_TOtargetHOXb
								ldi	r16,15			;build frame...
								rjmp	_serxHOMEactiv	;...und zurck

							
								;2)= OBJECT
							_TOtargetHOXb:
								cpi	r20,250
								brne	_TOtargetHOXc	;gem.type table als standard behandeln !
								jmp	_serx011txLABob		;== objekt direkt senden...
	

								;3)= FUTURE TYPE
							_TOtargetHOXc:
								jmp		_serx003_COMerr





						;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						;vom internen DMA-Profilframer 
						;wenn's ein datenobjekt ist, dann direkt verschicken
						;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
						_TOrfmSETUP:
							ldi	r16,5			;build frame...
							rjmp	_serx005_COM		;...und weiter











    ;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    ;VON EXTERN // REQUEST ODER CORE
    ;fr bearbeitung im CORE vorbereiten....
    ;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;+++..............................................................................................................
	;Hier type decodieren und entsprechend der Tabelle
	;den weiteren transfer entscheiden !
	;Z.Zt. nur Type:
	;0   = Error
	;1   = 1:1 SRAM Kopie
	;250 = Data-Object
	;255 = Acknowledge

_TOtargetHO:
	;-> TYPE AUS PUFFER LADEN
	clr	zh
	ldi	zl,6
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	ld	r20,z			;hole typebyte



;..............................
;1)  CORE / REQUEST
;..............................
_serx003_COMa:
;1)= 1:1 SRAM Kopie
	cpi	r20,1
	brne	_serx003_COMb
	sbrs	r18,request
	rjmp	_ser003_COManoreq
	;request
	ldi	r16,17
	rjmp	_serxHOMEactiv	;...und zurck
	;to core
_ser003_COManoreq:
	ldi	r16,10
	rjmp	_serxHOMEactiv	;...und zurck
	;

;...................................
;8)  receive & store the host-table
;...................................
_serx003_COMb:
	cpi	r20,8
	brne	_serx003_COMc
	sbrs	r18,RFsetup     ;only in setup mode !
	rjmp	_serx003_COMerr
    ;..........................
    ;NOTE: Every host-frame 
    ;contains a 10byte payload
    ;So serx020 becomes a kind
    ;of storage loop, because 
    ;every write operation to
    ;e2p needs a lot of time.
    ;FIRST STEP:
    ;clear the payload counter
    ;
    ;..........................
    ;payload
    clr r16
    sts bufCNT,r16
    ;matrix
	ldi	r16,20
	rjmp	_serxHOMEactiv	;...und zurck








;...........................................
;10)  received the remote slave instruction
;...........................................
_serx003_COMc:
	cpi	r20,10
	brne	_serx003_COMd
	sbrs	r18,RFsetup     ;only in setup mode !
	rjmp	_serx003_COMerr
     ;............................
    ;NOTE: The interface received
    ;the remote instruction from
    ;a master. Now set the system
    ;to slave mode !
    ;Slave-Mode:
    ;- Disable the net-framer
    ;- Only HIF Communication  
    ;..........................
    ;remote-flags
    clr	r16
	sbr	r16,1<<RTactivS
    sbr r16,1<<RTon
    sts remFLAG,r16
    ;system-state defaults
	;prozPSC=1
	;progPSC=0
	ldi	r16,1	
	sts	prozPSC,r16
	clr	r16
	sts	progPSC,r16
	;ausfhren
	clr	r16
	sbr	r16,1<<exeSPIdata
	sbr	r16,1<<spiMready
	sts	comFLAGspi,r16
    ;load master's adress as
    ;the slave adress...
	;-> TYPE AUS PUFFER LADEN
	clr	zh
	ldi	zl,1
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
    ld  r16,z
    sts REMslave,r16
    ;ready...
	rjmp	_serx003_COMerr	;...und zurck




;...........................................
;11)  received the remote off instruction
;...........................................
_serx003_COMd:
	cpi	r20,11
	brne	_serx003_COMdX
	sbrs	r18,RFsetup     ;only in setup mode !
	rjmp	_serx003_COMerr
    ;............................
    ;NOTE: The interface received
    ;the remote-off instruction 
    ;from a master or generates
    ;this intruction in order of
    ;a timeout itself. Now disable 
    ;the remote...
    ;RemoteOff-Mode:
    ;- Enable the net-framer 
    ;..........................
    ;remote-flags
    clr	r16
    sts remFLAG,r16
	sts	remTEfl,r16		;lokale remflags

    ;system-state defaults
	;prozPSC=1
	;progPSC=0
	ldi	r16,1	
	sts	prozPSC,r16
	clr	r16
	sts	progPSC,r16
	;ausfhren
;	clr	r16
;	sbr	r16,1<<exeSPIdata
;	sbr	r16,1<<spiMready
;	sts	comFLAGspi,r16
    ;ready...
	rjmp	_serx003_COMerr	;...und zurck




;...........................................
;99)  received the reset instruction
;...........................................
_serx003_COMdX:
	cpi	r20,99
	brne	_serx003_COMe
	sbrs	r18,RFsetup     ;only in setup mode !
	rjmp	_serx003_COMerr
    ;............................
    ;NOTE: The interface received
    ;the reset instruction 
    ;=> bootloader starting...
    ;............................
    cli
    ldi zh,high(BOOTstart)
    ldi zl,low(BOOTstart)
    ijmp



















;..............................
;for the future:
;check type 12...249
;..............................




;+++.............................................................
	;HIER entscheiden ob es sich um ein globales Datenobjekt 
	;handelt...
	;BASIS: type=250
	;-> Immer Broadcast adressiert (kein repeated tx)
	;-> Kein Acknowledge (bei Empfang)
	;INFO: Frame bereits vollstndig im objectframer generiert !
;+++.............................................................
;..............................
_serx003_COMe:
;250)= Data Object
	cpi	r20,250
	brne	_serx003_COMf			;
	;verzweige nach objects...
	ldi	r16,12
	jmp	_serx012_COM				;empfangenes objekt filtern...


;+++.............................................................
	;HIER entscheiden ob es sich um einen bootbefehl 
	;handelt...
	;BASIS: type=252
	;-> Immer Broadcast adressiert (kein repeated tx)
	;-> Kein Acknowledge (bei Empfang)
;+++.............................................................
;..............................
_serx003_COMf:
;252)= boot-befehl
	cpi		r20,252
	brne	_serx003_COMerr			;nichts dabei... / fehler==zurck !
	jmp		_BOOT000entry




;..........................
;type=fehler
_serx003_COMerr:
	;transferflags clear
	cbr	r22,1<<reqTX
	cbr	r19,1<<TXframe
	cbr	r19,1<<RXframe
	cbr	r22,1<<newTX
	;matrix-counter =0
	clr	r16
	jmp	_serxHOME
;..........................






;---------------------------------------------------------------------------------
;transmit setup instruction to the interface
;---------------------------------------------------------------------------------
_serx005_COM:
	cpi	r16,5
	brne	_serx010_COM
	cbr	r22,1<<reqTX
	sbr	r19,1<<TXframe
	cbr	r22,1<<newTX
	clr	r16
	rjmp	_serxHOME








;----------------------------------------------------------------------------------
;ADRESSIERUNG IN ALLEN SRAM SEGMENTEN
;NUR VON EXTERN NACH CORE ODER TWI!
;16-BIT ADRESSBEREICH
;..................................................................................

;DYNAMISCHES MODUL
;registerbelegung:
;r16,matrixCNT		;matrix counter variable
;r19,SEflagVARa		;serial flag variable A 
;r22,SEflagVARb		; "             "     B   "
;r17,bufCNT			;adress-counter (puffer)
;r20/frame size
;zl/zh z-pointer
;r23/puffer-daten
;r22/steuerflags
;--------------------------------------------------------------------------------
	;MATRIX-COUNTER =10 -> bestimme startadresse sram
                  _serx011_COMc:
                         jmp     _serx011_COM


_serx010_COM:
	cpi	r16,10
	brne	_serx011_COMc
	;adressiere puffer(16bit) 	
	ldi	r17,9
	clr	zh	
	mov	zl,r17			;
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	;adresse aus puffer nach r23/r20 - target-adress
	ld	r23,z+
	ld	r20,z	
	;+korrekturfaktor sram
	subi	r20,low(-koMAP)
	sbci	r23,high(-koMAP)
	;
	sts	targetADRA,r20		;LSB/speichere effektive target-startadresse 
	sts	targetADRB,r23		;MSB 
	;nchster schritt
	ldi	r16,11		;matrix counter =11
	ldi	r17,11		;adresscounter  =11
	sts	bufCNT,r17	;bufCNT sichern
	rjmp	_serxHOMEactiv
	;
	;
	;
;.................................................................................
;- hier framesize mit aktuellem status vergleichen 
;- hier bertragungsmode prfen und entsprechend verzweigen (request/newTX/...)
;.................................................................................
_serx011_READYx:
	jmp	_serx011_READY			
_serx012_COMx:
	rjmp	_serx012_COM
	;MATRIX-COUNTER =11 -> 
_serx011_COM:
	cpi	r16,11
	brne	_serx012_COMx
	lds	r17,bufCNT	;lade momentanen zhlerstand
	;lade framesize...
	clr	zh	
	ldi	zl,5			;empfangenes framesize-byte		
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	ld	r20,z			;frame-size
	;vergleiche adresszhler/framesize
	cp	r17,r20
	brsh	_serx011_READYx		;... fertig verarbeitet (nun sollte TXack und TXframe...folgen)
	rjmp	_serx011_NOREQ

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;KEIN request oder newTX-anforderung
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx011_NOREQ:
	;adressiere puffer wg. speicherung...
	;adressiere puffer(16bit)= ( RXbufferST + bufCNT)
	;speichere byte aus segment nach buffer
	cbr	r22,1<<reqTX		;signalisiert dass nur ein TXack gesendet werden muss
	clr	zh	
	mov	zl,r17			;		
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	ld	r23,z			;daten aus puffer nach r23 - zieladresse in allen sram-segmenten

	lds	zh,targetADRB
	lds	zl,targetADRA		;sram-adressvariable nach z-pointer(l)...
	st	z+,r23			;daten aus r23 nun in speichersegment 
	sts	targetADRA,zl
	sts	targetADRB,zh



;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;gezielt oder sequenziell
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;mi laden
	lds	r16,teMIstore
	sbrc	r16,adr_mod		;1=sequ / 0=gezielt
	rjmp	_serx011_BACK	;sequenziell

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;gezielte adressierung
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;zeigt auf msb target-adresse
	ldi	zh,3
	add	r17,zh
	clr	zh	
	mov	zl,r17			;		
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	ld	r23,z+			;daten aus puffer nach r23 - zieladresse in allen sram-segmenten
	ld	r20,z
	;+korrekturfaktor sram
	subi	r20,low(-koMAP)
	sbci	r23,high(-koMAP)

	sts	targetADRA,r20
	sts	targetADRB,r23
	ldi	zh,1
	add	r17,zh
	jmp	_serx011_BACK






;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;dynamisches modul ret
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx011_BACK:
	;puffer-counter +1
	ldi	r23,1
	add	r17,r23			;puffer-counter+2
	sts	bufCNT,r17		;bufCNT sichern
	;matrix =11
	ldi	r16,11
	rjmp	_serxHOMEactiv	;...und zurck




;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;ready
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx011_READY:	
	;eindeutig...
	;daten waren fr's system bestimmt
	cbr	r19,1<<RXframe		;rx frame komplett verarbeitet
	cbr	r22,1<<reqTX		
	clr	r16
	rjmp	_serxHOME















;----------------------------------------------------------------------------------
;NEUES DATENOBJEKT VERSCHICKEN 
;----------------------------------------------------------------------------------
_serx011txLABob:
	lds	r19,SEflagVARa		;serial flag variable A laden
	lds	r22,SEflagVARb		; "             "     B   "
	cbr	r22,1<<reqTX
	sbr	r19,1<<TXframe
	cbr	r22,1<<newTX
	clr	r16
	rjmp	_serxHOME
	;
;----------------------------------------------------------------------------------
;NEUES DATENOBJEKT EMPFANGEN... FILTERN USW....
;----------------------------------------------------------------------------------
;<CLASS_FILTER_DEFINE>	-> 	beinhaltet:	<data_class>	0...255
;										<client_adr>	0...255
;										<Xdata to sys>  0..4(max.5*8Bit Data)/255=no data to system
;										<sysADR>,<sysADR>...
;
;----------------------------------------------------------------------------------
;objectframe (sercom-format)
; ______________________________________________________________________________________
;|mi    |ho-tar |sub-tar|ho-sou |sub-sou|frame  |type   |data   |eol    |var-siz|data   |
;|byte  |adress |adress |adress |adress |size=  |=250   |class  |16bit  |max.5* |0...4  |
;|      |       |       |       |       |21     |object |       |msb/lsb|data   |msb/lsb|
;|______|_______|_______|_______|_______|_______|_______|_______|_______|_______|_______|
;
;.equ	recDCLASS	=	0x7bd
;.equ	recLABSOU	=	0x7be
;
;A)data-class & LAB-source-adress temporr speichern
_serx012_COM:
	cpi	r16,12
	brne _serx013_COM





	;hole data-class
	clr	zh	
	ldi	zl,7						;data-class adressieren...		
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	ld	r17,z	;data-class in r17
	;data-class temporr speichern
	sts	recDCLASS,r17
	;
	;.....................................................................
	;
	;hole lab-source adress
	clr	zh	
	ldi	zl,3						;lab-source adressieren...		
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	ld	r17,z	;lab-source in r17
	;lab-source temporr speichern
	sts	recLABSOU,r17
	;
	;pointer auf erste data-class im ersten filtersegment
	ldi	zl,low(class_filter_def)
	ldi	zh,high(class_filter_def)
	sts	targetADRA,zl
	sts	targetADRB,zh
	;
	;pointer auf erste source-lha im ersten filtersegment
	adiw	zh:zl,1
	sts	sourceADRA,zl
	sts	sourceADRB,zh
	ldi	r16,13
	rjmp	_serxHOMEactiv
	;
	;......................................................................
	;
;B)prfe ob data-class in den filtersegmenten enthalten ist
;-> JA!.... dann identitt lha prfen
;-> NEIN!.. objekt verwerfen & sercom wieder freigeben
_serx013_COM:
	cpi	r16,13
	brne _serx014_COM
	;data-class aus filtersegment
	lds	zh,targetADRB
	lds	zl,targetADRA
_serx013LOOP:
	lds	r18,recDCLASS	;empfangene class
	ld	r17,z+
	;vergleichen...
	cp	r17,r18
	breq	_serx013EQU	;dataclass gefunden
	;
_serx013LIMIT:
	adiw	zh:zl,class_filter_segment - 1	;pointer auf nchste data-class im filter
	;vor loop zuerst limit prfen
	cpi	zl,low(class_filter_limit)	;Compare low byte
	ldi	r18,high(class_filter_limit)	;
	cpc	zh,r18	;Compare high byte
	brsh	_serx013END	;limit errecht == fertig !
	jmp	_serx013LOOP	;loop...
	;
	;lha aus filtersegment holen und vergleichen
_serx013EQU:
	lds	r18,recLABSOU
	tst	r18
	breq	_serx013RUN	;filterwert=0 akzeptiert die dataclass einer beliebigen quelle
	ld	r17,z	;
	;vergleichen...
	cp	r17,r18
	breq	_serx013RUN	;lha der quelle im filtersegment gefunden...
	jmp	_serx013LIMIT

_serx013RUN:
	ldi	r16,14
	jmp	_serxHOMEactiv

;..............................................................................
;objekt ablehnen...
;..............................................................................
_serx013END:
	cbr	r22,1<<reqTX
	cbr	r19,1<<TXframe
	cbr	r19,1<<RXframe
	cbr	r22,1<<newTX
	clr	r16
	rjmp	_serxHOME	

;..............................................................................
;objekt akzeptiert... 9byte umfassendes object temp. sichern !
;..............................................................................

	;kopiere objekt in temporres speichersegment
_serx014_COM:
	cpi	r16,14
	brne _serx015_COM

    ;********* NEU ***************
    ;letztes objekt noch in bearbeitung ? 
	lds	r18,objectFLAG
	sbrs r18,newOB	;neues objekt
	rjmp    _serx014_COMrun
    jmp 	_serxHOMEactiv
    ;*****************************

_serx014_COMrun:
	;adressiere dataclass
	clr	zh	
	ldi	zl,7						;data-class adressieren...		
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	;initalisiere temporres speichersegment
	ldi	yh,high(dyn_object_TE)
	ldi	yl,low(dyn_object_TE)
	ld	r17,z
	st	y+,r17	;data-class temporr sichern
	;adressiere lab-source-adress (datenquelle)
	clr	zh	
	ldi	zl,3						;lab-source adressieren...		
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	ld	r17,z
	st	y+,r17	;lab-source-adress temporr sichern
	;adressiere eol
	clr	zh	
	ldi	zl,8						;data-class adressieren...		
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	;
_serx014LOOP:
	ld	r17,z+	;
	st	y+,r17
	;teste limit		
	cpi	yl,low(dyn_object_limitTE +1)	;Compare low byte
	ldi	r18,high(dyn_object_limitTE +1)	;
	cpc	yh,r18	;Compare high byte
	brsh	_serx014END	;limit errecht == fertig !
	jmp	_serx014LOOP	;loop...
	;
_serx014END:
	;fertig...
	;- nun ,new object'-flag setzen
	;- sercom freigeben
	lds	r18,objectFLAG
	sbr	r18,1<<newOB	;neues objekt
	sts	objectFLAG,r18
	jmp	_serx013END		;sercom freigeben...
	












;----------------------------------------------------------------------------------
;ADRESSIERUNG IN ALLEN SRAM SEGMENTEN
;PROFILFRAMER NACH EXTERN ODER TWI!
;16-BIT ADRESSBEREICH
;..................................................................................
;DYNAMISCHES MODUL
;registerbelegung:
;r16,matrixCNT		;matrix counter variable
;r19,SEflagVARa		;serial flag variable A 
;r22,SEflagVARb		; "             "     B   "
;r17,bufCNT			;adress-counter (puffer)
;r20/frame size
;zl/zh z-pointer
;r23/puffer-daten
;r22/steuerflags
;--------------------------------------------------------------------------------
	;MATRIX-COUNTER =15 -> bestimme startadresse sram
                  _serx016_COMc:
                         jmp     _serx016_COM


_serx015_COM:
	cpi	r16,15
	brne	_serx016_COMc
	;adressiere puffer(16bit) 	
	ldi	r17,7
	clr	zh	
	mov	zl,r17			;
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry
	;adresse aus puffer nach r23/r20 - source-adresse
	ld	r23,z+
	ld	r20,z+	
	;+korrekturfaktor sram
	subi	r20,low(-koMAP)
	sbci	r23,high(-koMAP)
	;		
	sts	sourceADRA,r20		;LSB/speichere effektive source-startadresse 
	sts	sourceADRB,r23		;MSB

	;nchster schritt
	ldi	r16,16		;matrix counter =16
	ldi	r17,11		;adresscounter  =14
	sts	bufCNT,r17	;bufCNT sichern
	rjmp	_serxHOMEactiv
	;
	;
	;
;.................................................................................
;- hier framesize mit aktuellem status vergleichen 
;- hier bertragungsmode prfen und entsprechend verzweigen (request/newTX/...)
;.................................................................................
_serx016_READYx:
	jmp	_serx016_READY			
_serx017_COMx:
	rjmp	_serx017_COM
	;MATRIX-COUNTER =11 -> 
_serx016_COM:
	cpi	r16,16
	brne	_serx017_COMx
	lds	r17,bufCNT	;lade momentanen zhlerstand
	;lade framesize...
	clr	zh	
	ldi	zl,5			;empfangenes framesize-byte		
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry
	ld	r20,z			;frame-size
	;vergleiche adresszhler/framesize
	cp	r17,r20
	brsh	_serx016_READYx		;... fertig verarbeitet (nun sollte TXack und TXframe...folgen)
	rjmp	_serx016build
	;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;newTX-anforderung
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx016build:
	;....request == sende zum definierten ziel!
	;
	sbr	r22,1<<reqTX		;signalisiert dass der pufferinhalt gesendet werden muss

	lds	zh,sourceADRB
	lds	zl,sourceADRA		;sram-adressvariable nach z-pointer(l)...
	;16-bit adressing
	ld	r23,z+			    ;source-daten aus speichersegment nun in r23
	sts	sourceADRA,zl
	sts	sourceADRB,zh
	;adressiere puffer wg. speicherung...
	;adressiere puffer(16bit)= ( RXbufferST + bufCNT)
	;speichere byte aus segment nach buffer
	;position: source-data
	clr	zh	
	mov	zl,r17			;	
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry
	st	z+,r23						;daten aus r23 nach  - -segmenten


;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;gezielt oder sequenziell
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;mi laden
	lds	r16,teMIstore
	sbrc	r16,adr_mod		;1=sequ / 0=gezielt
	rjmp	_serx016_BACK	;sequenziell

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;gezielte adressierung
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;zeigt auf msb source-adresse
	ldi	zh,1
	add	r17,zh
	clr	zh	
	mov	zl,r17			;		
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry
	ld	r23,z+			;daten aus puffer nach r23 - zieladresse in allen sram-segmenten
	ld	r20,z

	;+korrekturfaktor sram
	subi	r20,low(-koMAP)
	sbci	r23,high(-koMAP)

	sts	sourceADRA,r20
	sts	sourceADRB,r23
	;zeigt auf lsb target-adress
	ldi	zh,3
	add	r17,zh
	jmp	_serx016_BACK
	;
	;

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;dynamisches modul ret
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx016_BACK:
	;puffer-counter +1
	ldi	r23,1
	add	r17,r23			;puffer-counter+2
	sts	bufCNT,r17		;bufCNT sichern
	;matrix=16
	ldi	r16,16
	rjmp	_serxHOMEactiv	;...und zurck
	;
	;
	;





_serx016_READY:	
	;**********************************************************
	;eindeutig...
	;ist ein eben generiertes event-frame, aber....
	;a) auf LAB ausgeben ?
	;b) auf SAB ausgeben ?
	;
	;manipulation aufheben
	lds	r17,teMIstore		;das alte mi-byte holen
	ldi	zl, low(TXbufferST)	;Add low byte ( x -(-y)) = x + y
	ldi	zh, high(TXbufferST)	;Add high byte with carry
	st	z,r17
	;
	;nun transferflags testen
	sbrs	r17,trans0		      ;wenn trans0 & trans1 ==1
	rjmp		_serx016txLAB	;+
	sbrc	r17,trans1			;+		
	rjmp		_serx016txSAB	;dann direkt ber twi ausgeben...
	;
_serx016txLAB:	
	cbr	r22,1<<reqTX
	sbr	r19,1<<TXframe
	cbr	r22,1<<newTX
	clr	r16
	rjmp	_serxHOME
	;
_serx016txSAB:
	cbr	r22,1<<reqTX
;	cbr	r22,1<<newTX	;TESTWEISE AKTIV LASSEN !
	cbr	r19,1<<TXframe
	;in diesem fall an sab-source adressiert !!!
	clr	zh	
	ldi	zl,4						;sub-area-adressbyte		
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry
	ld	r17,z
	sts	twiADR,r17		;speichere twi-adresse
	;weiter zu TWI-Modul
	rjmp	_TOtargetSUB






;----------------------------------------------------------------------------------
;ADRESSIERUNG IN ALLEN SRAM SEGMENTEN
;REQUEST
;16-BIT ADRESSBEREICH
;..................................................................................
;DYNAMISCHES MODUL
;registerbelegung:
;r16,matrixCNT		;matrix counter variable
;r19,SEflagVARa		;serial flag variable A 
;r22,SEflagVARb		; "             "     B   "
;r17,bufCNT			;adress-counter (puffer)
;r20/frame size
;zl/zh z-pointer
;r23/puffer-daten
;r22/steuerflags
;--------------------------------------------------------------------------------
	;MATRIX-COUNTER =17 -> bestimme startadresse sram
                  _serx018_COMc:
                         jmp     _serx018_COM


_serx017_COM:
	cpi	r16,17
	brne	_serx018_COMc
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;kopiere header von RXbuffer
	;nach TXbuffer
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~
	lds	r17,bufCNT
	cpi	r17,7
	brsh	_serx017_COMgo
	;RXbuffer+bufCNT
	clr	zh	
	mov	zl,r17			;
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	;TXbuffer+bufCNT
	clr	yh	
	mov	yl,r17			;
	subi	yl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	yh, high(-TXbufferST)	;Add high byte with carry
	;kopiere
	ld	r2,z
	st	y,r2
	;bufCNT+1
	inc	r17
	sts	bufCNT,r17
	;zurck
	jmp	_serxHOMEactiv






_serx017_COMgo:
	;adressiere RXbuffer(16bit) 	
	ldi	r17,7
	clr	zh	
	mov	zl,r17			;
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	;adresse aus RXbuffer nach r23/r20 - source-adresse
	ld	r23,z+
	ld	r20,z+	
	;adressiere TXbuffer
	clr	yh	
	mov	yl,r17			;
	subi	yl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	yh, high(-TXbufferST)	;Add high byte with carry
	;sichere source adresse msb/lsb 
	st	y+,r23
	st	y+,r20
	;
	;+korrekturfaktor sram
	subi	r20,low(-koMAP)
	sbci	r23,high(-koMAP)
	;effektiv-adresse sichern		
	sts	sourceADRA,r20		;LSB/speichere effektive source-startadresse 
	sts	sourceADRB,r23		;MSB
	;adresse aus puffer nach r23/r20 - target-adress
	ld	r23,z+
	ld	r20,z	
	;sichere target adresse msb/lsb 
	st	y+,r23
	st	y,r20

	;nchster schritt
	ldi	r16,18		;matrix counter =18
	ldi	r17,11		;adresscounter  =14
	sts	bufCNT,r17	;bufCNT sichern
	rjmp	_serxHOMEactiv
	;
	;
	;
;.................................................................................
;- hier framesize mit aktuellem status vergleichen 
;- hier bertragungsmode prfen und entsprechend verzweigen (request/newTX/...)
;.................................................................................
_serx018_READYx:
	jmp	_serx018_READY			
_serx020_COMx:
	rjmp	_serx020_COM
	;MATRIX-COUNTER =11 -> 
_serx018_COM:
	cpi	r16,18
	brne	_serx020_COMx
	lds	r17,bufCNT	;lade momentanen zhlerstand
	;lade framesize...
	clr	zh	
	ldi	zl,5			;empfangenes framesize-byte		
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry
	ld	r20,z			;frame-size
	;vergleiche adresszhler/framesize
	cp	r17,r20
	brsh	_serx018_READYx		;... fertig verarbeitet (nun sollte TXack und TXframe...folgen)
	rjmp	_serx018build
	;
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;request-anforderung
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx018build:
	;....request == sende zum definierten ziel!
	;
;	sbr	r22,1<<reqTX		;signalisiert dass der pufferinhalt gesendet werden muss
	lds	zh,sourceADRB
	lds	zl,sourceADRA		;sram-adressvariable nach z-pointer(l)...

	;16-bit adressing
	ld	r23,z+			    ;source-daten aus speichersegment nun in r23
	sts	sourceADRA,zl
	sts	sourceADRB,zh
	;adressiere puffer wg. speicherung...
	;adressiere puffer(16bit)= ( RXbufferST + bufCNT)
	;speichere byte aus segment nach buffer
	;position: source-data
	clr	zh	
	mov	zl,r17			;	
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry
	st	z+,r23						;daten aus r23 nach  - -segmenten

	

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;gezielt oder sequenziell
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;mi laden
	lds	r16,teMIstore
	sbrc	r16,adr_mod		;1=sequ / 0=gezielt
	rjmp	_serx018_BACK	;sequenziell

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;gezielte adressierung
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;zeigt auf msb source-adresse RXbuffer
	ldi	zh,1
	add	r17,zh
	clr	zh	
	mov	zl,r17			;		
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	;zeigt auf msb source-adresse TXbuffer
	clr	yh	
	mov	yl,r17			;		
	subi	yl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	yh, high(-TXbufferST)	;Add high byte with carry
	;lade msb source-adresse aus RXbuffer
	ld	r23,z+
	;sichere msb source-adress nach TXbuffer
	st	y+,r23
	
	;lade lsb source-adresse aus RXbuffer
	ld	r20,z+
	;sichere lsb source-adress nach TXbuffer
	st	y+,r20

	;+korrekturfaktor sram
	subi	r20,low(-koMAP)
	sbci	r23,high(-koMAP)
	sts	sourceADRB,r23
	sts	sourceADRA,r20




	;lade msb target-adresse aus RXbuffer
	ld	r23,z+
	;sichere msb target-adress nach TXbuffer
	st	y+,r23

	;lade lsb target-adresse aus RXbuffer
	ld	r20,z
	;sichere lsb target-adress nach TXbuffer
	st	y,r20

	;zeigt auf lsb target-adress
	ldi	zh,3
	add	r17,zh
	jmp	_serx018_BACK
	;





;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;dynamisches modul ret
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx018_BACK:
	;puffer-counter +1
	ldi	r23,1
	add	r17,r23			;puffer-counter+2
	sts	bufCNT,r17		;bufCNT sichern
	ldi	r16,18		;matrix counter =18
	rjmp	_serxHOMEactiv	;...und zurck
	;
;------------------------------------------
;eindeutig...
;daten waren fr's system bestimmt
;request frame schicken
;------------------------------------------
_serx018_READY:	
	cbr	r19,1<<RXframe		;rx frame komplett verarbeitet
	sbr	r22,1<<reqTX		;signalisiert dass der pufferinhalt gesendet werden muss
	;hier sofort nach txini verzweigen
	rjmp	_ser001_TXini
	;






;------------------------------------------
;received host table to e2prom
;------------------------------------------
_serx020_COM:
    cpi r16,20
    brne _serx035_COM
    ;check the e2p
	sbic	eecr,eewe		            ; EEPE nicht gelscht ?
	rjmp	_serx020_COMloop			; warte...
    ;free
    ;adressiere puffer(16bit) 			;
	ldi	zl,low(RXbufferST)	;Add low byte ( x -(-y)) = x + y
	ldi	zh,high(RXbufferST)	;Add high byte with carry
    adiw    zh:zl,7
    ;+ bufCNT
    clr yh
    lds yl,bufCNT
    add zl,yl
    adc zh,yh
    ;load data
	ld	r18,z               ;load host adress
    ;adressing the e2p
    mov zl,r18
    clr zh
    subi    zl,low(-hostTABLE)
    sbci    zh,high(-hostTABLE)
    ;write to e2p
    cli
	out	eearh,zh
	out	eearl,zl		; adressierung e2prom
	out	eedr,r18          ; kopiere ins'e2prom-datenregister
	sbi	eecr,eemwe	
	sbi	eecr,eewe		;set EEPROM Write strobe
					; nach 4 maschinenzyklen ist der schreibprozess abgeschlossen 			
    ;check payload counter
    cpi yl,9
    brsh _serx020_COMready   
    inc yl
    sts bufCNT,yl
    sei
   ;back in loop    
_serx020_COMloop:
    jmp	_serxHOMEactiv	;...und zurck     

    ;ready
_serx020_COMready:
	cbr	r19,1<<RXframe		;rx frame komplett verarbeitet
	cbr	r22,1<<reqTX		
	clr	r16
    sei
	jmp	_serxHOME
    


 




;----------------------------------------------------------------------------------
;BUILD TX_REQUEST-FRAME ....
;DYNAMISCHES MODUL
;----------------------------------------------------------------------------------
;07-01-04

_serx035_COM:
	cpi	r16,35
	brne	_serx040_COM
	;lade ursprun-mi
	lds	r20,TEmiVAR
	;request
	cbr	r20,1<<request
	sbrs	r20,requ0
	rjmp		_serx035toLAB
_serx035toSAB:
	andi	r20,0b11110000
	cbr	r20,1<<requ0
	sbr	r20,1<<trans0
	rjmp		_serx035build
_serx035toLAB:
	sbrs	r20,requ1
	rjmp		_serx035noBUILD
	andi	r20,0b11110000
	cbr	r20,1<<requ1
	sbr	r20,1<<trans1
	;
_serx035build:
	;mi speichern
	ldi	zh,high(TXbufferST)
	ldi	zl,low(TXbufferST)
	st	z,r20
	;hole ursprngliche frame-size 
	;aus saveAREA
	ldi	yh,0				;saveAREA startadresse +5
	ldi	yl,5
	subi	yl,low(-savAREA)
	sbci	yh,high(-savAREA)
	ld	r14,y				;hole daten von saveAREA-adresse y
	;nun framesize in r14
	ldi	zh,0				;RXbuffer startadresse +5
	ldi	zl,5
	subi	zl,low(-TXbufferST)
	sbci	zh,high(-TXbufferST)
	st	z,r14				;hole daten von pufferadresse z
	;hole ursprngliche type-byte 
	;aus saveAREA				;y-pointer
	ldi	yh,0				;saveAREA startadresse +6
	ldi	yl,6
	subi	yl,low(-savAREA)
	sbci	yh,high(-savAREA)
	ld	r14,y
	;hole daten von saveAREA-adresse y
	;nun type-byte in r14
	ldi	zh,0				;RXbuffer startadresse +6
	ldi	zl,6
	subi	zl,low(-TXbufferST)
	sbci	zh,high(-TXbufferST)
	st	z,r14				;hole daten von pufferadresse z
	;fertig..
	;reqTX nun lschen
	;19-12-03
	cbr	r22,1<<reqTX
	;
	;
	ldi	r16,0
	sbr	r19,1<<TXframe
	rjmp	_serxHOME
	;frame wird anschlieend bertragen...
	;
	;fehler...
_serx035noBUILD:
	cbr	r22,1<<reqTX
	ldi	r16,0
	rjmp	_serxHOME

;----------------------------------------------------------------------------------
;SEND TX-FRAME ....
;DYNAMISCHES MODUL
;----------------------------------------------------------------------------------
_serx090_COMx:
	jmp	_serx090_COM
_serx040_COM:
	cpi	r16,40
	brne	_serx090_COMx	
	;nun daten aufbereiten !
	sbrs	r19,TXframe
	rjmp	_serx040_COMgo
	clr	r16
	rjmp	_serxHOME
_serx040_COMgo:
	ldi	r16,2
	;jetzt FRbuild lschen...
	lds	r17,SEflagVARc
	cbr	r17,1<<FRbuild
	sts	SEflagVARc,r17
	;
	rjmp _serxHOMEactiv
	;frame wird anschlieend bertragen...











;1-05
;twi auf neue protokollstruktur anpassen....
;----------------------------------------------------------------------------------
;TWI-MATRIX... ENTRY
;----------------------------------------------------------------------------------
;NEW TWI-BUFFER SIZE 245 BYTES !!!

	;
	;prfe ob anhand der empfangenen daten 
	;a)nach TWI/I2C Format konvertiert und an den sub-area-bus  gesendet ODER
	;b) "         "         "          das sub-area-bus zum request aufgefordert
	;                                  und anschlieend nach decodierung zum
	;				   	     local-area-bus(source) gesendet.
	;c)wie b) jedoch der Weitertransfer z. sub-area-bus definiert.
	;d)nach TWI / I2C Fromat konvertiert und an den sub-area-bus (source) gesendet
	;  sowie request-aufforderung an den core
	;
_TOtargetSUB:
    lds	r18,teMIstore
							;				    sabSOURCE	|	lhaSOURCE
	sbrc	r18,requ1			;			    	requ0 	|   	requ1
	rjmp	_REQsubNETsourceHO	;ja... variante b)	0		|	1
	sbrs	r18,requ0			;		    				|	
	rjmp	_WORKdataSUB		;nein..variante a)	0		|	0
	rjmp	_REQsubNETsourceSUB	;ja... variante c)	1		|	0
	;........							    		    	|



;----------------------------------------------------------------------------------
;REQUEST DATA F. SUB-NET-TARGET(...dieser baustein)  -> LOCAL-AREA-BUS (source ....)
;DYNAMISCHES MODUL
;----------------------------------------------------------------------------------
_REQsubNETsourceHO:			;					        	|
	sbrc	r18,requ0			;				    		|
	rjmp	_REQlokalNET 		;...variante d)		1		|	1
	lds	r20,twiSTATUS	        ;.. variante b)	    0		|	1
	sbr	r20,1<<TWIrequest
    sbr r20,1<<LHAreq   ;-> LHA
    sbr r20,1<<doTWI    ;weitere aktion
                        ;nach evtl.TXack
	sts	twiSTATUS,r20
	ldi	r16,90			;				            		
	rjmp	_serxHOMEactiv		;		
    				
	;.......										

;----------------------------------------------------------------------------------
;SEND DATA -> SUB-AREA-BUS(...dieser baustein) ....
;DYNAMISCHES MODUL [KEIN REQUEST]
;----------------------------------------------------------------------------------
_WORKdataSUB:				;			        	0	   |	0
	lds	r20,twiSTATUS
	cbr	r20,1<<TWIrequest
    cbr r20,1<<LHAreq   ;----
    cbr r20,1<<doTWI    ;keine aktion nach 
                        ;evtl.TXack
	sts	twiSTATUS,r20
	ldi	r16,90			;					        	   
	rjmp	_serxHOMEactiv		;					   
	;......										   

;----------------------------------------------------------------------------------
;REQUEST DATA F. SUB-AREA-BUS(....dieser baustein) -> SUB-AREA-BUS (source....)
;DYNAMISCHES MODUL
;----------------------------------------------------------------------------------
_REQsubNETsourceSUB:			;			        	1  |	0
	lds	r20,twiSTATUS
	sbr	r20,1<<TWIrequest
    sbr r20,1<<LHAreq   ;-> LHA/SAB
    sbr r20,1<<doTWI    ;weitere aktion
                        ;nach evtl.TXack
	sts	twiSTATUS,r20
	ldi	r16,90			;						   
	rjmp	_serxHOMEactiv		;					   
	;											   

;----------------------------------------------------------------------------------
;
;REQUEST DATA F. SUB-AREA-BUS (...dieser baustein) -> CORE (...dieser baustein)
;----------------------------------------------------------------------------------
_REQlokalNET:				;			        	1		|	1

	lds	r20,twiSTATUS
	sbr	r20,1<<TWIrequest
    cbr r20,1<<LHAreq   ;->CORE
    sbr r20,1<<doTWI    ;weitere aktion
                        ;nach evtl.TXack
	sts	twiSTATUS,r20
	ldi	r16,90			;						    
	rjmp	_serxHOMEactiv			;						
	;.....
	;....
	;...
	;..
	;.
	;


;----------------------------------------------------------------------------------
;
;DYNAMISCHES TWI-MODUL
;----------------------------------------------------------------------------------
;ab 90...COM

;----- twi - hardware - interface --------------------------------|
;-----------------------------------------------------------------|
;* keine interruptsteuerung -> twint polling					  |
;* integration nach multijobOS						  			  |
;-----------------------------------------------------------------|
;											                      |
;twar == twi adress register							          |
;											                      |
;	 _______________________________________________		      |
;	|twa6 |twa5 |twa4 |twa3 |twa2 |twa1 |twa0 |twgce|		      |
;	|_____|_____|_____|_____|_____|_____|_____|_____|		      |
;											                      |
;	twa0...twa6 = twi-slave-adress (7-bit)	          			  |
;	twgce		= twi-general-call-enabled (1==general call)	  |
;											                      |
;											                      |
;-----------------------------------------------------------------|
;-----------------------------------------------------------------|
;											                      |
;twdr == twi data register							              |
;											                      |
;	 _______________________________________________		      |
;	|twd7 |twd6 |twd5 |twd4 |twd3 |twd2 |twd1 |twd0 |		      |
;	|_____|_____|_____|_____|_____|_____|_____|_____|		      |
;											                      |
;	twa0...twa7 = twi-data (rx/tx)					              |
;											                      |
;											                      |
;											                      |
;-----------------------------------------------------------------|
;-----------------------------------------------------------------|
;											                      |
;twbr == twi bit-rate register						              |
;											                      |
;	 _______________________________________________		      |
;	|twbr7|twbr6|twbr5|twbr4|twbr3|twbr2|twbr1|twbr0|		      |
;	|_____|_____|_____|_____|_____|_____|_____|_____|		      |
;											                      |
;	twbr0...twbr7 = twi-bit-rate						          |
;											                      |
;	bsp.:  		   		cpu clock 					              |
;		Fscl	=   --------------------				          |
;						  		twps				              |
;			    16 + 2(twbr) * 4 					              |					
;											                      |
;						16000000hz						          |
;		Fscl	=   -------------------- 				  		  |
;										1(twps)				      |				                  
;			    		16 + 2(80)   * 4					      |
;											                      |
;		Fscl	=   24,39khz 						              |									
;											                      |
;-----------------------------------------------------------------|
;-----------------------------------------------------------------|
;											                      |
;twcr == twi control register							          |
;											                      |
;	 _______________________________________________		      |
;	|twint|twea |twsta|twsto|twwc |twen |---- |twie |		      |
;	|_____|_____|_____|_____|_____|_____|_____|_____|		      |
;											                      |
;	twint		= wird durch hardware nach erledigung gesetzt	  |
;	twea		= generiert acknowlege-bit				          |
;	twsta		= generiert start-bit					          |
;	twsto		= generiert stop-bit					          |
;	twwc		= kollision-bit durch hardware generiert		  |										
;	twen		= aktiviert das twi interface				      |
;	twie		= aktiviert twi-interrupt				          |
;											                      |
;											                      |
;-----------------------------------------------------------------|
;-----------------------------------------------------------------|
;											                      |
;twsr == twi status register							          |
;											                      |
;	 _______________________________________________		      |
;	|tws7 |tws6 |tws5 |tws4 |tws3 |---- |twps1|twps0|		      |
;	|_____|_____|_____|_____|_____|_____|_____|_____|		      |
;											                      |
;	tws3...twa7 = twi-status						              |
;	twps0/1	= twi clock prescaler bits				              |
;											                      |
;											                      |
;-----------------------------------------------------------------|
;											                      |
;											                      |
;defaults im testbetrieb:							              |
;											                      |
;- twsr = xxxxx-00	(prescaler = 1 )					          |
;- twbr = 01010000	(bit-rate  = 80)					          |
;											                      |
;-----------------------------------------------------------------|
;											                      |
;.equ	bitrate	=	80	;						                  |
;.equ	prescale	=	0	;						              |
;.equ	TWIstart	=	0x8	;twsr startmatrix				      |
;.equ	slaW		=	0x18	;	"					          |
;.equ	slaR		=	0x40	;	"					          |
;											                      |	
;bausteinadressen (range):							              |
;pcf8574:  	0x20-0x27 !!!	-> 0x20 - 0x23				          |
;pcd3311:	0x24-0x25 		-> 0x24 - 0x25				          |
;tda8444:	0x20-0x27 !!!	-> 0x26 - 0x27				          |
;max128:	0x28-0x2f		-> 0x28 - 0x2b				          |
;saa1064:	0x38-0x3b !!!	-> 0x38 - 0x3b				          |
;pcf8574A: 	0x38-0x3f		-> 0x3c - 0x3f				          |
;lm75:		0x48-0x4f !!!	-> 0x48 - 0x4b				          |						
;pcf8591:	0x48-0x4f !!!	-> 0x4c - 0x4f 				          |
;pcf8583:	0x50-0x51		-> 0x50 - 0x51                        |
;st24c08:   0x50-0x57       -> 0x52 - 0x57                        |
;led matrix 0x7c-0x7f       -> 0x7c - 0x7f                        |
;											                      |
;.equ	twiADR	=	0x133		;variable	slaveadresse	      |
;.equ	twiSIZ	=	0x135		;"		twi-datasize	          |
;.equ	twiSTATUS	=	0x13e	;"		twi-statusflag	     	  |
;flags										                      |
;.equ	control		=	0		;control-flag (twi-STATUS)	      |   
;.equ	TWIrequest	=	1		;request-flag (twi-STATUS)	      |
;.equ 	adrmod		=	3		;adressart (twi-STATUS)		      |
;-----------------------------------------------------------------|
;
;------------------------------------------------------------------
;folgende parameter an nachfolgende blcke bergeben:			   | 
;- slaveadresse (sla) inkl. w/r-bit 					           |
;- twi-framesize  (errechnet sich aus der local-area frame-size)   |
;- twi-control-byte (beinhaltet slaveabhngige adress-,controldaten|			
;------------------------------------------------------------------
_serx095_COMx:
	rjmp	_serx095_COM
;

_serx090_COM:
	cpi	r16,90
	brne	_serx095_COMx
	;
	;lade local-area frame size
	;und konvertiere nach sub-area
	;frame-size !!
	clr	zh	
	ldi	zl,5			;empfangene frame-size	
	;bei newTX daten aus txbuffer	
	sbrc	r22,newTX
	rjmp	_serx090_COMframer		


	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;von extern
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	rjmp	_serx090_COMgo
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx090_COMframer:
	;vom framer
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry




_serx090_COMgo:
	ld	r17,z
	subi	r17,11			;(local-area-framesize) - (header/adresses)11 = twiSIZ !!! 
	sts	tempTWIsiz,r17		;temporre twi-framesize
	;effektive twi-framesize
	;wird spter berechnet
	clr	r17
	sts	twiSIZ,r17		;speichere basis-twi-frame-size
	;
	;hole twiADRESSE 
	lds	r17,twiADR

;-----------------------------------------------------------------------
;funktionsmerkmale (twiCON) der jeweiligen bausteine festlegen 
;-----------------------------------------------------------------------
	;
	;
	;prfe nun welcher baustein 
	;adressiert werden soll !
	lds	r20,twiSTATUS		;twi-status-flags
	cpi	r17,_ADR_pcf8574
	brsh	valid8574dev		;<0x20 ist ungltig
	clr	r16
	rjmp	_serxHOME
	;
	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------
	;
valid8574dev:
	cpi	r17,_ADR_pcd3311
	brsh	valid3311dev		
    ;0x20 - 0x23 == pcf8574
	;-> strukturiere daten f.pcf8574
	rjmp	_serx090_COMPLUSnone
	;
	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------	
	;
valid3311dev:
	cpi	r17,_ADR_tda8444
	brsh	valid8444dev		
    ;0x24 - 0x25 == pcd3311
	;-> strukuriere daten f pcd3311
	rjmp	_serx090_COMPLUSnone
	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------
	;
valid8444dev:
	cpi	r17,_ADR_max128			
	brsh	valid128dev
	;0x26 - 0x27 == tda8444
	;-> strukturiere daten f. tda8444 (ONLY WRITE)
	ldi	r21,0x0			;controlbyte... auto-increment 
	ldi	zl,low(TWIbuffer)
	ldi	zh,high(TWIbuffer)
	st	z+,r21 ;store controlbyte
	sts	tempTWIbufL,zl ;twibuffer pointer l
	sts	tempTWIbufH	,zh ;                  h
	;twisize +1
	lds	r21,twiSIZ
	inc	r21
	sts	twiSIZ,r21
	rjmp	_serx090_COMadressPRE
	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------
	;
valid128dev:
	cpi	r17,0x2c			
	brsh	validRESdevA
    ;0x28 - 0x2b == max128
	;-> strukturiere daten f. max128 (ad-channelwahl etc. ber erstes LSB-source-adress)
	rjmp	PLUSadress


	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------
									validRESdevA:
										cpi	r17,_ADR_saa1064			
										brsh	valid1064dev
	;
	;--------------------------------------------------------------------
	;->>>  strukturiere daten f. knftige devices
	;--------------------------------------------------------------------
	;
	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------
	;
	;
valid1064dev:
	cpi	r17,_ADR_pcf8574A			
	brsh	valid8574Adev
	;0x38 - 0x3b == saa1064
	;-> strukturiere daten f. saa1064 (ONLY WRITE)
	;nur schreiben
	;1)instruction
	clr		r21										;instructionbyte -> beginne mit controlbyte, dann weiter 
													;mit den 4 datenbytes fr's display
	;2)control
	ldi		r20,0b01010111							;controlbyte...dynamic-mode/15mA display
													;current/digit 1/2/3/4 not blanked 
	;3)nach twibuffer
	ldi		zl,low(TWIbuffer)
	ldi		zh,high(TWIbuffer)
	st		z+,r21 	;store instruction
	st		z+,r20	;store control
	sts		tempTWIbufL,zl ;twibuffer pointer l
	sts		tempTWIbufH	,zh ;                  h
	;twisize +2
	lds		r21,twiSIZ
	inc		r21
	inc		r21
	sts		twiSIZ,r21
	jmp		_serx090_COMadressPRE
	;
	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------
	;
valid8574Adev:
	cpi	r17,0x40			;0x3c - 0x3f == pcf8574A
	brsh	validRESdevB
	;
	;
	;-> strukturiere daten f. pcf8574A
	jmp	_serx090_COMPLUSnone
	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------
													;
													;
												validRESdevB:
													cpi	r17,_ADR_lm75			;
													brsh	valid75dev
													;
	;--------------------------------------------------------------------
	;->>> strukturiere f. knftige devices
	;--------------------------------------------------------------------
	;
	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------
	;
valid75dev:
	cpi	r17,_ADR_pcf8591			
	brsh	valid8591dev
	;0x48 - 0x4b == lm75
	;-> strukturiere f. lm75 (WRITE POINTER & READ TEMPERATURE)
	ldi	r21,0x0				;controlbyte... read temperatur-register (2 byte msb/lsb!)
	jmp		PLUSadressREQb
	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------
	;
valid8591dev:
	cpi	r17,_ADR_pcf8583			
	brsh	valid8583dev
	;0x4c - 0x4f == pcf8591
	;-> strukturiere daten f. pcf8591
	;A) nur schreiben     <dev-adrW><control><data1><data2>.... -> set controlbyte & transfer D/A-byte
	;B) read a/d autoinc  <dev-adrR><data0><data1>...       	-> read a/d-channels autoincrement
	sbrc	r20,TWIrequest
	rjmp	valid8591devREQ			;requestanforderung !
	;nur schreiben
	;1)controldata
	ldi		r21,0b01000100		;controlbyte...d/a aktiv / autoinkrement
	;2)nach twibuffer
	ldi		zl,low(TWIbuffer)
	ldi		zh,high(TWIbuffer)
	st		z+,r21 	;store controladress
	sts		tempTWIbufL,zl ;twibuffer pointer l
	sts		tempTWIbufH	,zh ;                  h
	;twisize +1
	lds		r21,twiSIZ
	inc		r21
	sts		twiSIZ,r21
	jmp		_serx090_COMadressPRE
	;
	;
	;
	;lese sofort daten == kein repstart
valid8591devREQ:
	;twisize berechnen
	;...................................................................................
	;berechne sptere <read> reqTWIsiz == nur daten
	;aus <tempTWIsiz> == alle data-adresss & daten im RXbuffer
	lds	r21,tempTWIsiz	;z.Zt. RXbuffer-framesize OHNE header/cksum/first s/t-adresses
	subi	r21,1		;-1tes datenbyte
	ldi	yl,1			;help twi-datacounter =1
	;...................................................................................
	lds	r20,twiSTATUS		;twi-status-flags
	
	;bei newTX daten aus txbuffer	
	sbrc	r22,newTX
	rjmp	_valid8591devREQframer		


	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;von extern
	ldi	zl, low(RXbufferST)	;Add low byte ( x -(-y)) = x + y
	ldi	zh, high(RXbufferST)	;Add high byte with carry
	rjmp	_valid8591devREQgo	
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_valid8591devREQframer:
	;vom framer
	ldi	zl, low(TXbufferST)	;Add low byte ( x -(-y)) = x + y
	ldi	zh, high(TXbufferST)	;Add high byte with carry

		
_valid8591devREQgo:
	ld	r17,z
	sbrc	r17,adr_mod	;....1=sequ / 0=gezielt
	rjmp	valid8591devSEQU
	;gezielt/direct...
valid8591devDIR:
	cbr		r20,1<<adrmod
valid8591devDIRx:
	tst		r21
	breq	valid8591devGO
	subi	r21,5	;framesize - 5 (2*16bit adress / 1*8bit data)
	inc		yl		;twisize+1
	jmp		valid8591devDIRx
	;
	;sequenz
valid8591devSEQU:
	sbr		r20,1<<adrmod
valid8591devSEQUx:
	tst		r21
	breq	valid8591devGO
	subi	r21,1	;framesize -1 (1*8bit data)
	inc		yl		;twisize+1
	jmp		valid8591devSEQUx
	;
	;.....................................................................
valid8591devGO:
	;kein repeated start erforderlich!
	cbr	r20,1<<repSTART	 
	sts	twiSTATUS,r20
	;store TWIsiz
	sts		TWIsiz,yl
	;hole twiADRESSE
	lds	r17,twiADR
	;7bit adresse !!!!
	lsl	r17
	;lesen!
	sbr	r17,1<<0		;+lesen !!!
	sts	twiADR,r17		;speichere twi-adresse
	;zur twi-bertragung....
	ldi	r16,97
	rjmp	_serxHOMEactiv
	;
	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------
	;
	;
valid8583dev:
	cpi	r17,_ADR_st24c08			
	brsh	valid24c08dev
	;0x50 - 0x51 == pcf8583
	;-> strukturiere daten f. pcf8583
	;A) nur schreiben     <dev-adrW><control-adress0><control-data(adr+0)>|<data(adr+1)>.... -> set clock
	;B) read ab adresseX  <dev-adrW><adressX><dev-adrR><data(adrX+0)><data(adrX+1)>...       -> read clock ab adrX
	;
	sbrc	r20,TWIrequest
	rjmp	valid8583devREQ			;requestanforderung !
	;nur schreiben
	;1)control adresse
	clr		r21			;ab 0x0
	;2)controldata
	clr		r20 		;controldata... clock-mode 32,768KHz
	;3)nach twibuffer
	ldi		zl,low(TWIbuffer)
	ldi		zh,high(TWIbuffer)
	st		z+,r21 	;store controladress
	st		z+,r20	;store controldata
	sts		tempTWIbufL,zl ;twibuffer pointer l
	sts		tempTWIbufH	,zh ;                  h
	;twisize +2
	lds		r21,twiSIZ
	inc		r21
	inc		r21
	sts		twiSIZ,r21
	jmp		_serx090_COMadressPRE
	;
	;schreibe start-adresse(1) & lese anschlieend daten ab adresse(1)
valid8583devREQ:
	;1)twi read adresse
	ldi		r21,1			;ab 0x1
	jmp		PLUSadressREQb
	;
	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------
	;
	;
valid24c08dev:
	cpi	r17,0x58			
	brsh	validLEDmat01dev
	;0x52 - 0x57 == st24c08
	;-> strukturiere daten f. st24c08
	;flexible adressierung
	rjmp	PLUSadress
	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------
	;
validLEDmat01dev:
	cpi	r17,_ADR_ledMAT 			;0x7c - 0x7f == led matrix modul
	brsh	validLEDmat01devA
    ;<0x7c ==nicht definiert !
	rjmp	invalidRESdevC		
	;
validLEDmat01devA:
	;nur schreiben mglich !
	;nach twibuffer
	ldi		zl,low(TWIbuffer)
	ldi		zh,high(TWIbuffer)
	;+1funktion
	ldi		r20,0b10111010    ;fullCOPY(.0)- lineCOPY(.1)- singleCOPY(.2)- toggleFU(.3)
	st		z+,r20		;dimmFU(.4)- blendFU(.5)- changeSZEN(.6)- sequFU(.7)

	;+2loader
	ldi		r20,0b00001010	;equ	EDITrun(.1)  =gibt funktionen zur ausfhrung frei
	st		z+,r20		;LOADe2p(.2)  =bertrgt e2prom table ins rgb-ram & anschlieende ausfhrung
						;SEQUslot(.3) =aktiviert ablauftimer


	sts		tempTWIbufL,zl ;twibuffer pointer l
	sts		tempTWIbufH,zh ;                  h
	;twisize +2
	lds		r21,twiSIZ
	ldi		r20,2
	add		r21,r20
	sts		twiSIZ,r21
	jmp		_serx090_COMadressPRE
	
	
	;
	;--------------------------------------------------------------------
	;********************************************************************
	;--------------------------------------------------------------------
	
		






	;not valid !!!
invalidRESdevC:
	clr	r16
	rjmp	_serxHOME
	;
	;
	;

									;
									;----------------------------------------------------------
									;KEIN control-,od.instructionbyte beim schreiben oder lesen
									;----------------------------------------------------------
								_serx090_COMPLUSnone:
									;
									ldi	zl,low(TWIbuffer)
									ldi	zh,high(TWIbuffer)
									sts	tempTWIbufL,zl ;twibuffer pointer l
									sts	tempTWIbufH	,zh ;                  h
									;A) nur schreiben     <dev-adrW><data1>....
									;B) read ab adresseX  <dev-adrR><data1>...  
									;
									lds	r20,twiSTATUS		;twi-status-flags
									sbrc	r20,TWIrequest
									rjmp	_serx090_COMPLUSREQ				;requestanforderung !
									;schreiben
									jmp		_serx090_COMadressPRE					
									;
									;
									;sofortiges lesen ohne repstart
								_serx090_COMPLUSREQ:
									;kein repeated start erforderlich!
									cbr	r20,1<<repSTART	 
									;twisize berechnen
									;...................................................................................
									;berechne sptere <read> reqTWIsiz == nur daten
									;aus <tempTWIsiz> == alle data-adresss & daten im RXbuffer
									lds	r21,tempTWIsiz	;z.Zt. RXbuffer-framesize OHNE header/cksum/first s/t-adresses
									subi	r21,1		;-1tes datenbyte
									ldi	yl,1			;help twi-datacounter =1
									;...................................................................................
									;bei newTX daten aus txbuffer	
									sbrc	r22,newTX
									rjmp	_serx090_COMPLUSframer		
									;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
									;von extern
									ldi	zl, low(RXbufferST)	;Add low byte ( x -(-y)) = x + y
									ldi	zh, high(RXbufferST)	;Add high byte with carry
									rjmp	_serx090_COMPLUSgoon
									;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
								_serx090_COMPLUSframer:
									;vom framer
									ldi	zl, low(TXbufferST)	;Add low byte ( x -(-y)) = x + y
									ldi	zh, high(TXbufferST)	;Add high byte with carry

								_serx090_COMPLUSgoon:
									ld	r17,z
									sbrc	r17,adr_mod	;....1=sequ / 0=gezielt
									rjmp	_serx090_COMPLUSSEQU
									;gezielt/direct...
								_serx090_COMPLUSDIR:
									cbr	r20,1<<adrmod
									sts	twiSTATUS,r20
								_serx090_COMPLUSDIRx:
									tst		r21
									breq	_serx090_COMPLUSGO
									subi	r21,5	;framesize - 5 (2*16bit adress / 1*8bit data)
									inc		yl		;twisize+1
									jmp		_serx090_COMPLUSDIRx
									;
									;sequenz
								_serx090_COMPLUSSEQU:
									sbr	r20,1<<adrmod
									sts	twiSTATUS,r20
								_serx090_COMPLUSSEQUx:
									tst		r21
									breq	_serx090_COMPLUSGO
									subi	r21,1	;framesize -1 (1*8bit data)
									inc		yl		;twisize+1
									jmp		_serx090_COMPLUSSEQUx
									;
									;.....................................................................
								_serx090_COMPLUSGO:
									;store TWIsiz
									sts		TWIsiz,yl
									;hole twiADRESSE
									lds	r17,twiADR
									;7bit adresse !!!!
									lsl	r17
									;lesen!
									sbr	r17,1<<0		;+lesen !!!
									sts	twiADR,r17		;speichere twi-adresse
									;zur twi-bertragung....
									ldi	r16,97
									rjmp	_serxHOMEactiv




									;
									;-----------------------------------------------------------------------
									;NUR SCHREIBEN
									;vorbereitung DATEN schreiben (mit control/instruction/adressbytes)
								_serx090_COMadressPRE:
									;status
									lds	r20,twiSTATUS		;twi-status-flags
									cbr	r20,1<<repSTART	;nur schreiben also kein repeated start erforderlich!
									sts	twiSTATUS,r20
									;
									;hole twiADRESSE
									lds	r17,twiADR
									;7bit adresse !!!!
									lsl	r17
									;schreiben!
									cbr	r17,1<<0		;+write !!!
									sts	twiADR,r17		;speichere twi-adresse
									ldi	r16,95
									rjmp	_serxHOMEactiv
									;-------------------------------------------------------------------------









	




	;
	;
	;
	;
	;
	;
	;------------------------------------------------------------------------------------------------------------------
	;+adressbyte (variabel) z.b speicheradressierung in twi-devices via source/target adressierung (1te 16bit adressen)
	;------------------------------------------------------------------------------------------------------------------
	;INFO: Hier noch prfen ob
	;      fr twi request adressierung ok????
	;GRUNDLAGE:
	;- bei request-anforderung (read): S<device-adr>W/A<byte-adr>A/S<device-adr>R/A<data>..
	;- non request-anforderung (write):S<device-adr>W/A<byte-adr>A<data>...
	;
	;- bei request-anforderung (read) wird LSB-source angenommen
	;- bei non-request-anforderung (write) wird LSB-target angenommen
	;
	;-----------------------------------------------------------------------------------
PLUSadress:
	;lade datenadresse 8-bit ist maximum f. twi !
	;A)prfe ob request anforderung
	lds		r20,twiSTATUS		;twi-status-flags
	sbrc	r20,TWIrequest
	rjmp	PLUSadressREQ			;requestanforderung !
PLUSadressNOREQ:	
	clr	zh	
	ldi	zl,10
	
	;bei newTX daten aus txbuffer	
	sbrc	r22,newTX
	rjmp	PLUSadressNOREQa		
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;von extern
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	rjmp	PLUSadressNOREQgo
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
PLUSadressNOREQa:
	;vom framer
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry


PLUSadressNOREQgo:	
	;empfangenen LSB target-adresspointer		
	ld	r21,z
	;
	ldi	zl,low(TWIbuffer)
	ldi	zh,high(TWIbuffer)
	st	z+,r21 ;store adressbyte
	sts	tempTWIbufL,zl ;twibuffer pointer l
	sts	tempTWIbufH	,zh ;                  h
	;twisize +1
	lds	r21,twiSIZ
	inc	r21
	sts	twiSIZ,r21
	;non request == nur schreiben
	rjmp	_serx090_COMadressPRE
	;
PLUSadressREQ:
	;request == <start><device-adr><write><ack>&<byte-adr><ack> schreiben
	;dann <rep.start><device-adr><read><ack><data>... lesen !
	;
	;-> rep.start flag gesetzt (bausteinabhngig) ?
	;   ... zuerst control/instruction oder adresse schreiben !
	clr	zh	
	ldi	zl,8	
	
	
	;bei newTX daten aus txbuffer	
	sbrc	r22,newTX
	rjmp	PLUSadressREQa		
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;von extern
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	rjmp	PLUSadressREQgo
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
PLUSadressREQa:
	;vom framer
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry

PLUSadressREQgo:	
	;empfangenen LSB source-adresspointer		
	ld	r21,z
	;
	;entry bei fixer adresse !
PLUSadressREQb:
	ldi	zl,low(TWIbuffer)
	ldi	zh,high(TWIbuffer)
	st	z+,r21 ;store adressbyte
	;twisize +1
	lds	r21,twiSIZ
	inc	r21
	sts	twiSIZ,r21
	;...................................................................................
	;berechne sptere <read> reqTWIsiz == nur daten
	;aus <tempTWIsiz> == alle data-adresss & daten im RXbuffer
	lds	r21,tempTWIsiz	;z.Zt. RXbuffer-framesize OHNE header/cksum/first s/t-adresses
	subi	r21,1		;-1tes datenbyte
	ldi	yl,1			;help twi-datacounter =1
	;...................................................................................

	;bei newTX daten aus txbuffer	
	sbrc	r22,newTX
	rjmp	PLUSadressPREQa		
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;von extern
	ldi	zl, low(RXbufferST)	;Add low byte ( x -(-y)) = x + y
	ldi	zh, high(RXbufferST)	;Add high byte with carry
	rjmp	PLUSadressPREQgo
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
PLUSadressPREQa:
	;vom framer
	ldi	zl, low(TXbufferST)	;Add low byte ( x -(-y)) = x + y
	ldi	zh, high(TXbufferST)	;Add high byte with carry

PLUSadressPREQgo:	
	;adressierungsart ??
	ld	r17,z
	sbrc	r17,adr_mod	;....1=sequ / 0=gezielt
	rjmp	PLUSadressSEQU
	;gezielt/direct...
PLUSadressDIR:
	cbr	r20,1<<adrmod
PLUSadressDIRx:
	tst		r21
	breq	PLUSadressGO
	subi	r21,5	;framesize - 5 (2*16bit adress / 1*8bit data)
	inc		yl		;twisize+1
	jmp		PLUSadressDIRx
	;
	;sequenz
PLUSadressSEQU:
	sbr		r20,1<<adrmod
PLUSadressSEQUx:
	tst		r21
	breq	PLUSadressGO
	subi	r21,1	;framesize -1 (1*8bit data)
	inc		yl		;twisize+1
	jmp		PLUSadressSEQUx
	;
	;.....................................................................
PLUSadressGO:
	;store reqTWIsiz
	sts		reqTWIsiz,yl
	;bereite repeated start vor
	sbr	r20,1<<repSTART	;fr anschl. lesen repeated start erforderlich!
	sts	twiSTATUS,r20
	;hole twiADRESSE
	lds	r17,twiADR
	;7bit adresse !!!!
	lsl	r17
	;schreiben!
	cbr	r17,1<<0		;+write !!!
	sts	twiADR,r17		;speichere twi-adresse
	;zur twi-bertragung....
	ldi	r16,97
	rjmp	_serxHOMEactiv















								;-----------------------------------------------------------------
								;Daten aus TX/RXbuffer nach TWIbuffer (TWI-Standard) konvertieren |
								;-> SEUQUENZIELLE u. GEZIELTE Adressierung !				      |
								;-> Kovertierter Frame ab RXbuffer (sram) 			              |
								;tempTWIsiz	=tempTWI-size						                  |
								;tempTWIbufL=tempTWIbufL					                      |
								;tempTWIbufH=temp TWIbufH				                          |
								;twibuffer-inhalt:					                              |
								;(<control><instr.>)<data0><data1><...>			                  |
								;-----------------------------------------------------------------
								_serx095_COM:
									cpi	r16,95
									brne	_serx096_COM									
									;bei newTX daten aus txbuffer	
									sbrc	r22,newTX
									rjmp	_serx095_COMa	
									;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
									;von extern
									ldi	zl, low(RXbufferST)	;Add low byte ( x -(-y)) = x + y
									ldi	zh, high(RXbufferST)	;Add high byte with carry
									rjmp	_serx095_COMgo
									;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
								_serx095_COMa:
									;vom framer
									ldi	zl, low(TXbufferST)	;Add low byte ( x -(-y)) = x + y
									ldi	zh, high(TXbufferST)	;Add high byte with carry

								_serx095_COMgo:										
									;prfe adressierungbit im Mode-ID (RXbuffer)
									;empfangene MI		
									ld	r17,z
									sbrc	r17,adr_mod	;....1=sequ / 0=gezielt
									rjmp	_serx095sequ
									;
									;...gezielt == jedes 5.te byte verwenden !!!
								_serx095gez:
									lds	r17,twiSTATUS
									cbr	r17,1<<adrmod
									sts	twiSTATUS,r17
									rjmp	_serx095prep
									;...sequenziell === jedes byte verwenden
								_serx095sequ:
									lds	r17,twiSTATUS
									sbr	r17,1<<adrmod
									sts	twiSTATUS,r17
									rjmp	_serx095prep
									;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
								_serx095prep:
									;...adresspointer auf 1tes datenbyte
									clr	zh	
									ldi	zl,11				;adresspointer auf daten

									;bei newTX daten aus txbuffer	
									sbrc	r22,newTX
									rjmp	_serx095_COMprepa	
									;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
									;von extern
									subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
									sbci	zh, high(-RXbufferST)	;Add high byte with carry
									rjmp	_serx095_COMprepgo
									;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
								_serx095_COMprepa:
									;vom framer
									subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
									sbci	zh, high(-TXbufferST)	;Add high byte with carry


								_serx095_COMprepgo:	
									sts	tempRXbufL,zl
									sts	tempRXbufH,zh
									ldi	r16,96
									jmp	_serxHOMEactiv
									;
									;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
									;gezielt store-loop (verwende jedes 5.te byte) hole daten
								_serx096_COM:
									cpi	r16,96
									brne	_serx097_COM
									lds	yl,tempTWIbufL	;twi buffer adress pointer
									lds	yh,tempTWIbufH	
									lds	zl,tempRXbufL	;rx buffer  adress pointer
									lds	zh,tempRXbufH
									lds	r21,tempTWIsiz	;temporre twi datensize OHNE twi-adress,control-u.instr.byte
									lds	r20,twiSTATUS
									;
									ld	r17,z
									;
													;------------------------
													;einige devices arbeiten 
													;nicht mit 8-bit werten
													;wie z.b. tda8444...
													;hier muss von 8-bit nach
													;6-bit konvertiert werden
													;-> devices werden ber deren
													;adresse identifiziert !
													;------------------------
													;0x26 - 0x27 == tda8444
													;-> 0x4c - 0x4e
													lds	r18,twiADR	;berechnete twi-adresse laden
													cpi	r18,0x4c
													brsh	_serx096_COM8444inrange
													rjmp	_serx096_COM8444outrange	;lower !
												_serx096_COM8444inrange:
													cpi	r18,0x4f
													brlo	_serx096_COM8444go
													rjmp	_serx096_COM8444outrange	;higher !
													;
													;tda8444 in range // para:4
												_serx096_COM8444go:
													lsr	r17
													lsr	r17
													;-------------------------

								_serx096_COM8444outrange:
									st	y+,r17
									sbrs	r20,adrmod	;1== sequ(jedes folgende byte verw.)  0== gezielt (jedes 5.te byte verw.)
									rjmp	_serx096_gez
									;sequ.
									adiw	zh:zl,1	;rxbuffer+1
									subi	r21,1		;temp. framesize-1
									jmp	_serx096_STORE
									;gez.

									;+++ TEST
								_serx096_gez:
									;tempsize? 
									cpi	r21,6
									brsh	_serx096_gezGO
									clr	r21
									jmp	_serx096_STORE


									;TEST ENDE ++++
								_serx096_gezGO:
									adiw	zh:zl,5	;rxbuffer+5 (nchstes datenbyte)
									subi	r21,5		;temp. framesize-5
									jmp	_serx096_STORE
									;
								_serx096_STORE:
									sts	tempTWIbufL,yl
									sts	tempTWIbufH	,yh
									sts	tempRXbufL,zl
									sts	tempRXbufH,zh
									sts	tempTWIsiz,r21	;store temporary twi-size
									;
									;effektive(netto) twi-framesize +1 (inkl.instruction byte etc.)
									lds	zl,twiSIZ
									inc	zl
									sts	twiSIZ,zl
									;
									tst	r21
									breq	_serx096end
									rjmp	_serxHOMEactiv
								_serx096end:
									lds	r21,twiSIZ	;framesize (ohne device-adressbyte & incl. Instructionbyte etc.)
									sts	tempTWIsiz,r21	;nach temp. kopieren
									ldi	r16,100		;TEST 97
									rjmp	_serxHOMEactiv

								;--------------------------------------------------------------------------------------













;--------------------------------------------------------------------------------------
;nun datenbertragung nach sub-area-bus beginnen....
;--------------------------------------------------------------------------------------
;

;------------------------
;stop condition
;------------------------
_serx097_COM:
	cpi	r16,97
	brne	_serx100_COM

	;timeoutcounter default
	ldi		r17,i2cout
	sts		I2Ctimeout,r17
	ldi	r20,(1<<twint) | (1<<twen) | (1<<twsto)
	sts	twcr,r20
	ldi	r16,100
	rjmp	_serxHOMEactiv


;------------------------
;start condition
;------------------------
_serx100_COM:
	cpi	r16,100
	brne	_serx101_COM
	;timeoutcounter default
	ldi		r17,i2cout
	sts		I2Ctimeout,r17
	ldi	r20,(1<<twint) | (1<<twen) | (1<<twsta)
	sts	twcr,r20
	ldi	r16,101
	;rjmp	_serxHOMEactiv
	
;------------------------
;check twint
;------------------------
_serx101_COM:
	cpi	r16,101
	brne	_serx102_COM
	lds 	r20,twcr
	sbrs	r20,twint
	rjmp	_serxi2Ctimeout		;hier knftig auf timeout-routine verzweigen - start prozedur noch nicht abgeschlossen
	ldi	r16,102		;....................abgeschlossen
	;rjmp	_serxHOMEactiv

;------------------------
;check status matrix
;------------------------
_serx102_COM:
	cpi	r16,102
	brne	_serx103_COM
	lds	r20,twsr
	andi	r20,0xf8	;maskiere wg. prescaler (... darf nicht beeinflusst werden)
	cpi	r20,TWIstart	;HIER entsprechend der ablaufsteuerungsmatrix prfen
	brne	_serx102rep	;passt nicht == rep.start ODER fehler !
	ldi	r16,103
	jmp	_serx103_COM	;_serxHOMEactiv
_serx102rep:
	cpi	r20,TWIrep		;repeated start detected ?
	brne	_serx102ERR
	ldi	r16,103
	jmp	_serxHOMEactiv
_serx102ERR:
	;TEST
	jmp		_serxI2Cstop	;ready !
	;TEST END




;------------------------
;tx sla(w/r)
;------------------------
_serx103_COM:
	cpi	r16,103
	brne	_serx104_COM
	;timeoutcounter default
	ldi		r17,i2cout
	sts		I2Ctimeout,r17
	lds	r20,twiADR	;bertrage slaveadresse (7-bit) + w/r bit 
	sts	twdr,r20
	ldi	r20,(1<<twint) | (1<<twen)
	sts	twcr,r20
	ldi	r16,104
;	rjmp	_serxHOMEactiv

;------------------------
;check twint
;------------------------
_serx104_COM:
	cpi	r16,104
	brne	_serx105_COM
	lds 	r20,twcr
	sbrs	r20,twint
	rjmp	_serxi2Ctimeout		;hier knftig auf timeout-routine verzweigen - tx adresse- prozedur noch nicht abgeschlossen
	ldi	r16,105		;....................abgeschlossen
	;rjmp	_serxHOMEactiv




;------------------------
;check status matrix
;------------------------
_serx105_COM:
	cpi	r16,105
	brne	_serx106_COM
	lds	r20,twsr
	andi	r20,0xf8	;maskiere wg. prescaler (... darf nicht beeinflusst werden)
	cpi	r20,slaW	;HIER entsprechend der ablaufsteuerungsmatrix prfen
	brne	_serx105b_COM	
	ldi	r16,126		;...masterTX mode
	rjmp	_serx126_COM	;TEST_serxHOMEactiv
_serx105b_COM:
	cpi	r20,slaR
	brne	_serx105ERR	;passt nicht == fehler !
	ldi	r16,106	;...masterRX mode
	rjmp	_serxHOMEactiv 
_serx105ERR:
	ldi	r16,150
	jmp	_serx150_COM


	;kein passender statuswert....
	clr	r16				;beende die prozedur	
	;----------------------
	;TEST clear rwi status
	sts	twiSTATUS,r16
	;----------------------
	jmp		_serxI2Cstop	;ready !
	;TEST END





;------------------------
;check timeout
;------------------------
_serxi2Ctimeout:
	lds	r17,I2Ctimeout
	tst	r17	
	breq	_serxI2CstopX
	dec	r17
	sts	I2Ctimeout,r17
	rjmp	_serxHOMEaction
_serxI2CstopX:
	rjmp	_serxI2Cstop





;------------------------
;I2Cstop instruction
;nur bus release OHNE stop
;------------------------
_serxI2Cstop:
	ldi	r21,(1<<twint)	;TEST|(1<<twen)|(1<<twsto)
	sts	twcr,r21
	;kein passender statuswert....
	clr	r16				;beende die prozedur	
	;----------------------
	;TEST clear twi status
	sts	twiSTATUS,r16
	;----------------------
	;TESTWEISE newTX=0
	cbr	r22,1<<newTX
	;----------------------
	rjmp	_serxHOME











;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;V NEW
;-----------------------------
;ab _serx106 masterRX		 |
;... lese daten aus slave	 |
;-----------------------------
_serx106_COM:
	cpi	r16,106
	brne	_serx107_COM	;
	;entry twsr=0x40
	;- sla+R transmitted
	;  + ACK from slave received
	;
	lds	r21,twiSIZ	;twisize inkl. instr.bytes (werden im rx-mode nicht verwendet
	sts	tempTWIsiz,r21	;temp.twisize
	;startadresse twibuffer laden
	ldi	zl,low(TWIbuffer)
	ldi	zh,high(TWIbuffer)
	sts	tempTWIbufL,zl ;twibuffer pointer l
	sts	tempTWIbufH,zh ;  
	ldi	r16,107         
	



	       
	;start twi-rx loop....
	;-> empfange daten
_serx107_COM:
	cpi	r16,107
	brne	_serx108_COM
	;prfe size..
	lds	r21,tempTWIsiz
	cpi	r21,2
	brsh	_serx107_COMb	;>=2
	;<2 nach datenempfang ein nack !
	ldi	r21,(1<<twint) | (1<<twen)
	rjmp	_serx107_COMgo
_serx107_COMb:
	ldi	r21,(1<<twint) | (1<<twen)| (1<<twea)
_serx107_COMgo:
	sts	twcr,r21
	ldi	r16,108
	;timeoutcounter default
	ldi		r17,i2cout
	sts		I2Ctimeout,r17
	rjmp	_serxHOMEactiv




	;->twint checken
_serx108_COM:
	cpi	r16,108
	brne	_serx109_COM
	lds 	r20,twcr
	sbrs	r20,twint
	rjmp	_serxi2Ctimeout			;hier knftig auf timeout-routine verzweigen - prozedur noch nicht abgeschlossen
	ldi	r16,109		;....................abgeschlossen
	rjmp	_serxHOMEactiv


	;-> daten holen 
_serx109_COM:
	cpi	r16,109
	brne	_serx110_COM
	lds	zl,tempTWIbufL	;twibuffer adress pointer l
	lds	zh,tempTWIbufH		;				  h
	lds	r20,twdr	;daten von slve rx
	st	z+,r20	;daten nach twibuffer
	sts	tempTWIbufL,zl	;aktuellen twi-adresspointer sichern
	sts	tempTWIbufH	,zh

	;& statusmatrix checken
	lds	r20,twsr
	andi	r20,0xf8	;maskiere wg. prescaler (... darf nicht beeinflusst werden)
	cpi	r20,RECack	;HIER entsprechend der ablaufsteuerungsmatrix prfen
	breq	twiRXstatusOK ;master erhielt ack !
	ldi	r16,111	;...master hat nack erhalten -> nun stop !
	rjmp	_serxHOMEactiv
	;ack
twiRXstatusOK:
	ldi	r16,110
	rjmp	_serxHOMEactiv





	;-> twisize checken
_serx110_COM:
 	cpi	r16,110
	brne	_serx111_COM
	lds	r21,tempTWIsiz	;temp twisize laden
	dec	r21
	tst	r21
	brne	_serx110twiLOOP
	ldi	r16,111	;ende erreicht - nun stop !
	rjmp	_serxHOMEactiv
_serx110twiLOOP:
	sts	tempTWIsiz,r21
	ldi	r16,107
	rjmp	_serxHOMEactiv
	;-> stop
_serx111_COM:
	cpi	r16,111
	brne	_serx112_COM
	ldi	r21,(1<<twint)|(1<<twen)|(1<<twsto)
	sts	twcr,r21
					;
					;******************************************************
					;TEST -> einige bausteindaten mssen konvertiert werden
					;        abhngig der deviceart
					;******************************************************

					;0x48 - 0x4b == lm75
					;adressen liegen hier *2 multipliziert im read mode vor !
					;=>0x90(+1) - 0x96(+1)
					lds	r21,TWIadr
					cpi	r21,0x91
					brsh	_serx111_COMcalcLM75a
					rjmp	_serx111_COMready	;lower !
				_serx111_COMcalcLM75a:
					cpi	r21,0x98
					brlo	_serx111_COMcalcLM75b
					rjmp	_serx111_COMready	;higher !
					;-------------------------------------
					;lm75 temp. sensoric device
					;in range !
				_serx111_COMcalcLM75b:
					;lese msb/lsb
					;startadresse twibuffer laden
					ldi	zl,low(TWIbuffer)
					ldi	zh,high(TWIbuffer)
					ld	yh,z+
					sts	tempTWIbufH,yh ;msb
					ld	yl,z
					sts	tempTWIbufL,yl ;  
					;msb=yh
					;lsb=yl
					;a) 0,negativ oder positiv ?
					sbrs	yh,7	;0=positiv
					rjmp	_serx111_COMcalcLM75positiv



					;---------------------------------------------
					;b)negativ
					sbrs	yl,7
					rjmp	_serx111_COMcalcLM75negno
					clc
					rjmp	_serx111_COMcalcLM75negativ
				_serx111_COMcalcLM75negno:
					sec
				_serx111_COMcalcLM75negativ:
					;shift left with carry to bit 0
					rol	yh
					;255-yh
					ldi	zl,0xff
					sub	zl,yh
					;div2
					lsr	zl
					;*10
					;msb*10
					ldi	zh,10
					mul	zh,zl
					movw	zh:zl,r1:r0
					;bit7 lsb =1 (-0,5 Grad)
					;         =0 (-0,0 Grad)
					sbrs	yl,7
					rjmp	_serx111_COMcalcLM75min5neg
					rjmp	_serx111_COMcalcLM75calc2500neg
					;+5
				_serx111_COMcalcLM75min5neg:
					adiw	zh:zl,5
					;	subi	zl,low(5)
					;	sbci	zh,high(5)
				_serx111_COMcalcLM75calc2500neg:
					ldi	yl,low(2500)
					ldi	yh,high(2500)
					sub	yl,zl
					sbc	yh,zh
					movw	zh:zl,yh:yl
					rjmp	_serx111_COMcalcLM75store
					;-----------------------------------------------


					;---------------------------------------------
					;c)positiv
				_serx111_COMcalcLM75positiv:
					;msb*10
					ldi	zl,10
					mul	yh,zl
					movw	zh:zl,r1:r0
					;bit7 lsb =1 (+0,5 Grad)
					;         =0 (+0,0 Grad)
					sbrc	yl,7
					rjmp	_serx111_COMcalcLM75plus5pos
					rjmp	_serx111_COMcalcLM75calc2500plus	
					;+5
				_serx111_COMcalcLM75plus5pos:
					adiw	zh:zl,5
				_serx111_COMcalcLM75calc2500plus:
					subi	zl,low(-2500)
					sbci	zh,high(-2500)
					rjmp	_serx111_COMcalcLM75store
					;--------------------------------------------
				_serx111_COMcalcLM75store:
					;lese msb/lsb
					;startadresse twibuffer laden
					ldi	yl,low(TWIbuffer)
					ldi	yh,high(TWIbuffer)
					st	y+,zh
					st	y,zl
					rjmp	_serx111_COMready











_serx111_COMready:
	ldi	r16,112
	jmp	_serxHOMEactiv



    ;=============================================================================================
    ;lesen aus twi fertig...
    ;A) Dann TWI-Buffer nach RX oder TX-Buffer
    ;B) Gem request-flags prfen ob Daten zum CORE oder LHA/SAB gehen sollen
    ;   -> zum CORE:   daten werden wie ein externes RXframe verarbeitet
    ;   -> zum LHA/SAB:daten werden im REQUEST-Bereich aufbereitst und verschickt
    ;=============================================================================================
_serx112_COM:
    cpi r16,112
    brne    _serx121_COM
    ;zuerst doTWI lschen
    lds r16,TWIstatus
    cbr r16,1<<doTWI    
    sts TWIstatus,r16
    ;zu B)
    ldi r16,121
    jmp _serxHOMEaction














	;B)anschlieend twi-daten in den rxbuffer 	
;-----------------------------------------------------------------
;nach erledigter rx-sequenz...						              |
;Daten aus TWIbuffer (TWI-Standard)nach RXbuffer konvertieren	  |
;-> SEUQUENZIELLE u. GEZIELTE Adressierung !				      |
;-> Kovertierter Frame ab 0x37a (sram) 			                  |
;tempTWIsiz=temp TWI-size						                  |
;tempTWIbufL=temp	TWIbufL					                  	  |
;tempTWIbufH	=temp TWIbufH				                      |
;twibuffer-inhalt:					                  			  |
;<control><instr.><data0><data1><...>                             |
;ACHTUNG: CK-Summe neu generieren								  |  					
;-----------------------------------------------------------------
;
;
_serx121_COM:
	cpi	r16,121
	brne	_serx122_COM
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx121prep:
	;RXbuffer auf erstes datenbyte 
	clr	zh
	ldi	zl,11
	;intern /extern?
  	sbrc    r22,newTX
    rjmp    _serx121TXbuf
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	rjmp	_serx121go
_serx121TXbuf:
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry

_serx121go:
	sts	tempRXbufL,zl
	sts	tempRXbufH,zh
	;daten ohne instruction-,controlbytes im twi-buffer
	ldi	zl, low(TWIbuffer)	;Add low byte ( x -(-y)) = x + y
	ldi	zh, high(TWIbuffer)	;Add high byte with carry
	sts	tempTWIbufL,zl
	sts	tempTWIbufH	,zh
	;next step
	ldi	r16,122
	jmp	_serxHOMEactiv


	;
	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx123_COMx:
	jmp	_serx123_COM
	;store-loop
_serx122_COM:
	cpi	r16,122
	brne	_serx123_COMx
	lds	yl,tempTWIbufL	;twi buffer adress pointer
	lds	yh,tempTWIbufH	
	lds	zl,tempRXbufL	;rx buffer  adress pointer
	lds	zh,tempRXbufH
	;
	lds	r21,twiSIZ	;twi datensize OHNE adress,control-u.instr.byte
	lds	r20,twiSTATUS
	;lade aus twibuffer
	ld	r17,y+
	;speichere nach rxbuffer (data)
	st	z,r17
	sbrs	r20,adrmod	;1== sequ(jedes weitere byte verw.)  0== gezielt (jedes 5.te byte verw.)
	rjmp	_serx122_gez
	;sequ.
	adiw	zh:zl,1	;rxbuffer+1
	jmp	_serx122_STORE
	;gez.
_serx122_gez:
	adiw	zh:zl,5	;rxbuffer+5
_serx122_STORE:
	sts	tempTWIbufL,yl
	sts	tempTWIbufH	,yh
	sts	tempRXbufL,zl
	sts	tempRXbufH,zh
	dec	r21
	sts	twiSIZ,r21	;store twi-size
	tst	r21
	breq	_serx122end
	rjmp	_serxHOMEactiv





	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx122end:
    ;wohin geht die reise?
    ;a)	newTX noch gesetzt -> dann kommt request-anforderung vom internen eventframe
    ; 
    ;b) newTXnicht gesetzt -> dann kommt request anforderung von extern
    ;
    sbrc    r22,newTX
    rjmp    _serx122noCHANGE
	jmp		_serx122extern	;anforderung (request von extern)
 _serx122noCHANGE:
   jmp		_serx122intern	;anforderung vom system (newTX=1)





;--------------------------------------------------------------------------------------
;A)-> anfrage von extern (daten im rxbuffer) 
_serx122extern:
    ;
    ;____________________________
    ;request0 | request1| aktion |
    ;_________|_________|________|
    ;   1     |    0    | ext.SAB|
    ;_________|_________|________|
    ;   0     |    1    | ext LHA|
    ;_________|_________|________|
    ;   1     |    1    | CORE   |
    ;_________|_________|________|
    ;hole gesichter mi-byte
    lds	r16,TEmiVAR
    sbrs    r16,requ0
    rjmp    _serx122EXT
    sbrs    r16,requ1
    rjmp    _serx122EXT


    ;nach internem core !
    ;-> mi modifizieren
	cbr	r16,1<<requ0
    cbr	r16,1<<requ1
	cbr	r16,1<<trans1
	sbr	r16,1<<trans0
    cbr r16,1<<request
	;
	;mi speichern
	ldi	zh,high(RXbufferST)
	ldi	zl,low(RXbufferST)
	st	z,r16
    ;
    cbr r22,1<<reqTX    ;ntig?
	sbr	r22,1<<BRcast	;unterdrckt ein weiteres txack
    ldi r16,2
   	jmp	_serxHOMEaction




;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;rxbuffer nach txbuffer kopieren
;anschlieend nach build request frame (35)
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx122EXT:
	;bufCNT
	clr	r16
	sts	bufCNT,r16
	;
 	ldi	r16,135	
	jmp	_serxHOMEactiv













;-------------------------------------------------------------------------
;B)-> anfrage von intern (via eventframe)
_serx122intern:

    ;____________________________
    ;request0 | request1| aktion |
    ;_________|_________|________|
    ;   1     |    0    | ext.SAB|
    ;_________|_________|________|
    ;   0     |    1    | ext LHA|
    ;_________|_________|________|
    ;   1     |    1    | CORE   |
    ;_________|_________|________|

    ;mi byte TXbuffer holen
	ldi	yl, low(TXbufferST)	;Add low byte ( x -(-y)) = x + y
	ldi	yh, high(TXbufferST)	;Add high byte with carry
	ld	r20,y
    ;request flags prfen
    sbrs    r20,requ0
    rjmp    _serx122LHA
    sbrs    r20,requ1
    rjmp    _serx122SAB


	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    ;nach internem core !
	;ACHTUNG bei weiterverarbeitung TXack verhindern !
    ;-> mi modifizieren
    cbr	r20,1<<requ0
    cbr	r20,1<<requ1
	sbr	r20,1<<trans1
	cbr	r20,1<<trans0
    cbr r20,1<<request
	;
	;mi speichern
	ldi	zh,high(TXbufferST)
	ldi	zl,low(TXbufferST)
	st	z,r20

    ldi r16,123
   	jmp	_serxHOMEaction




	;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx122SAB:
	;mi modifizieren
    ;-> mi modifizieren
    cbr	r20,1<<requ0
    cbr	r20,1<<requ1
	cbr	r20,1<<trans1
	sbr	r20,1<<trans0
    cbr r20,1<<request
	;
    cbr r22,1<<reqTX    ;ntig?
	cbr	r22,1<<newTX	;ntig?
	;mi speichern
	ldi	zh,high(TXbufferST)
	ldi	zl,low(TXbufferST)
	st	z,r20
    ;frame senden 
    sbr r19,1<<TXframe	
	clr r16
   	jmp	_serxHOME


;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx122LHA:
	;mi modifizieren
    ;-> mi modifizieren
    cbr	r20,1<<requ0
    cbr	r20,1<<requ1
	sbr	r20,1<<trans1
	cbr	r20,1<<trans0
    cbr r20,1<<request
	;
    cbr r22,1<<reqTX    ;ntig?
	cbr	r22,1<<newTX	;ntig?
	;mi speichern
	ldi	zh,high(TXbufferST)
	ldi	zl,low(TXbufferST)
	st	z,r20
    ;frame senden 
    sbr r19,1<<TXframe	
	clr r16
   	jmp	_serxHOME









;----------------------------------------------------------------------------------------------
;verarbeite von framer an twi angefordertes frame (request) im core
;_serx123_COM
;----------------------------------------------------------------------------------------------

_serx124_COMc:
        jmp     _serx124_COM

_serx123_COM:
		cpi	r16,123
		brne	_serx124_COM	
	;MATRIX-COUNTER =7 -> bestimme startadresse sram              
	;adressiere puffer(16bit) 	
	ldi	r17,7
	clr	zh	
	mov	zl,r17			;
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry
	;adresse aus puffer nach r23/r20 - target-adress
	ld	r23,z+
	ld	r20,z	
	;+korrekturfaktor sram
	subi	r20,low(-koMAP)
	sbci	r23,high(-koMAP)
	;
	sts	targetADRA,r20		;LSB/speichere effektive target-startadresse 
	sts	targetADRB,r23		;MSB 
	;nchster schritt
	ldi	r16,124		;matrix counter =124
	ldi	r17,9		;adresscounter  =9
	sts	bufCNT,r17	;bufCNT sichern
	rjmp	_serxHOMEactiv
	;
	;
	;
;.................................................................................
;- hier framesize mit aktuellem status vergleichen 
;- hier bertragungsmode prfen und entsprechend verzweigen (request/newTX/...)
;.................................................................................
_serx124_READYx:
	jmp	_serx124_READY			
_serx125_COMx:
	rjmp	_serx125_COM
	;MATRIX-COUNTER =11 -> 
_serx124_COM:
	cpi	r16,124
	brne	_serx125_COMx
	lds	r17,bufCNT	;lade momentanen zhlerstand
	;lade framesize...
	clr	zh	
	ldi	zl,5			;empfangenes framesize-byte		
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry
	ld	r20,z			;frame-size
	;vergleiche adresszhler/framesize
	cp	r17,r20
	brsh	_serx124_READYx		;... fertig verarbeitet (nun sollte TXack und TXframe...folgen)
	rjmp	_serx124_NOREQ

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;KEIN request oder newTX-anforderung
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx124_NOREQ:
	;adressiere puffer wg. speicherung...
	;adressiere puffer(16bit)= ( RXbufferST + bufCNT)
	;speichere byte aus segment nach buffer
	cbr	r22,1<<reqTX		;signalisiert dass nur ein TXack gesendet werden muss
	clr	zh	
	mov	zl,r17			;		
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry
	ld	r23,z			;daten aus puffer nach r23 - zieladresse in allen sram-segmenten

	lds	zh,targetADRB
	lds	zl,targetADRA		;sram-adressvariable nach z-pointer(l)...
	st	z+,r23			;daten aus r23 nun in speichersegment 
	sts	targetADRA,zl
	sts	targetADRB,zh



;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;gezielt oder sequenziell
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;mi laden
	lds	r16,teMIstore
	sbrc	r16,adr_mod		;1=sequ / 0=gezielt
	rjmp	_serx124_BACK	;sequenziell

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;gezielte adressierung
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;zeigt auf msb target-adresse
	ldi	zh,3
	add	r17,zh
	clr	zh	
	mov	zl,r17			;		
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry
	ld	r23,z+			;daten aus puffer nach r23 - zieladresse in allen sram-segmenten
	ld	r20,z
	;+korrekturfaktor sram
	subi	r20,low(-koMAP)
	sbci	r23,high(-koMAP)

	sts	targetADRA,r20
	sts	targetADRB,r23
	ldi	zh,1
	add	r17,zh
	jmp	_serx124_BACK






;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;dynamisches modul ret
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx124_BACK:
	;puffer-counter +1
	ldi	r23,1
	add	r17,r23			;puffer-counter+2
	sts	bufCNT,r17		;bufCNT sichern
	;matrix =11
	ldi	r16,124
	rjmp	_serxHOMEactiv	;...und zurck
	;



;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;ready
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
_serx124_READY:	
	;eindeutig...
	;daten waren fr's system bestimmt
	cbr	r19,1<<RXframe		;rx frame komplett verarbeiten
	cbr	r22,1<<reqTX	 
	cbr	r22,1<<newTX	;		
	clr		r16
	jmp	_serxHOME

;----------------------------------------------------------------------------------------------













;----------------------------------------------------------------------------------------------
;kopiere rxbuffer nach txbuffer - anschlieend request 
;_serx125_COM
;----------------------------------------------------------------------------------------------
_serx125_COM:
	cpi	r16,125
	brne	_serx126_COM
	lds	r17,bufCNT	;lade momentanen zhlerstand
	;block size
	ldi	r23,6
	;lade framesize...
	clr	zh	
	ldi	zl,5			;empfangenes framesize-byte		
	subi	zl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-RXbufferST)	;Add high byte with carry
	ld	r20,z			;frame-size
	;block loop
_serx125_COMloop:	
	;vergleiche adresszhler/framesize
	cp	r17,r20
	brsh	_serx125_COMready		;... fertig verarbeitet 
	;copy 10byte block
	tst	r23
	breq	_serx125_COMloopBACK	;block ready	
	dec	r23

	clr	zh
	mov	zl,r17
	subi	zl, low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh, high(-TXbufferST)	;Add high byte with carry

	clr	yh
	mov	yl,r17
	subi	yl, low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	yh, high(-RXbufferST)	;Add high byte with carry

	ld	r2,y
	st	z,r2

	inc	r17
	sts	bufCNT,r17

	rjmp _serx125_COMloop	



_serx125_COMloopBACK:
	ldi	r16,125
	jmp	_serxHOMEaction



_serx125_COMready:
   	sbr r22,1<<reqTX
    ;request senden und fertig	
	ldi r16,35
   	jmp	_serxHOMEaction

;----------------------------------------------------------------------------------------------










						;-----------------------------
						;ab _serx126 masterTX		  |
						;... schreibe daten nach slave|
						;-----------------------------
						_serx126_COM:
							cpi	r16,126
							brne	_serx127_COM	
							;hier daten aus TWIbuffer senden / framegre gem. TWIsiz
							ldi	zl,low(TWIbuffer)
							ldi	zh,high(TWIbuffer)
							sts	tempTWIbufL,zl ;twibuffer pointer l
							sts	tempTWIbufH	,zh ;  
							lds	r21,twiSIZ	;twisize laden
							sts	tempTWIsiz,r21	;temp.twisize
							ldi	r16,127                
							;start twi-tx loop....
							;-> bermittle daten
						_serx127_COM:
							cpi	r16,127
							brne	_serx128_COM
							lds	zl,tempTWIbufL	;twibuffer adress pointer l
							lds	zh,tempTWIbufH		;				  h
							ld	r20,z+	;daten aus twibuffer
							sts	twdr,r20	;...nach twi
							ldi	r21,(1<<twint) | (1<<twen)
							sts	twcr,r21
							sts	tempTWIbufL,zl	;aktuellen twi-adresspointer sichern
							sts	tempTWIbufH	,zh
							ldi	r16,128
							;timeoutcounter default
							ldi		r17,i2cout
							sts		I2Ctimeout,r17
							;rjmp	_serxHOMEactiv TEST
							;->twint checken
						_serx128_COM:
							cpi	r16,128
							brne	_serx129_COM
							lds 	r20,twcr
							sbrs	r20,twint
							rjmp	_serxi2Ctimeout			;hier knftig auf timeout-routine verzweigen - prozedur noch nicht abgeschlossen
							ldi	r16,129		;....................abgeschlossen
							;rjmp	_serxHOMEactiv
							;-> statusmatrix checken
						_serx129_COM:
							cpi	r16,129
							brne	_serx130_COM
							lds	r20,twsr
							andi	r20,0xf8	;maskiere wg. prescaler (... darf nicht beeinflusst werden)
							cpi	r20,rxACK	;HIER entsprechend der ablaufsteuerungsmatrix prfen
							breq	twiTXstatusOK ;master erhielt ack !
							ldi	r16,131	;...master hat nack erhalten -> nun stop !
							rjmp	_serx131_COM	;TEST_serxHOMEactiv
						twiTXstatusOK:
							ldi	r16,130
							;rjmp	_serxHOMEactiv
							;->framesize checken 
						_serx130_COM:
						 	cpi	r16,130
							brne	_serx131_COM
							lds	r21,tempTWIsiz	;temp twisize laden
							dec	r21
							tst	r21
							brne	_serx130twiLOOP
							ldi	r16,132	;ende erreicht - nun stop oder rep.start !
							rjmp	_serxHOMEactiv
						_serx130twiLOOP:
							sts	tempTWIsiz,r21
							ldi	r16,127
							rjmp	_serxHOMEactiv
							;-> stop wg. nack....
						_serx131_COM:						
							cpi	r16,131
							brne	_serx132_COM
							jmp		_serxI2Cstop	;ready !
					




		

;*********************************************
;hier entscheiden
;was nach stop-bit passieren soll
;AUFGEPASST !
;vielleicht soll ja statt stop ein repstart
;angeleiert werden ?
;hierfr flag <repSTART> in TWIstatus
;prfen...
;0==kein anschlieendes read
;1==anschlieend read   reqTWIsiz
;*********************************************
_serx132_COM:
	cpi		r16,132
	brne	_serx150_COM
	;prfe ob repSTART ==1
	lds		r20,twiSTATUS		;twi-status-flags
	sbrc	r20,repSTART		;0=ready / 1=repSTART
	rjmp	_serx132_COMrep
	;ready
	jmp		_serxI2Cstop	;ready !
	;..........................................
	;repSTART !!!
_serx132_COMrep:
	;A) read 
	lds		r21,twiADR		
	sbr		r21,1<<0
	sts		twiADR,r21
	;B) restore twisize
	lds		r21,reqTWIsiz
	sts		TWIsiz,r21
	;C) timeoutcounter default
	ldi		r17,i2cout
	sts		I2Ctimeout,r17
	;D) next repSTART
	ldi	r16,100	;nun rep.start !
	jmp	_serxHOMEactiv
				



















;----------------------------------------------------------------------------------
;
;noch nicht definiert - DYNAMISCHES MODUL
;----------------------------------------------------------------------------------
_serx150_COM:
	cpi	r16,150
	brne	_serxHOME
	;TEST
	cbr	r22,1<<newTX
	jmp		_serxI2Cstop	;ready !
	;TEST END


	rjmp	_serxHOME
;----------------------------------------------------------------------------------
;ZURCK....
;----------------------------------------------------------------------------------
_serxHOMEactiv:
	;setze sercom-action flag
	lds	r17,SEflagVARc
	sbr	r17,1<<SERcomACT
	sts	SEflagVARc,r17
	;
	;...bye bye
_serxHOME:
	;prfe counter...
	tst	r16
	brne	_serxHOMEaction

    ;TEST clear twi flag
;	lds	r20,twiSTATUS
;    cbr r20,1<<doTWI    ;keine aktion nach 
;  	sts	twiSTATUS,r20

	;lsche sercom-action flag
	lds	r17,SEflagVARc
	cbr	r17,1<<SERcomACT
	sts	SEflagVARc,r17
_serxHOMEaction:
	sts	matrixCNT,r16		;lade matrix counter variable
	sts	SEflagVARa,r19		;serial flag variable A laden
	sts	SEflagVARb,r22		; "             "     B   "	
	ret














.exit















