;----------------------------------------------------------------------------------------
;OBJECT_FRAMER
;........................................................................................
;->Generierung von Datenobjekten entsprechend Data-Class Tabelle & konvertierung nach sercom 
;->Vorbereitung zum Transfer via L(ocal) A(rea) B(us)
;->Direct Memory Access Profiles nach sercom konvertieren
;----------------------------------------------------------------------------------------
;
;Belegte Speicherbereiche 
;a)
;<CLASS_DEF>			->	beinhaltet:	<data_class> 	0...255
;										<eol> 			0...32768 
;										<data_length> 	1...5
;										<sou_var_adr>   0...65535 (max.5*16Bit-Adresse)
;										<event>			0...255
;

;.equ	class_define    					;table startadresse
;.equ	class_defineEEPROM		=	    	;table startadresse
;.equ	class_segment			=	15		;gre eines objekt-segmentes
;.equ	class_count				=	20		;maximale anzahl der objekte
;.equ	class_def_end			=	class_define + (class_segment * class_count) 
;-> entspricht einem eeprom-speicherplatzbedarf von 300Bytes
;-> wird beim system start vom e2prom ins sram kopiert
;-> wird beim setup ins e2prom & ins sram kopiert
;
;---------------------------------------------------------------------------------------
;
;b)
;<CLASS_FILTER_DEFINE>	-> 	beinhaltet:	<data_class>	0...255
;										<client_adr>	0...255
;										<Xdata to sys>  1..5(max.5*8Bit Data)/255=no data to system
;										<sysADR16bit>,<sysADR16bit>...
;
;.equ	class_filter_def		=	class_define + (class_segment * class_count) +1 ;table start
;.equ	class_filter_defEEPROM	=	class_defineEEPROM + (class_segment * class_count) +1 ;table start
;.equ	class_filter_segment	=	13		;gre eines filtersegmentes
;.equ	class_filter_count		=	30		;maximale filteranzahl
;.equ	class_filter_limit		=	(class_filter_def + class_filter_count) -1
;-> entspricht einem eeprom/sram-speicherplatzbedarf von 390Bytes
;-> wird beim system start vom e2prom ins sram kopiert
;-> wird beim setup ins e2prom & ins sram kopiert
;
;---------------------------------------------------------------------------------------
;
;c)
;<DYN_OBJECT_DEFINE>	->	beinhaltet:	<data_class>	0...255
;										<client_adr>	0...255
;										<eol>			0...32768
;										<Xdata to sys>	1...5(max.5*8Bit Data)
;										<data>,<data>...
;
;.equ	dyn_object_define		=	0xab9	;object-table startadresse
;.equ	dyn_object_segment		=	10		;gre eines objekt-segmentes
;.equ	dyn_object_count		=	40		;maximale anzahl von objekten
;-> entspricht einem sram-speicherplatzbedarf von 400Bytes
;
;---------------------------------------------------------------------------------------
;
;d)
;<dyn_object_TE>  	    -> beinhaltet: 	<data_class>	0...255
;										<client_adr>	0...255
;										<eol>			0...32768
;										<Xdata to sys>	1...5(max.5*8Bit Data)
;										<data>,<data>...
;
;temporrer objektspeicher
;.equ	dyn_object_TE			=	0x7d0 ;
;.equ	dyn_object_limitTE		=	(dyn_object_TE + dyn_object_segment) - 1
;
;---------------------------------------------------------------------------------------
;
;Modulvariablen:
;.equ	object_state			;ablaufvariable
;.equ	object_counter			;anzahl, zum event verknpfter objekte
;.equ	temp_Ctable_adrL		;temporre Class-Tabellenadresse lsb
;.equ	temp_Ctable_adrH		; "               		   "     msb
;.equ	temp_event_adrL			; "        eventvariablenadresse lsb
;.equ	temp_event_adrH			; "        eventvariablenadresse msb
;.equ	temp_event				; "        eventX (0...255)
;.equ	temp_event_var			; "        event flagvariable (daten)
;.equ	temp_bitmask			; "		   event bitmask
;
;
;..............................................................................................
;Externe Steuerungsvariablen:
;TWIstatus	-> TWI Status Flags
;..............................................................................................
;Eventbuffer:
;STARTeventBUF		=	0xc95		+ koMAP		;startadresse eventbuffer

;
;..............................................................................................
;
;modulstart
_ob000fr:

	;prfe ob als remote aktiv
	;ttig?
	lds	r17,remFLAG
	sbrc	r17,RTon			;activ remote
	rjmp	_ob999fr				;...dann kein framer-mode mglich !
	;------------------------------------------------------------------------
	;NEU ab4/07
	;Objectstates 0,1,3 sind immer mglich !
	;------------------------------------------------------------------------
	lds	r20,object_state	;ablaufvariable laden
	tst	r20
	brne	_ob000frA
	jmp		_ob010fr		;0= neues event auf tabellenverknpfung prfen
_ob000frA:
	cpi	r20,1
	brne	_ob000frB
	jmp		_ob020fr		;1= eventbezogene objekte 
_ob000frB:
	cpi	r20,3
	brne	_ob000frC
	jmp		_ob050fr		;3= eventbezogene DMA-profiles
	;------------------------------------------------------------------------



_ob000frC:
	;bei aktiver datenbertragung ist diese routine blockiert !!!
	lds	r16,SEflagVARa		;serial flag variable A laden
	lds	r17,SEflagVARb		; "             "     B
	lds	r18,SEflagVARc

	;TEST
;	sbrc	r16,RXframe
;	rjmp	_ob999fr		;buffer besetzt 

	sbrc	r16,TXframe
	rjmp	_ob999fr		; "             "            "         "

	sbrc	r16,SEbusy
	rjmp	_ob999fr		;aktiver bertragung


	;TEST
	sbrc	r18,FRAMEfreeze	;extended if instruction running
	rjmp	_ob999fr
	;

	;vorbereitung im sercom luft
	sbrc	r17,newTX
	rjmp	_ob999fr

	;vorbereitung im sercom luft
	sbrc	r17,reqTX
	rjmp	_ob999fr


   ;vielleicht noch einen
   ;twi-job zu erledigen, dann kein neues frame !
    lds     r19,TWIstatus
    sbrc    r19,doTWI 
    rjmp    _ob999fr

	sbrs	r16,SEenabled
	rjmp	_ob999fr		;serielle module disabled

    ;any frame in tx-buffer to transmit
    sbrc    r18,FRbuild
    rjmp	_ob999fr
   ;hot-shot framebuilder is blocking?
    sbrc    r18,FRAMEhot
    rjmp	_ob999fr
 
	;now disable other framebuilders
    sbr r18,1<<FRAMEdma        
	sts	SEflagVARc,r18
  
;
;...............................................................................................
;modulstatus prfen
;0= neues event auf verknpfung (objekt/dma) prfen
;1= objektframes generieren & bertragen
;2=	dma-frames generieren & bertragen
_ob001fr:
	lds	r20,object_state	;ablaufvariable laden
	cpi	r20,2
	brne	_ob002fr
	jmp		_ob030fr		;2= objektframes generieren & bertragen
_ob002fr:
	cpi	r20,4
	brne	_ob006fr	
	jmp		_ob060fr		;4= DMA-profiles generieren & bertragen

	;object-state >4
	;startbedingungen setzen
_ob006fr:
	;status auf=0
	clr		r20
	sts		object_state,r20
	;event = 1
	ldi		r20,1
	sts		temp_event,r20
	;objekt-counter =0
	clr		r20
	sts		object_counter,r20
	;back
	jmp		_ob999fr		;x= nicht gltig - mit startwerten zurck!
;
;
;
;
;...............................................................................................
;status=0
;event in flash-tabelle suchen & event-flag status prfen
_ob010fr:
	;zuerst prfen ob event aktiv ?
	;a)eventvariable holen
		;zuletzt aktiviertes event aus buffer holen
		clr		yh
		lds		yl,eventPOINT		;eventpointer laden (zeigt auf letztes event)
		tst		yl
		breq	_ob011fr 			;0= no events in buffer...
		cpi		yl,100				;>=100 ?
		brsh	_ob010frLIMIT		;overflow...too much events...
		;in range...& adressieren
		subi	yl,low(-STARTeventBUF)
		sbci	yh,high(-STARTeventBUF)
		;eventadresse (table) speichern
		sts		temp_event_adrL,yl	
		sts		temp_event_adrH,yh
		;event holen
		ld		zl,y
		lds		yl,eventPOINT		;eventpointer laden (zeigt auf letztes event)
		;event pointer korrigieren (-1)
		dec		yl
		sts		eventPOINT,yl
		;prfe ob event >0	(0=nicht gltig!)
		tst		zl
		breq	_ob011fr			;zurck & nchstes event prfen						
		;aktives event speichern
		sts		temp_event,zl
		;event im buffer lschen
		clr	r20
		lds	zh,temp_event_adrH
		lds	zl,temp_event_adrL
		st	z,r20 			;event lschen (=0)
		jmp		_ob015fr
		;nochmal...
_ob010frLIMIT:
		ldi		yl,99
		sts		eventPOINT,yl
		jmp		_ob010fr	
		;zurck...
_ob011fr:
		jmp		_ob999fr





		;
		;.......................................................
		;eventflag zu eventX aktiv
		;ab modul-status=1 prfen ob in <class_def>-table enthalten ?
_ob015fr:
		ldi		r20,1
		sts		object_state,r20
		;class-table pointer auf erstes objekt
		ldi		zh,high(class_define)
		ldi		zl,low(class_define)
		;adresse speichern
		sts		temp_Ctable_adrL,zl
		sts		temp_Ctable_adrH,zh
		;
		jmp		_ob999fr
;
;
;
;
;............................................................................................................
;status=1
;zu aktivem event verknpfte objekte prfen
;a)<class_def>-table + object_counter laden
_ob020fr:

		;nun objekt-event aus class-table holen
		;a)class_define-eventposition holen
		lds		zh,temp_Ctable_adrH
		lds		zl,temp_Ctable_adrL
		;class-table pointer auf event...
		ldi		yl,class_segment-1
		clr		yh
		add		zl,yl
		adc		zh,yh
		;range prfen...
		ldi		r21,high(class_def_end)
		ldi		r20,low(class_def_end)
		cp		zl,r20
		cpc		zh,r21
		brsh	_ob022fr				;object segment out of range...evtl.dma-frame??
		;in range...
		;& adresse speichern
		sts		temp_Ctable_adrL,zl
		sts		temp_Ctable_adrH,zh
		;aktuelles event laden
		lds		r20,temp_event	
		;b)event aus table laden
		ld		r21,z		
		;nun auf bereinstimmung prfen...
		cp		r20,r21
		breq	_ob025fr				;dann kann's ja losgehen...
	    ;nicht dabei...auf nchstes segment setzen
		adiw	zh:zl,1
		sts		temp_Ctable_adrL,zl
		sts		temp_Ctable_adrH,zh
		jmp		_ob999fr	;zurck






;------------------------------------------------------------
;object segment out of range - prfe ob event im DMA-segment
;enthalten ist ....
;------------------------------------------------------------
_ob022fr:
		ldi		r20,3
		sts		object_state,r20
		;dma-table pointer auf event im ersten profile
		ldi		zh,high(profiSTram)
		ldi		zl,low(profiSTram)
		;adresse speichern
		sts		temp_Ctable_adrL,zl
		sts		temp_Ctable_adrH,zh
		;
		jmp		_ob999fr


;.................................................
;pointer auf erstes byte des objekts
;.................................................
_ob025fr:
	ldi		r20,2
	sts		object_state,r20
	;-14
	lds		zh,temp_Ctable_adrH
	lds		zl,temp_Ctable_adrL
	clr		yh
	ldi		yl,class_segment-1
	sub		zl,yl
	sbc		zh,yh
	sts		temp_Ctable_adrL,zl
	sts		temp_Ctable_adrH,zh
	jmp		_ob999fr




;............................................................................................. 
;status =2
;
;objectframe (sercom-format)
; ______________________________________________________________________________________
;|mi    |ho-tar |sub-tar|ho-sou |sub-sou|frame  |type   |data   |eol    |var-siz|data   |
;|byte  |adress |adress |adress |adress |size=  |=250   |class  |16bit  |max.5* |1...5  |
;|      |       |       |       |       |21     |object |       |msb/lsb|data   |msb/lsb|
;|______|_______|_______|_______|_______|_______|_______|_______|_______|_______|_______|
;
;
;mode-identifier: 
;          _______________________________________________________________
;		  |mi7    |mi6    |mi5    |mi4    |mi3    |mi2    |mi1    |mi0    |
;		  |_______|_______|_______|_______|_______|_______|_______|_______|
;		  |broadc.|setup  |request|adr_mod|trans1 |trans0 |requ1  |requ0  |
;		  |_______|_______|_______|_______|_______|_______|_______|_______|
;         |1=adr. |1=set. |1=req  |1=inkr.|data to|data to|request|request|
;         |0=broa.|0=kom. |0=n.req|0=gezi.|hostTAR|snetTAR|hostSRC|snetSRC|
;		  |	      |       |       |       |       |       |requ1&0=1 lokal|
;		  |_______|_______|_______|_______|_______|_______|_______________| 

_ob030fr:
;-> zuerst alle objekt parameter & adressen aus dem speichersegment laden...
;-> dann frame nach sercom-standard erstellen & in buffer ablegen
;-> transfer initialisieren (flags setzen)...
	;
	;a) startadresse transferbuffer laden
	ldi	zh,high(TXbufferST)
	ldi	zl,low(TXbufferST)
	;b) startadresse dma-table laden
	lds	yh,temp_Ctable_adrH
	lds	yl,temp_Ctable_adrL
	;c) mi generieren & nach transferbuffer (0)
	ldi		r20,0b00001000	;broadcast & data to host target
	st		z,r20
	;d) source net-adress nach transferbuffer
	adiw	zh:zl,3
	lds		r20,net_sou_adress	;source adresse laden
	st		z,r20
	;e) framesize=16 nach transferbuffer
	adiw	zh:zl,2
	ldi		r20,16
	st		z+,r20
	;f) type=250 nach transferbuffer
	ldi		r20,250
	st		z+,r20  ;! z+
	;g) data-class aus objekt holen & nach transferbuffer
	ld		r20,y+	;dataclass holen
	st		z+,r20
	;h) eol (16bit)aus objekt holen & nach transferbuffer
	ld		r20,y+
	st		z+,r20
	ld		r20,y+
	st		z+,r20
	;i) data-length aus objekt holen & nach transferbuffer
	ld		r20,y+
	st		z+,r20
	;j) variablenadressen aus objekt extrahieren, daten aus adresse holen
	;   & nach transferbuffer speichern
	;   immer 5 variablen adressieren (auch wenn weniger definiert sind?!)  ;why!!!!!!
	ldi		r20,5
_ob031fr:
	tst		r20
	breq	_ob032fr	;fertig
	mov	r2,zl
	mov	r3,zh
	ld		zh,y+ ;msb
	ld		zl,y+	;lsb
	;----------------------------------------------
	;adressen wurden netto (ohne komap) definiert
	;-> jetzt +koMAP
	subi	zl,low(-koMAP)
	sbci	zh,high(-koMAP)
	ld		r21,z		;8bit daten holen
	;----------------------------------------------
	mov	zl,r2
	mov	zh,r3
	st		z+,r21
	;
	dec		r20
	jmp		_ob031fr	;loop
	;
_ob032fr:
	;k) class_table adresse speichern (zeigt nun auf anfang des nchsten segments)
	;NEU!
	adiw		yh:yl,1	;+1 weil ja nur bis zur letzten Dataadresse der loop luft
	sts		temp_Ctable_adrL,yl
	sts		temp_Ctable_adrH,yh
	;l) modul-status =1 (nchstes objekt)
	ldi		r20,1
	sts		object_state,r20
	;m) transfer initialisieren
	;framebuilder startet...
	lds	r18,SEflagVARc
	sbr	r18,1<<FRbuild
	;now enable other framebuilders
    cbr r18,1<<FRAMEdma        
	sts	SEflagVARc,r18
	;n)bertrage objectframe
	lds	r22,SEflagVARb
	sbr	r22,1<<newTX
	sts	SEflagVARb,r22
	;zurck
	jmp	_ob999fr













;...........................................................................................................
;status=3
;DMA-Profile (44Bytes)
; ______________________________________________________________________________________________
;|mi    |ho-tar |sub-tar|ho-sou |sub-sou|data   |type   |source |target |data   |       |event  |
;|byte  |adress |adress |adress |adress |size   |=X     |adress |adress |8bit   |       |gem.   |
;|      |       |       |       |       |       |(table)|16bit  |16bit  |       |...    |table  |
;|______|_______|_______|_______|_______|_______|_______|_______|_______|_______|_______|_______|
;
;Belegte Speicherbereiche 
;a)
;<profiSTram>			->	beinhaltet:	<mi	>		 	0...255
;										<host_target> 	0...255 
;										<sub_target> 	0...255
;										<host_source>   0...255
;										<sub_source>	0...255
;										<data_size>		1...240
;										<transfer_type> 1...255 (...250 == object !)
;										<sou/tar_adress>0...65535 (9*16bit sou/tar) 
;										<event>			0...255	
;<profiSEGsize>			=	44											;segmentgre 44bytes
;<profiSTcnt>			=	10											;10 profiles
;<profiSTlim>			=	profiSTram + (profiSEGsize * profiSTcnt)	;profile limit
;...........................................................................................................
_ob050fr:
		;nun event aus profile-table holen
		;a)profiSTram-eventposition holen
		lds		zh,temp_Ctable_adrH
		lds		zl,temp_Ctable_adrL
		;dma-table pointer auf event
		ldi		yl,profiSEGsize-1	;44
		clr		yh
		add		zl,yl
		adc		zh,yh
		;range prfen...
		ldi		r21,high(profiSTlim)
		ldi		r20,low(profiSTlim)
		cp		zl,r20
		cpc		zh,r21
		brsh	_ob059fr	;dma segment out of range...
		;in range...
		;adresse speichern
		sts		temp_Ctable_adrL,zl
		sts		temp_Ctable_adrH,zh
		;aktuelles event holen...
		lds		r20,temp_event
		;event aus table holen
		ld		r21,z		
		;nun auf bereinstimmung prfen...
		cp		r20,r21
		breq	_ob055fr	
		;nicht dabei...auf nchstes segment setzen
		adiw	zh:zl,1
		sts		temp_Ctable_adrL,zl
		sts		temp_Ctable_adrH,zh
		jmp		_ob999fr	;zurck

					



;.................................................
;pointer auf erstes byte des frames
;.................................................
_ob055fr:
	ldi		r20,4
	sts		object_state,r20
	;-43
	lds		zh,temp_Ctable_adrH
	lds		zl,temp_Ctable_adrL
	clr		yh
	ldi		yl,profiSEGsize-1
	sub		zl,yl
	sbc		zh,yh
	sts		temp_Ctable_adrL,zl
	sts		temp_Ctable_adrH,zh
	jmp		_ob999fr


;.....................................................
;event in data-objects & dma-profiles nicht enthalten
;.....................................................
_ob059fr:
	clr		r20
	sts		object_state,r20
	jmp		_ob999fr






;...............................................................................................
;generiere DMA-Frame
;...............................................................................................
_ob060fr:
;-> zuerst alle dma parameter & adressen aus dem speichersegment laden...
;-> dann frame nach sercom-standard erstellen & in buffer ablegen
;-> transfer initialisieren (flags setzen)...
	;
	;a) startadresse dma-table laden
	lds	zh,temp_Ctable_adrH
	lds	zl,temp_Ctable_adrL
	;aktuelle startadresse sichern
	mov	yl,zl
	mov	yh,zh
	;adressierung korrigieren == auf nchstes segment !
	adiw	zh:zl,profiSEGsize	;44
	;dma table adresse speichern (zeigt nun auf anfang des nchsten segments)
	sts		temp_Ctable_adrL,zl
	sts		temp_Ctable_adrH,zh
	;
	;b) startadresse transferbuffer laden
	ldi	zh,high(TXbufferST)
	ldi	zl,low(TXbufferST)

	;c) copy size 
	ldi	r20,5
	;d) mi temporr sichern
	ld	r16,y										;mi in r16
	;..............................................
	;e) phys. adresses/mi.. 1:1 kopieren	
_ob060frLOOP:
	tst		r20
	breq	_ob061fr		;durchgang fertig
	ld		r21,y+
	st		z+,r21
	dec		r20
	jmp		_ob060frLOOP	;loop
	;..............................................
	;f) framesize temporr sichern & kopieren
_ob061fr:
	;datasize holen
	ld		r21,y+
	;temp. sichern
	mov		r17,r21									;datasize in r17
	;datasize speichern (hier spter framesize einsetzen)
	st		z+,r21
	;copy size 
	ldi		r20,5
_ob062frLOOP:
	tst		r20
	breq	_ob063fr		;durchgang fertig
	ld		r21,y+
	st		z+,r21
	dec		r20
	jmp		_ob062frLOOP	;loop
	;...................................................................
	;i) - txbuffer zeigt auf erstes datenbyte
	;	- temp_Ctable_adr zeigt auf nchste source adresse msb
	;	mi -> bei sequ. adressierung hier nach back verzweigen
	;	mi -> bei gezielter adressierung weiter adressieren
	;...................................................................
_ob063fr:
	sbrs	r16,adr_mod		;0= gezielt / 1= sequ.
	rjmp	_ob064fr		;0
	jmp		_ob068fr		;1
	;gezielt...
	;zuerst nchstes byte im txbuffer (daten == werden in sercom eingefgt) 
	;berspringen
_ob064fr:
	;txbuffer +1 = nchste adresse
	adiw	zh:zl,1
	;copy segment size
	ldi		r20,4
	;copy x segmente (1x ist bereits kopiert)
	;hierfr datasize-1 holen...
	mov		r16,r17
	subi	r16,1
	;war nur datasize=1 ?
	tst		r16
	brne	_ob064frLOOP
	jmp		_ob069fr
	;
_ob064frLOOP:
	tst		r20
	breq	_ob065fr		;durchgang fertig
	ld		r21,y+
	st		z+,r21
	dec		r20
	jmp		_ob064frLOOP	;loop
_ob065fr:
	dec		r16
	tst		r16
	breq	_ob069fr		;durchgang fertig - 

	;txbuffer +1
	adiw	zh:zl,1
	;copy segment size
	ldi		r20,4
	jmp		_ob064frLOOP

	


;....................................................................................
;sequ. adressierung ende
;....................................................................................
_ob068fr:
	;framesize aus datasize generieren
	;header(7)+ adresses(4)+datasize
	;=16+datasize
	ldi		yl,11
	add		r17,yl					;16+datasize
	;nun in txbuffer integrieren
	clr		yh
	ldi		yl,5
	subi	yl,low(-TXbufferST)
	sbci	yh,high(-TXbufferST)
	st		y,r17		 			;framesize nach TXbuffer
	;fertig...
	jmp		_ob070fr


;....................................................................................
;gez. adressierung ende
;....................................................................................
_ob069fr:
	;framesize aus datasize generieren
	;header(7)+(datasize * 5)
	;7+(datasize*5)
	ldi		yl,7
	ldi		yh,5
	mul		yh,r17					;datasize*5
	add		yl,r0					;+7=framesize
	mov		r17,yl
	;nun in txbuffer integrieren
	clr		yh
	ldi		yl,5
	subi	yl,low(-TXbufferST)
	sbci	yh,high(-TXbufferST)
	st		y,r17		 			;framesize nach TXbuffer
	;fertig...
	jmp		_ob070fr			





;....................................................................................
;bertragung starten
;....................................................................................
_ob070fr:
	;m) modul-status =3 (nchstes dma-profile)
	ldi		r20,3
	sts		object_state,r20
	;n) transfer initialisieren
	;framebuilder startet...
	lds	r18,SEflagVARc
	sbr	r18,1<<FRbuild
	;now disable other framebuilders
    cbr r18,1<<FRAMEdma        
	sts	SEflagVARc,r18
	;bertrage objectframe
	lds	r22,SEflagVARb
	sbr	r22,1<<newTX
	sts	SEflagVARb,r22
	;zurck
	jmp	_ob999fr


;................................................................................................
;zurck/ret
;................................................................................................
_ob999fr:
	ret




