;------------------------------------------------------------------------------
;key-sequencer
;------------------------------------------------------------------------------
;- schalter-eingnge lsen, in abhngigkeit des schaltintervalles maximal 3 
;  definierbare befehlssequenzen aus
;- das schaltintervall variiert zwischen 1-255 (*100ms) <wait-parameter>
;- jedem schaltbefehl ist, im bedarfsfall ein weiterer toggle-befehl zugeordnet. 
;  D.h. dass bei wiederholter ausfhrung des schaltbefehls der weitere Execution
;  Befehl ausgefhrt wird.
;- pro befehlssequenz wird ein stimuliparameter an die zieladresse bertragen, 
;  sowie ein systemevent an das system zur weiterverarbeitung bergeben.
;------------------------------------------------------------------------------
;
;____       __......
;    |     |
;    |_____|
;          .

;         a) H/L-Flanke 
;            = nchste instruction 
;		     vorbereiten
;         b) L/H-Flanke startet den 
;         	 temp. wait-counter.
;			 instruction nach erreichen
;			 wait-time ausfhren...
;         c) L-Level 
;            = warte auf H-Level
;			 -> Pin-Status auf LOW setzen
;         d) H-Level auerhalb wait-time
;            = aktuelle Instruction ausfhren
;            -> Pin-Status auf HIGH setzen

_key000SEQ:
	;prfe letzten pinstatus
	;0=low / 1=high
	lds		r16,keyEDGE	;edge-flags
	in		r17,PINE	;pins portE holen
	;ablaufcounter...
	lds		r18,keyPINcnt
	tst		r18
	brne	_key001SEQ
	inc		r18
	sts		keyPINcnt,r18	
	;eingang 0
	sbrs	r17,PINE3
	rjmp	_CHKin0Lx
	call	_CHKin0H
	jmp		_key999SEQ
	;	
_key001SEQ:
	;eingang 1
	cpi		r18,1
	brne	_key002SEQ
	inc		r18
	sts		keyPINcnt,r18
	sbrs	r17,PINE4
	rjmp	_CHKin1Lx
	call	_CHKin1H
	jmp		_key999SEQ
	;
_key002SEQ:
	;eingang 2
	cpi		r18,2
	brne	_key003SEQ
	inc		r18
	sts		keyPINcnt,r18
	sbrs	r17,PINE5
	rjmp	_CHKin2Lx
	call	_CHKin2H
	jmp		_key999SEQ
	;
_key003SEQ:
	;eingang 3
	cpi		r18,3
	brne	_key004SEQ
	inc		r18
	sts		keyPINcnt,r18
	sbrs	r17,PINE6
	rjmp	_CHKin3Lx
	call	_CHKin3H
	;
_key004SEQ:
	;eingang 4
	cpi		r18,4
	brne	_key005SEQ
	clr		r18
	sts		keyPINcnt,r18
	sbrs	r17,PINE7
	rjmp	_CHKin4Lx
	call	_CHKin4H
	jmp		_key999SEQ
	;...
_key005SEQ:
	clr		r18
	sts		keyPINcnt,r18
	jmp		_key999SEQ
	;
_CHKin0Lx:
	call	_CHKin0L
	jmp		_key999SEQ
_CHKin1Lx:
	call	_CHKin1L
	jmp		_key999SEQ
_CHKin2Lx:
	call	_CHKin2L
	jmp		_key999SEQ
_CHKin3Lx:
	call	_CHKin3L
	jmp		_key999SEQ
_CHKin4Lx:
	call	_CHKin4L
	jmp		_key999SEQ

;-----------------------------------------------------------------------------------
;eingang0= high
;-----------------------------------------------------------------------------------
;prfe ob's:
;a) eine l/h-flanke (edge-flag =0)
;oder
;b) ein h-level (edge-flag=1) ist
;INFO: edge-flag=1 wird bei ist-time >=wait-time gesetzt
_CHKin0H:
	sbrs	r16,K0edge
	rjmp	_CHKin0Hlow		;edge=0
	jmp		_CHKin0Hback	;edge=1 ...back !	
	;...............................................................
	;edge=0 <=> counter prfen...
_CHKin0Hlow:

	;wait state setzen
	lds		yl,keyWTstate
	sbr		yl,1<<K0wait
	sts		keyWTstate,yl


	;wait-counter soll (table) holen
	ldi		yl,0		;eingang0
	call	_RDkeyWAIT	;soll-parameter in yh
	;wait-counter ist holen
	lds		yl,key0CNT  ;ist- 8bit counter
	;vergleiche...
	cp		yl,yh			;ist|soll
	brsh 	_CHKin0Hexe		;ist-time >= wait-time == execute
	;<...warte
	;counter pro 100ms +1
	lds		yl,key0CNTte 	;temp.100ms counter
	lds		yh,_100ms		;100ms system counter
	cp		yl,yh
	brne	_CHKin0HlowTIME	;zeitkorrektur....
	jmp		_CHKin0Hback	;ohne aktion zurck...

	;................................................................
_CHKin0HlowTIME:
	;100ms ist-time temp. == 100ms systemcounter
	mov		yl,yh
	sts		key0CNTte,yl
	;wait time +1
	lds		yl,key0CNT  ;wait time- 8bit counter
	inc		yl
	sts		key0CNT,yl
	jmp		_CHKin0Hback	;zurck...


	;................................................................
	;execute sequence
	;................................................................	
	;ist-time >= wait-time == execute
_CHKin0Hexe:
	;l/h-eventflag
	ldi		yl,162
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;
	;edge-flag=1
	sbr		r16,1<<K0edge
	;ist-counter=0
	clr		yl
	sts		key0CNT,yl	;
	;soll-toggle-flags zu key-segment (toggeln ja/nein)
	ldi		yl,0
	call	_RDkeyTOG	;...in yh !
	;welche sequenz (im segment) wurde angewhlt ?
	;..geschah bei h/l flanke !!!
	lds		yl,key0SEQ	;1-3
	;sequenz 1
	cpi		yl,1
	brne	_CHKin0HexeA	
	;
	;---------------------
	;key0/sequ1-eventflag
	ldi		yl,141
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG1seq			;togglen erlaubt ?
	rjmp	_CHKin0HexeDO	;nein...
	jmp		_CHKin0Hexe1TOG		;...ja !
	;sequenz 2
_CHKin0HexeA:
	cpi		yl,2
	brne	_CHKin0HexeB
	;
	;---------------------
	;key0/sequ2-eventflag
	ldi		yl,142
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG2seq			;togglen erlaubt ?
	rjmp	_CHKin0HexeDO	;nein...
	jmp		_CHKin0Hexe2TOG		;...ja !
	;sequenz 3
_CHKin0HexeB:
	cpi		yl,3
	brne	_CHKin0HexeERR
	;
	;---------------------
	;key0/sequ3-eventflag
	ldi		yl,143
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG3seq			;togglen erlaubt ?
	rjmp	_CHKin0HexeDO	;nein...
	jmp		_CHKin0Hexe3TOG		;...ja !
	;>3 error
_CHKin0HexeERR:
	clr		yl
	sts		key0SEQ,yl
_CHKin0Hback:
	ret			;zurck...




	;......................................................
	;check toggle status sequenz 1
	;......................................................
_CHKin0Hexe1TOG:
	lds		zl,key0TOG			;key-segment toggle flags
	sbrs	zl,TOG1seq			;bit 0/1 ?
	rjmp	_CHKin0Hexe1DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG1seq
	sts		key0TOG,zl
	jmp		_CHKin0HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin0Hexe1DO:
	;toggeln h
	sbr		zl,1<<TOG1seq
	sts		key0TOG,zl
	rjmp	_CHKin0HexeDO		;a/b/c instructions ausfhren


	;......................................................
	;check toggle status sequenz 2
	;......................................................
_CHKin0Hexe2TOG:
	lds		zl,key0TOG			;key-segment toggle flags
	sbrs	zl,TOG2seq			;bit 0/1 ?
	rjmp	_CHKin0Hexe2DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG2seq
	sts		key0TOG,zl
	jmp		_CHKin0HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin0Hexe2DO:
	;toggeln h
	sbr		zl,1<<TOG2seq
	sts		key0TOG,zl
	rjmp	_CHKin0HexeDO		;a/b/c instructions ausfhren

	;......................................................
	;check toggle status sequenz 3
	;......................................................
_CHKin0Hexe3TOG:
	lds		zl,key0TOG			;key-segment toggle flags
	sbrs	zl,TOG3seq			;bit 0/1 ?
	rjmp	_CHKin0Hexe3DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG3seq
	sts		key0TOG,zl
	jmp		_CHKin0HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin0Hexe3DO:
	;toggeln h
	sbr		zl,1<<TOG3seq
	sts		key0TOG,zl
	rjmp	_CHKin0HexeDO		;a/b/c instructions ausfhren






	;................................................................
	;1..3 instructions zu sequence + event ausfhren NO TOGGLE/FIRST
	;................................................................

_CHKin0HexeDO:	
	;welches systemevent in aktueller sequenz setzen ?
	lds		yh,key0SEQ			;1-3
	ldi		yl,0
	call	_RDkeyEVE			;event in yh
	sts		eventTE,yh			;
	call	_MAKEeventSUB		;submodul !
	;wieviel instructions in dieser sequenz ?
	;-> ausfhren !
	adiw	zh:zl,1
	ld		r18,z+		;instruction in yh
_CHKin0HexeDOloop:
	ld		yh,z+		;msb		
	ld		yl,z+		;lsb
	ld		r19,z+		;stimuli parameter
	st		y,r19		;&store
	;instruction -1
	dec		r18
	tst		r18
	breq	_CHKin0HexeREADYa
	jmp		_CHKin0HexeDOloop	
_CHKin0HexeREADYa:
	;sequence counter zurcksetzen !
	clr		yl
	sts		key0SEQ,yl
	ret		;ready


	;................................................................
	;1..3 instructions zu sequence + event ausfhren TOGGLE/SECOND
	;................................................................

_CHKin0HexeTOGDO:	
	;welches systemevent in aktueller sequenz setzen ?
	lds		yh,key0SEQ			;1-3
	ldi		yl,0
	call	_RDkeyEVEtog		;event in yh
	sts		eventTE,yh			;
	call	_MAKEeventSUB		;submodul !
	;wieviel instructions in dieser sequenz ?
	;-> ausfhren !
	adiw	zh:zl,1
	ld		r18,z+		;instruction in yh
_CHKin0HexeTOGDOloop:
	ld		yh,z+		;msb		
	ld		yl,z+		;lsb
	ld		r19,z+		;stimuli parameter
	st		y,r19		;&store
	;instruction -1
	dec		r18
	tst		r18
	breq	_CHKin0HexeREADYb
	jmp		_CHKin0HexeTOGDOloop	
_CHKin0HexeREADYb:
	;sequence counter zurcksetzen !
	clr		yl
	sts		key0SEQ,yl
	ret			;ready


;-----------------------------------------------------------------------------------
;eingang0= low
;-----------------------------------------------------------------------------------
;prfe ob's:
;a) eine h/l-flanke (edge-flag =1)
;oder
;b) ein l-level (edge-flag=0) ist
;INFO: edge-flag=1 wird bei ist-time >=wait-time  & eingang0= high, gesetzt
_CHKin0L:

	lds		yl,keyWTstate
	sbrc	r16,K0edge
	rjmp	_CHKin0Lhigh	;edge=1

	;wait state aus high prfen
	;timeout erreicht?
	;ja== keyseq bereits wieder 0 // k0edge 1

	sbrc	yl,K0wait		;wait state 1 == ex-timer aus Luft noch...
	rjmp	_CHKin0Lhigh	;edge=1 & wait state =1 (aktuelle vorbereitung zur ausfhrung abbrechen)
	jmp		_CHKin0Lback	;edge=0 & wait state=0...back !


	;...............................................................
	;edge=1 <=> next sequence
_CHKin0Lhigh:
	cbr		yl,1<<K0wait
	sts		keyWTstate,yl



	ldi		yl,0
	call	_RDkeySEQ	;defined sequences in yh
	lds		yl,key0SEQ	;aktuelle sequence
	cp		yl,yh
	brsh	_CHKin0LhighREADYa
	inc		yl
	jmp		_CHKin0LhighREADYb
_CHKin0LhighREADYa:
	clr		yl
_CHKin0LhighREADYb:
	sts		key0SEQ,yl
	;h/l-eventflag
	ldi		yl,161
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;
	cbr		r16,1<<K0edge
	;ist-counter=0
	clr		yl
	sts		key0CNT,yl	
_CHKin0Lback:
	ret


















;-----------------------------------------------------------------------------------
;eingang1= high
;-----------------------------------------------------------------------------------
;prfe ob's:
;a) eine l/h-flanke (edge-flag =0)
;oder
;b) ein h-level (edge-flag=1) ist
;INFO: edge-flag=1 wird bei ist-time >=wait-time gesetzt
_CHKin1H:
	sbrs	r16,K1edge
	rjmp	_CHKin1Hlow		;edge=0
	jmp		_CHKin1Hback	;edge=1 ...back !	
	;...............................................................
	;edge=0 <=> counter prfen...
_CHKin1Hlow:

	;wait state setzen
	lds		yl,keyWTstate
	sbr		yl,1<<K1wait
	sts		keyWTstate,yl

	;wait-counter soll (table) holen
	ldi		yl,1		;eingang0
	call	_RDkeyWAIT	;soll-parameter in yh
	;wait-counter ist holen
	lds		yl,key1CNT  ;ist- 8bit counter
	;vergleiche...
	cp		yl,yh			;ist|soll
	brsh 	_CHKin1Hexe		;ist-time >= wait-time == execute
	;warte
	;counter pro 100ms +1
	lds		yl,key1CNTte 	;temp.100ms counter
	lds		yh,_100ms		;100ms system counter
	cp		yl,yh
	brne	_CHKin1HlowTIME	;zeitkorrektur....
	jmp		_CHKin1Hback	;ohne aktion zurck...

	;................................................................
_CHKin1HlowTIME:
	;100ms ist-time temp. == 100ms systemcounter
	mov		yl,yh
	sts		key1CNTte,yl
	;wait time +1
	lds		yl,key1CNT  ;wait time- 8bit counter
	inc		yl
	sts		key1CNT,yl
	jmp		_CHKin1Hback	;zurck...


	;................................................................
	;execute sequence
	;................................................................	
	;ist-time >= wait-time == execute
_CHKin1Hexe:
	;l/h-eventflag
	ldi		yl,164
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;
	;edge-flag=1
	sbr		r16,1<<K1edge
	;ist-counter=0
	clr		yl
	sts		key1CNT,yl	;
	;soll-toggle-flags zu key-segment (toggeln ja/nein)
	ldi		yl,1
	call	_RDkeyTOG	;...in yh !
	;welche sequenz (im segment) wurde angewhlt ?
	;..geschah bei h/l flanke !!!
	lds		yl,key1SEQ	;1-3
	;sequenz 1
	cpi		yl,1
	brne	_CHKin1HexeA	
	;
	;---------------------
	;key1/sequ1-eventflag
	ldi		yl,144
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG1seq			;togglen erlaubt ?
	rjmp	_CHKin1HexeDO	;nein...
	jmp		_CHKin1Hexe1TOG		;...ja !
	;sequenz 2
_CHKin1HexeA:
	cpi		yl,2
	brne	_CHKin1HexeB
	;
	;---------------------
	;key1/sequ2-eventflag
	ldi		yl,145
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG2seq			;togglen erlaubt ?
	rjmp	_CHKin1HexeDO	;nein...
	jmp		_CHKin1Hexe2TOG		;...ja !
	;sequenz 3
_CHKin1HexeB:
	cpi		yl,3
	brne	_CHKin1HexeERR
	;
	;---------------------
	;key1/sequ3-eventflag
	ldi		yl,146
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG3seq			;togglen erlaubt ?
	rjmp	_CHKin1HexeDO	;nein...
	jmp		_CHKin1Hexe3TOG		;...ja !
	;>3 error
_CHKin1HexeERR:
	clr		yl
	sts		key1SEQ,yl
_CHKin1Hback:
	ret			;zurck...




	;......................................................
	;check toggle status sequenz 1
	;......................................................
_CHKin1Hexe1TOG:
	lds		zl,key1TOG			;key-segment toggle flags
	sbrs	zl,TOG1seq			;bit 0/1 ?
	rjmp	_CHKin1Hexe1DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG1seq
	sts		key1TOG,zl
	jmp		_CHKin1HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin1Hexe1DO:
	;toggeln h
	sbr		zl,1<<TOG1seq
	sts		key1TOG,zl
	rjmp	_CHKin1HexeDO		;a/b/c instructions ausfhren


	;......................................................
	;check toggle status sequenz 2
	;......................................................
_CHKin1Hexe2TOG:
	lds		zl,key1TOG			;key-segment toggle flags
	sbrs	zl,TOG2seq			;bit 0/1 ?
	rjmp	_CHKin1Hexe2DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG2seq
	sts		key1TOG,zl
	jmp		_CHKin1HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin1Hexe2DO:
	;toggeln h
	sbr		zl,1<<TOG2seq
	sts		key1TOG,zl
	rjmp	_CHKin1HexeDO		;a/b/c instructions ausfhren

	;......................................................
	;check toggle status sequenz 3
	;......................................................
_CHKin1Hexe3TOG:
	lds		zl,key1TOG			;key-segment toggle flags
	sbrs	zl,TOG3seq			;bit 0/1 ?
	rjmp	_CHKin1Hexe3DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG3seq
	sts		key1TOG,zl
	jmp		_CHKin1HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin1Hexe3DO:
	;toggeln h
	sbr		zl,1<<TOG3seq
	sts		key1TOG,zl
	rjmp	_CHKin1HexeDO		;a/b/c instructions ausfhren






	;................................................................
	;1..3 instructions zu sequence + event ausfhren NO TOGGLE/FIRST
	;................................................................

_CHKin1HexeDO:	
	;welches systemevent in aktueller sequenz setzen ?
	lds		yh,key1SEQ			;1-3
	ldi		yl,1
	call	_RDkeyEVE			;event in yh
	sts		eventTE,yh			;
	call	_MAKEeventSUB		;submodul !
	;wieviel instructions in dieser sequenz ?
	;-> ausfhren !
	adiw	zh:zl,1
	ld		r18,z+		;instruction in yh
_CHKin1HexeDOloop:
	ld		yh,z+		;msb		
	ld		yl,z+		;lsb
	ld		r19,z+		;stimuli parameter
	st		y,r19		;&store
	;instruction -1
	dec		r18
	tst		r18
	breq	_CHKin1HexeREADYa
	jmp		_CHKin1HexeDOloop	
_CHKin1HexeREADYa:
	;sequence counter zurcksetzen !
	clr		yl
	sts		key1SEQ,yl
	ret		;ready


	;................................................................
	;1..3 instructions zu sequence + event ausfhren TOGGLE/SECOND
	;................................................................

_CHKin1HexeTOGDO:	
	;welches systemevent in aktueller sequenz setzen ?
	lds		yh,key1SEQ			;1-3
	ldi		yl,1
	call	_RDkeyEVEtog		;event in yh
	sts		eventTE,yh			;
	call	_MAKEeventSUB		;submodul !
	;wieviel instructions in dieser sequenz ?
	;-> ausfhren !
	adiw	zh:zl,1
	ld		r18,z+		;instruction in yh
_CHKin1HexeTOGDOloop:
	ld		yh,z+		;msb		
	ld		yl,z+		;lsb
	ld		r19,z+		;stimuli parameter
	st		y,r19		;&store
	;instruction -1
	dec		r18
	tst		r18
	breq	_CHKin1HexeREADYb
	jmp		_CHKin1HexeTOGDOloop	
_CHKin1HexeREADYb:
	;sequence counter zurcksetzen !
	clr		yl
	sts		key1SEQ,yl
	ret			;ready


;-----------------------------------------------------------------------------------
;eingang1= low
;-----------------------------------------------------------------------------------
;prfe ob's:
;a) eine h/l-flanke (edge-flag =1)
;oder
;b) ein l-level (edge-flag=0) ist
;INFO: edge-flag=1 wird bei ist-time >=wait-time  & eingang0= high, gesetzt
_CHKin1L:

	lds		yl,keyWTstate
	sbrc	r16,K1edge
	rjmp	_CHKin1Lhigh	;edge=1

	;wait state aus high prfen
	;timeout erreicht?
	;ja== keyseq bereits wieder 0 // k0edge 1

	sbrc	yl,K1wait		;wait state 1 == ex-timer aus Luft noch...
	rjmp	_CHKin1Lhigh	;edge=1 & wait state =1 (aktuelle vorbereitung zur ausfhrung abbrechen)
	jmp		_CHKin1Lback	;edge=0 & wait state=0...back !


	;...............................................................
	;edge=1 <=> next sequence
_CHKin1Lhigh:
	cbr		yl,1<<K1wait
	sts		keyWTstate,yl

	ldi		yl,1
	call	_RDkeySEQ	;defined sequences in yh
	lds		yl,key1SEQ	;aktuelle sequence
	cp		yl,yh
	brsh	_CHKin1LhighREADYa
	inc		yl
	jmp		_CHKin1LhighREADYb
_CHKin1LhighREADYa:
	clr		yl
_CHKin1LhighREADYb:
	sts		key1SEQ,yl
	;h/l-eventflag
	ldi		yl,163
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;
	cbr		r16,1<<K1edge
	;ist-counter=0
	clr		yl
	sts		key1CNT,yl	
_CHKin1Lback:
	ret






;-----------------------------------------------------------------------------------
;eingang2= high
;-----------------------------------------------------------------------------------
;prfe ob's:
;a) eine l/h-flanke (edge-flag =0)
;oder
;b) ein h-level (edge-flag=1) ist
;INFO: edge-flag=1 wird bei ist-time >=wait-time gesetzt
_CHKin2H:
	sbrs	r16,K2edge
	rjmp	_CHKin2Hlow		;edge=0
	jmp		_CHKin2Hback	;edge=1 ...back !	
	;...............................................................
	;edge=0 <=> counter prfen...
_CHKin2Hlow:

	;wait state setzen
	lds		yl,keyWTstate
	sbr		yl,1<<K2wait
	sts		keyWTstate,yl
	
	;wait-counter soll (table) holen
	ldi		yl,2		;eingang2
	call	_RDkeyWAIT	;soll-parameter in yh
	;wait-counter ist holen
	lds		yl,key2CNT  ;ist- 8bit counter
	;vergleiche...
	cp		yl,yh			;ist|soll
	brsh 	_CHKin2Hexe		;ist-time >= wait-time == execute
	;warte
	;counter pro 100ms +1
	lds		yl,key2CNTte 	;temp.100ms counter
	lds		yh,_100ms		;100ms system counter
	cp		yl,yh
	brne	_CHKin2HlowTIME	;zeitkorrektur....
	jmp		_CHKin2Hback	;ohne aktion zurck...

	;................................................................
_CHKin2HlowTIME:
	;100ms ist-time temp. == 100ms systemcounter
	mov		yl,yh
	sts		key2CNTte,yl
	;wait time +1
	lds		yl,key2CNT  ;wait time- 8bit counter
	inc		yl
	sts		key2CNT,yl
	jmp		_CHKin2Hback	;zurck...


	;................................................................
	;execute sequence
	;................................................................	
	;ist-time >= wait-time == execute
_CHKin2Hexe:
	;l/h-eventflag
	ldi		yl,166
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;
	;edge-flag=1
	sbr		r16,1<<K2edge
	;ist-counter=0
	clr		yl
	sts		key2CNT,yl	;
	;soll-toggle-flags zu key-segment (toggeln ja/nein)
	ldi		yl,2
	call	_RDkeyTOG	;...in yh !
	;welche sequenz (im segment) wurde angewhlt ?
	;..geschah bei h/l flanke !!!
	lds		yl,key2SEQ	;1-3
	;sequenz 1
	cpi		yl,1
	brne	_CHKin2HexeA	
	;
	;---------------------
	;key2/sequ2-eventflag
	ldi		yl,147
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG1seq			;togglen erlaubt ?
	rjmp	_CHKin2HexeDO	;nein...
	jmp		_CHKin2Hexe1TOG		;...ja !
	;sequenz 2
_CHKin2HexeA:
	cpi		yl,2
	brne	_CHKin2HexeB
	;
	;---------------------
	;key2/sequ2-eventflag
	ldi		yl,148
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG2seq			;togglen erlaubt ?
	rjmp	_CHKin2HexeDO	;nein...
	jmp		_CHKin2Hexe2TOG		;...ja !
	;sequenz 3
_CHKin2HexeB:
	cpi		yl,3
	brne	_CHKin2HexeERR
	;
	;---------------------
	;key2/sequ3-eventflag
	ldi		yl,149
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG3seq			;togglen erlaubt ?
	rjmp	_CHKin2HexeDO	;nein...
	jmp		_CHKin2Hexe3TOG		;...ja !
	;>3 error
_CHKin2HexeERR:
	clr		yl
	sts		key2SEQ,yl
_CHKin2Hback:
	ret			;zurck...




	;......................................................
	;check toggle status sequenz 1
	;......................................................
_CHKin2Hexe1TOG:
	lds		zl,key2TOG			;key-segment toggle flags
	sbrs	zl,TOG1seq			;bit 0/1 ?
	rjmp	_CHKin2Hexe1DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG1seq
	sts		key2TOG,zl
	jmp		_CHKin2HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin2Hexe1DO:
	;toggeln h
	sbr		zl,1<<TOG1seq
	sts		key2TOG,zl
	rjmp	_CHKin2HexeDO		;a/b/c instructions ausfhren


	;......................................................
	;check toggle status sequenz 2
	;......................................................
_CHKin2Hexe2TOG:
	lds		zl,key2TOG			;key-segment toggle flags
	sbrs	zl,TOG2seq			;bit 0/1 ?
	rjmp	_CHKin2Hexe2DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG2seq
	sts		key2TOG,zl
	jmp		_CHKin2HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin2Hexe2DO:
	;toggeln h
	sbr		zl,1<<TOG2seq
	sts		key2TOG,zl
	rjmp	_CHKin2HexeDO		;a/b/c instructions ausfhren

	;......................................................
	;check toggle status sequenz 3
	;......................................................
_CHKin2Hexe3TOG:
	lds		zl,key2TOG			;key-segment toggle flags
	sbrs	zl,TOG3seq			;bit 0/1 ?
	rjmp	_CHKin2Hexe3DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG3seq
	sts		key2TOG,zl
	jmp		_CHKin2HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin2Hexe3DO:
	;toggeln h
	sbr		zl,1<<TOG3seq
	sts		key2TOG,zl
	rjmp	_CHKin2HexeDO		;a/b/c instructions ausfhren






	;................................................................
	;1..3 instructions zu sequence + event ausfhren NO TOGGLE/FIRST
	;................................................................

_CHKin2HexeDO:	
	;welches systemevent in aktueller sequenz setzen ?
	lds		yh,key2SEQ			;1-3
	ldi		yl,2
	call	_RDkeyEVE			;event in yh
	sts		eventTE,yh			;
	call	_MAKEeventSUB		;submodul !
	;wieviel instructions in dieser sequenz ?
	;-> ausfhren !
	adiw	zh:zl,1
	ld		r18,z+		;instruction in yh
_CHKin2HexeDOloop:
	ld		yh,z+		;msb		
	ld		yl,z+		;lsb
	ld		r19,z+		;stimuli parameter
	st		y,r19		;&store
	;instruction -1
	dec		r18
	tst		r18
	breq	_CHKin2HexeREADYa
	jmp		_CHKin2HexeDOloop	
_CHKin2HexeREADYa:
	;sequence counter zurcksetzen !
	clr		yl
	sts		key2SEQ,yl
	ret		;ready


	;................................................................
	;1..3 instructions zu sequence + event ausfhren TOGGLE/SECOND
	;................................................................

_CHKin2HexeTOGDO:	
	;welches systemevent in aktueller sequenz setzen ?
	lds		yh,key2SEQ			;1-3
	ldi		yl,2
	call	_RDkeyEVEtog		;event in yh
	sts		eventTE,yh			;
	call	_MAKEeventSUB		;submodul !
	;wieviel instructions in dieser sequenz ?
	;-> ausfhren !
	adiw	zh:zl,1
	ld		r18,z+		;instruction in yh
_CHKin2HexeTOGDOloop:
	ld		yh,z+		;msb		
	ld		yl,z+		;lsb
	ld		r19,z+		;stimuli parameter
	st		y,r19		;&store
	;instruction -1
	dec		r18
	tst		r18
	breq	_CHKin2HexeREADYb
	jmp		_CHKin2HexeTOGDOloop	
_CHKin2HexeREADYb:
	;sequence counter zurcksetzen !
	clr		yl
	sts		key2SEQ,yl
	ret			;ready


;-----------------------------------------------------------------------------------
;eingang2= low
;-----------------------------------------------------------------------------------
;prfe ob's:
;a) eine h/l-flanke (edge-flag =1)
;oder
;b) ein l-level (edge-flag=0) ist
;INFO: edge-flag=1 wird bei ist-time >=wait-time  & eingang0= high, gesetzt
_CHKin2L:

	lds		yl,keyWTstate
	sbrc	r16,K2edge
	rjmp	_CHKin2Lhigh	;edge=1

	;wait state aus high prfen
	;timeout erreicht?
	;ja== keyseq bereits wieder 0 // k0edge 1

	sbrc	yl,K2wait		;wait state 1 == ex-timer aus Luft noch...
	rjmp	_CHKin2Lhigh	;edge=1 & wait state =1 (aktuelle vorbereitung zur ausfhrung abbrechen)
	jmp		_CHKin2Lback	;edge=0 & wait state=0...back !


	;...............................................................
	;edge=1 <=> next sequence
_CHKin2Lhigh:
	cbr		yl,1<<K2wait
	sts		keyWTstate,yl

	ldi		yl,2
	call	_RDkeySEQ	;defined sequences in yh
	lds		yl,key2SEQ	;aktuelle sequence
	cp		yl,yh
	brsh	_CHKin2LhighREADYa
	inc		yl
	jmp		_CHKin2LhighREADYb
_CHKin2LhighREADYa:
	clr		yl
_CHKin2LhighREADYb:
	sts		key2SEQ,yl
	;h/l-eventflag
	ldi		yl,165
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;
	cbr		r16,1<<K2edge
	;ist-counter=0
	clr		yl
	sts		key2CNT,yl	
_CHKin2Lback:
	ret




;-----------------------------------------------------------------------------------
;eingang3= high
;-----------------------------------------------------------------------------------
;prfe ob's:
;a) eine l/h-flanke (edge-flag =0)
;oder
;b) ein h-level (edge-flag=1) ist
;INFO: edge-flag=1 wird bei ist-time >=wait-time gesetzt
_CHKin3H:
	sbrs	r16,K3edge
	rjmp	_CHKin3Hlow		;edge=0
	jmp		_CHKin3Hback	;edge=1 ...back !	
	;...............................................................
	;edge=0 <=> counter prfen...
_CHKin3Hlow:

	;wait state setzen
	lds		yl,keyWTstate
	sbr		yl,1<<K3wait
	sts		keyWTstate,yl

	;wait-counter soll (table) holen
	ldi		yl,3		;eingang3
	call	_RDkeyWAIT	;soll-parameter in yh
	;wait-counter ist holen
	lds		yl,key3CNT  ;ist- 8bit counter
	;vergleiche...
	cp		yl,yh			;ist|soll
	brsh 	_CHKin3Hexe		;ist-time >= wait-time == execute
	;warte
	;counter pro 100ms +1
	lds		yl,key3CNTte 	;temp.100ms counter
	lds		yh,_100ms		;100ms system counter
	cp		yl,yh
	brne	_CHKin3HlowTIME	;zeitkorrektur....
	jmp		_CHKin3Hback	;ohne aktion zurck...

	;................................................................
_CHKin3HlowTIME:
	;100ms ist-time temp. == 100ms systemcounter
	mov		yl,yh
	sts		key3CNTte,yl
	;wait time +1
	lds		yl,key3CNT  ;wait time- 8bit counter
	inc		yl
	sts		key3CNT,yl
	jmp		_CHKin3Hback	;zurck...


	;................................................................
	;execute sequence
	;................................................................	
	;ist-time >= wait-time == execute
_CHKin3Hexe:
	;l/h-eventflag
	ldi		yl,168
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;
	;edge-flag=1
	sbr		r16,1<<K3edge
	;ist-counter=0
	clr		yl
	sts		key3CNT,yl	;
	;soll-toggle-flags zu key-segment (toggeln ja/nein)
	ldi		yl,3
	call	_RDkeyTOG	;...in yh !
	;welche sequenz (im segment) wurde angewhlt ?
	;..geschah bei h/l flanke !!!
	lds		yl,key3SEQ	;1-3
	;sequenz 1
	cpi		yl,1
	brne	_CHKin3HexeA	
	;
	;---------------------
	;key3/sequ1-eventflag
	ldi		yl,150
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG1seq			;togglen erlaubt ?
	rjmp	_CHKin3HexeDO	;nein...
	jmp		_CHKin3Hexe1TOG		;...ja !
	;sequenz 2
_CHKin3HexeA:
	cpi		yl,2
	brne	_CHKin3HexeB
	;
	;---------------------
	;key3/sequ2-eventflag
	ldi		yl,151
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG2seq			;togglen erlaubt ?
	rjmp	_CHKin3HexeDO	;nein...
	jmp		_CHKin3Hexe2TOG		;...ja !
	;sequenz 3
_CHKin3HexeB:
	cpi		yl,3
	brne	_CHKin3HexeERR
	;
	;---------------------
	;key3/sequ3-eventflag
	ldi		yl,152
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG3seq			;togglen erlaubt ?
	rjmp	_CHKin3HexeDO	;nein...
	jmp		_CHKin3Hexe3TOG		;...ja !
	;>3 error
_CHKin3HexeERR:
	clr		yl
	sts		key3SEQ,yl
_CHKin3Hback:
	ret			;zurck...




	;......................................................
	;check toggle status sequenz 1
	;......................................................
_CHKin3Hexe1TOG:
	lds		zl,key3TOG			;key-segment toggle flags
	sbrs	zl,TOG1seq			;bit 0/1 ?
	rjmp	_CHKin3Hexe1DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG1seq
	sts		key3TOG,zl
	jmp		_CHKin3HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin3Hexe1DO:
	;toggeln h
	sbr		zl,1<<TOG1seq
	sts		key3TOG,zl
	rjmp	_CHKin3HexeDO		;a/b/c instructions ausfhren


	;......................................................
	;check toggle status sequenz 2
	;......................................................
_CHKin3Hexe2TOG:
	lds		zl,key3TOG			;key-segment toggle flags
	sbrs	zl,TOG2seq			;bit 0/1 ?
	rjmp	_CHKin3Hexe2DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG2seq
	sts		key3TOG,zl
	jmp		_CHKin3HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin3Hexe2DO:
	;toggeln h
	sbr		zl,1<<TOG2seq
	sts		key3TOG,zl
	rjmp	_CHKin3HexeDO		;a/b/c instructions ausfhren

	;......................................................
	;check toggle status sequenz 3
	;......................................................
_CHKin3Hexe3TOG:
	lds		zl,key3TOG			;key-segment toggle flags
	sbrs	zl,TOG3seq			;bit 0/1 ?
	rjmp	_CHKin3Hexe3DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG3seq
	sts		key3TOG,zl
	jmp		_CHKin3HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin3Hexe3DO:
	;toggeln h
	sbr		zl,1<<TOG3seq
	sts		key3TOG,zl
	rjmp	_CHKin3HexeDO		;a/b/c instructions ausfhren






	;................................................................
	;1..3 instructions zu sequence + event ausfhren NO TOGGLE/FIRST
	;................................................................

_CHKin3HexeDO:	
	;welches systemevent in aktueller sequenz setzen ?
	lds		yh,key3SEQ			;1-3
	ldi		yl,3
	call	_RDkeyEVE			;event in yh
	sts		eventTE,yh			;
	call	_MAKEeventSUB		;submodul !
	;wieviel instructions in dieser sequenz ?
	;-> ausfhren !
	adiw	zh:zl,1
	ld		r18,z+		;instruction in yh
_CHKin3HexeDOloop:
	ld		yh,z+		;msb		
	ld		yl,z+		;lsb
	ld		r19,z+		;stimuli parameter
	st		y,r19		;&store
	;instruction -1
	dec		r18
	tst		r18
	breq	_CHKin3HexeREADYa
	jmp		_CHKin3HexeDOloop	
_CHKin3HexeREADYa:
	;sequence counter zurcksetzen !
	clr		yl
	sts		key3SEQ,yl
	ret		;ready


	;................................................................
	;1..3 instructions zu sequence + event ausfhren TOGGLE/SECOND
	;................................................................

_CHKin3HexeTOGDO:	
	;welches systemevent in aktueller sequenz setzen ?
	lds		yh,key3SEQ			;1-3
	ldi		yl,3
	call	_RDkeyEVEtog		;event in yh
	sts		eventTE,yh			;
	call	_MAKEeventSUB		;submodul !
	;wieviel instructions in dieser sequenz ?
	;-> ausfhren !
	adiw	zh:zl,1
	ld		r18,z+		;instruction in yh
_CHKin3HexeTOGDOloop:
	ld		yh,z+		;msb		
	ld		yl,z+		;lsb
	ld		r19,z+		;stimuli parameter
	st		y,r19		;&store
	;instruction -1
	dec		r18
	tst		r18
	breq	_CHKin3HexeREADYb
	jmp		_CHKin3HexeTOGDOloop	
_CHKin3HexeREADYb:
	;sequence counter zurcksetzen !
	clr		yl
	sts		key3SEQ,yl
	ret			;ready


;-----------------------------------------------------------------------------------
;eingang3= low
;-----------------------------------------------------------------------------------
;prfe ob's:
;a) eine h/l-flanke (edge-flag =1)
;oder
;b) ein l-level (edge-flag=0) ist
;INFO: edge-flag=1 wird bei ist-time >=wait-time  & eingang0= high, gesetzt
_CHKin3L:

	lds		yl,keyWTstate
	sbrc	r16,K3edge
	rjmp	_CHKin3Lhigh	;edge=1

	;wait state aus high prfen
	;timeout erreicht?
	;ja== keyseq bereits wieder 0 // k0edge 1

	sbrc	yl,K3wait		;wait state 1 == ex-timer aus Luft noch...
	rjmp	_CHKin3Lhigh	;edge=1 & wait state =1 (aktuelle vorbereitung zur ausfhrung abbrechen)
	jmp		_CHKin3Lback	;edge=0 & wait state=0...back !


	;...............................................................
	;edge=1 <=> next sequence
_CHKin3Lhigh:
	cbr		yl,1<<K3wait
	sts		keyWTstate,yl

	ldi		yl,3
	call	_RDkeySEQ	;defined sequences in yh
	lds		yl,key3SEQ	;aktuelle sequence
	cp		yl,yh
	brsh	_CHKin3LhighREADYa
	inc		yl
	jmp		_CHKin3LhighREADYb
_CHKin3LhighREADYa:
	clr		yl
_CHKin3LhighREADYb:
	sts		key3SEQ,yl
	;h/l-eventflag
	ldi		yl,167
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;
	cbr		r16,1<<K3edge
	;ist-counter=0
	clr		yl
	sts		key3CNT,yl	
_CHKin3Lback:
	ret




;-----------------------------------------------------------------------------------
;eingang4= high
;-----------------------------------------------------------------------------------
;prfe ob's:
;a) eine l/h-flanke (edge-flag =0)
;oder
;b) ein h-level (edge-flag=1) ist
;INFO: edge-flag=1 wird bei ist-time >=wait-time gesetzt
_CHKin4H:
	sbrs	r16,K4edge
	rjmp	_CHKin4Hlow		;edge=0
	jmp		_CHKin4Hback	;edge=1 ...back !	
	;...............................................................
	;edge=0 <=> counter prfen...
_CHKin4Hlow:

	;wait state setzen
	lds		yl,keyWTstate
	sbr		yl,1<<K4wait
	sts		keyWTstate,yl

	;wait-counter soll (table) holen
	ldi		yl,4		;eingang3
	call	_RDkeyWAIT	;soll-parameter in yh
	;wait-counter ist holen
	lds		yl,key4CNT  ;ist- 8bit counter
	;vergleiche...
	cp		yl,yh			;ist|soll
	brsh 	_CHKin4Hexe		;ist-time >= wait-time == execute
	;warte
	;counter pro 100ms +1
	lds		yl,key4CNTte 	;temp.100ms counter
	lds		yh,_100ms		;100ms system counter
	cp		yl,yh
	brne	_CHKin4HlowTIME	;zeitkorrektur....
	jmp		_CHKin4Hback	;ohne aktion zurck...

	;................................................................
_CHKin4HlowTIME:
	;100ms ist-time temp. == 100ms systemcounter
	mov		yl,yh
	sts		key4CNTte,yl
	;wait time +1
	lds		yl,key4CNT  ;wait time- 8bit counter
	inc		yl
	sts		key4CNT,yl
	jmp		_CHKin4Hback	;zurck...


	;................................................................
	;execute sequence
	;................................................................	
	;ist-time >= wait-time == execute
_CHKin4Hexe:
	;l/h-eventflag
	ldi		yl,170
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;
	;edge-flag=1
	sbr		r16,1<<K4edge
	;ist-counter=0
	clr		yl
	sts		key4CNT,yl	;
	;soll-toggle-flags zu key-segment (toggeln ja/nein)
	ldi		yl,4
	call	_RDkeyTOG	;...in yh !
	;welche sequenz (im segment) wurde angewhlt ?
	;..geschah bei h/l flanke !!!
	lds		yl,key4SEQ	;1-3
	;sequenz 1
	cpi		yl,1
	brne	_CHKin4HexeA
	;
	;---------------------
	;key4/sequ1-eventflag
	ldi		yl,153
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG1seq			;togglen erlaubt ?
	rjmp	_CHKin4HexeDO	;nein...
	jmp		_CHKin4Hexe1TOG		;...ja !
	;sequenz 2
_CHKin4HexeA:
	cpi		yl,2
	brne	_CHKin4HexeB
	;
	;---------------------
	;key4/sequ2-eventflag
	ldi		yl,154
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG2seq			;togglen erlaubt ?
	rjmp	_CHKin4HexeDO	;nein...
	jmp		_CHKin4Hexe2TOG		;...ja !
	;sequenz 3
_CHKin4HexeB:
	cpi		yl,3
	brne	_CHKin4HexeERR
	;
	;---------------------
	;key4/sequ3-eventflag
	ldi		yl,155
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;---------------------
	;	
	sbrs	yh,TOG3seq			;togglen erlaubt ?
	rjmp	_CHKin4HexeDO	;nein...
	jmp		_CHKin4Hexe3TOG		;...ja !
	;>3 error
_CHKin4HexeERR:
	clr		yl
	sts		key4SEQ,yl
_CHKin4Hback:
	ret			;zurck...




	;......................................................
	;check toggle status sequenz 1
	;......................................................
_CHKin4Hexe1TOG:
	lds		zl,key4TOG			;key-segment toggle flags
	sbrs	zl,TOG1seq			;bit 0/1 ?
	rjmp	_CHKin4Hexe1DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG1seq
	sts		key4TOG,zl
	jmp		_CHKin4HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin4Hexe1DO:
	;toggeln h
	sbr		zl,1<<TOG1seq
	sts		key4TOG,zl
	rjmp	_CHKin4HexeDO		;a/b/c instructions ausfhren


	;......................................................
	;check toggle status sequenz 2
	;......................................................
_CHKin4Hexe2TOG:
	lds		zl,key4TOG			;key-segment toggle flags
	sbrs	zl,TOG2seq			;bit 0/1 ?
	rjmp	_CHKin4Hexe2DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG2seq
	sts		key4TOG,zl
	jmp		_CHKin4HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin4Hexe2DO:
	;toggeln h
	sbr		zl,1<<TOG2seq
	sts		key4TOG,zl
	rjmp	_CHKin4HexeDO		;a/b/c instructions ausfhren

	;......................................................
	;check toggle status sequenz 3
	;......................................................
_CHKin4Hexe3TOG:
	lds		zl,key4TOG			;key-segment toggle flags
	sbrs	zl,TOG3seq			;bit 0/1 ?
	rjmp	_CHKin4Hexe3DO		;a/b/c instructions ausfhren
	;toggeln l
	cbr		zl,1<<TOG3seq
	sts		key4TOG,zl
	jmp		_CHKin4HexeTOGDO   	;TOG-a/b/c- instructions zu sequence ausfhren	
_CHKin4Hexe3DO:
	;toggeln h
	sbr		zl,1<<TOG3seq
	sts		key4TOG,zl
	rjmp	_CHKin4HexeDO		;a/b/c instructions ausfhren






	;................................................................
	;1..3 instructions zu sequence + event ausfhren NO TOGGLE/FIRST
	;................................................................

_CHKin4HexeDO:	
	;welches systemevent in aktueller sequenz setzen ?
	lds		yh,key4SEQ			;1-3
	ldi		yl,4
	call	_RDkeyEVE			;event in yh
	sts		eventTE,yh			;
	call	_MAKEeventSUB		;submodul !
	;wieviel instructions in dieser sequenz ?
	;-> ausfhren !
	adiw	zh:zl,1
	ld		r18,z+		;instruction in yh
_CHKin4HexeDOloop:
	ld		yh,z+		;msb		
	ld		yl,z+		;lsb
	ld		r19,z+		;stimuli parameter
	st		y,r19		;&store
	;instruction -1
	dec		r18
	tst		r18
	breq	_CHKin4HexeREADYa
	jmp		_CHKin4HexeDOloop	
_CHKin4HexeREADYa:
	;sequence counter zurcksetzen !
	clr		yl
	sts		key4SEQ,yl
	ret		;ready


	;................................................................
	;1..3 instructions zu sequence + event ausfhren TOGGLE/SECOND
	;................................................................

_CHKin4HexeTOGDO:	
	;welches systemevent in aktueller sequenz setzen ?
	lds		yh,key4SEQ			;1-3
	ldi		yl,4
	call	_RDkeyEVEtog		;event in yh
	sts		eventTE,yh			;
	call	_MAKEeventSUB		;submodul !
	;wieviel instructions in dieser sequenz ?
	;-> ausfhren !
	adiw	zh:zl,1
	ld		r18,z+		;instruction in yh
_CHKin4HexeTOGDOloop:
	ld		yh,z+		;msb		
	ld		yl,z+		;lsb
	ld		r19,z+		;stimuli parameter
	st		y,r19		;&store
	;instruction -1
	dec		r18
	tst		r18
	breq	_CHKin4HexeREADYb
	jmp		_CHKin4HexeTOGDOloop	
_CHKin4HexeREADYb:
	;sequence counter zurcksetzen !
	clr		yl
	sts		key4SEQ,yl
	ret			;ready


;-----------------------------------------------------------------------------------
;eingang4= low
;-----------------------------------------------------------------------------------
;prfe ob's:
;a) eine h/l-flanke (edge-flag =1)
;oder
;b) ein l-level (edge-flag=0) ist
;INFO: edge-flag=1 wird bei ist-time >=wait-time  & eingang0= high, gesetzt
_CHKin4L:
	
	lds		yl,keyWTstate
	sbrc	r16,K4edge
	rjmp	_CHKin4Lhigh	;edge=1

	;wait state aus high prfen
	;timeout erreicht?
	;ja== keyseq bereits wieder 0 // k0edge 1

	sbrc	yl,K4wait		;wait state 1 == ex-timer aus Luft noch...
	rjmp	_CHKin4Lhigh	;edge=1 & wait state =1 (aktuelle vorbereitung zur ausfhrung abbrechen)
	jmp		_CHKin4Lback	;edge=0 & wait state=0...back !


	;...............................................................
	;edge=1 <=> next sequence
_CHKin4Lhigh:
	cbr		yl,1<<K4wait
	sts		keyWTstate,yl

	ldi		yl,4
	call	_RDkeySEQ	;defined sequences in yh
	lds		yl,key4SEQ	;aktuelle sequence
	cp		yl,yh
	brsh	_CHKin4LhighREADYa
	inc		yl
	jmp		_CHKin4LhighREADYb
_CHKin4LhighREADYa:
	clr		yl
_CHKin4LhighREADYb:
	sts		key4SEQ,yl
	;h/l-eventflag
	ldi		yl,169
	sts		eventTE,yl			;
	call	_MAKEeventSUB		;submodul !
	;
	cbr		r16,1<<K4edge
	;ist-counter=0
	clr		yl
	sts		key4CNT,yl	
_CHKin4Lback:
	ret

















;----------------------------------------------------------------------------------------------------------------------------------------------------------------
;1) TABELLENADRESSIERUNG:
;a) <sequence to key> (Byte0) im jeweiligen key-segment adressieren:
;   = keyTBLstart + (key_segment[0..4])*keySEQ_full[69])
;
;b) <system event> zur jeweiligen sequence adressieren:
;   = <sequence to key> + sequence-header[3] + ((sequence[1..3]-1)* <keySEQ_size[22]>
;
;c) <system event>TOG zur jeweiligen sequence adressieren:
;	= <system event>+ <togSIZ[11]>
;
;d) <execute-adress 1..3> zur jeweiligen instruction adressieren:
;   = <system event> + 2 + ((instruction[1..3]-1) *3)
;
;e) <execute-adress 1..3TOG> zur jeweiligen instruction adressieren:
;	= <execute-adress 1..3> + <togSIZ[11]>
;
;
;2) KONSTANTEN:
;.equ	keyTBLstart		=	0x...	;key-table-startadresse - bestandteil memMAP !!!
;.equ	keySEQ_size		=	22		;eine key-sequence umfasst 22bytes
;.equ	keyTOG_size		=	keySEQ_size/2	;11
;.equ	keySEQ_full		=	69		;gre eines kompletten key-segmentes

;3)VARIABLEN:
;.equ	keyEDGE			=	0x...	;l/h-flanken flag's fr alle eingnge				- bestandteil memMAP !!!
;->
;.equ	key0TOG 		=	0x...	;toggle-status-flags fr sequence 1..3 / eingang0 	- bestandteil memMAP !!!
;.equ	key1TOG 		=	0x...	;toggle-status-flags fr sequence 1..3 / eingang1 	- bestandteil memMAP !!!
;.equ	key2TOG 		=	0x...	;toggle-status-flags fr sequence 1..3 / eingang2 	- bestandteil memMAP !!!
;.equ	key3TOG 		=	0x...	;toggle-status-flags fr sequence 1..3 / eingang3 	- bestandteil memMAP !!!
;.equ	key4TOG 		=	0x...	;toggle-status-flags fr sequence 1..3 / eingang4 	- bestandteil memMAP !!!
;->
;.equ	key0CNT 		=	0x...	;wait-time counter fr eingang0 	- bestandteil memMAP !!!
;.equ	key1CNT 		=	0x...	;wait-time counter fr eingang1 	- bestandteil memMAP !!!
;.equ	key2CNT 		=	0x...	;wait-time counter fr eingang2 	- bestandteil memMAP !!!
;.equ	key3CNT 		=	0x...	;wait-time counter fr eingang3 	- bestandteil memMAP !!!
;.equ	key4CNT 		=	0x...	;wait-time counter fr eingang4 	- bestandteil memMAP !!!
;->
;.equ	key0CNTte 		=	0x...	;temp. ist-time counter fr eingang0 	- bestandteil memMAP !!!
;.equ	key1CNTte 		=	0x...	;temp. ist-time counter fr eingang1 	- bestandteil memMAP !!!
;.equ	key2CNTte 		=	0x...	;temp. ist-time counter fr eingang2 	- bestandteil memMAP !!!
;.equ	key3CNTte 		=	0x...	;temp. ist-time counter fr eingang3 	- bestandteil memMAP !!!
;.equ	key4CNTte 		=	0x...	;temp. ist-time counter fr eingang4 	- bestandteil memMAP !!!

;.equ	key0SEQ			=	0x...	;aktive sequenz(1..3) fr eingang0	- bestandteil memMAP !!!
;.equ	key1SEQ			=	0x...	;aktive sequenz(1..3) fr eingang1	- bestandteil memMAP !!!
;.equ	key2SEQ			=	0x...	;aktive sequenz(1..3) fr eingang2	- bestandteil memMAP !!!
;.equ	key3SEQ			=	0x...	;aktive sequenz(1..3) fr eingang3	- bestandteil memMAP !!!
;.equ	key4SEQ			=	0x...	;aktive sequenz(1..3) fr eingang4	- bestandteil memMAP !!!
;->
;.equ	TOG1seq			=	0
;.equ	TOG2seq			=	1
;.equ	TOG3seq			=	2
;

; ______________________________________________________________________________________________________________________________________________________________________________________________
;|Anzahl der |Wait     |Toggle     |System |Anzahl      |Execute |Stimuli  |Execute |Stimuli  |Execute |Stimuli  |System  |Anzahl      |Execute |Stimuli  |Execute |Stimuli  |Execute |Stimuli  |....         ...|
;|Sequenzen  |Parameter|Instruction|Event  |Instructions|Adresse |Parameter|Adresse |Parameter|Adresse |Parameter|Evevent |Instructions|Adresse |Parameter|Adresse |Parameter|Adresse |Parameter|
;| Zu keyX	 |         |Sequ 1..3  |  (1)  | (1)        |(1)a    |   (1)a  | (1)b   |  (1)b   |  (1)c  |  (1)c   |(1)TOG  | (1)TOG     |(1)aTOG |(1)aTOG  |(1)bTOG |(1)bTOG  |(1)cTOG |(1)cTOG  |
;|___________|_________|___________|_______|____________|________|_________|________|_________|________|_________|________|____________|________|_________|________|_________|________|_________|
;|   1...3   | 1...255 |   bit0..2 |1...255|    1...3   |  16bit |   8bit  |  16bit |  8bit   |  16bit |   8bit  |1...255 |   1...3    |  16bit | 8bit    |  16bit |   8bit  |  16bit |  8bit   |....         ...|
;|___________|_________|___________|_______|____________|________|_________|________|_________|________|_________|________|____________|________|_________|________|_________|________|_________|
;
;|<--------Sequence-Header ------->|<----------------------------------------------------Sequence1--------------------------------------------------------------------------------------------->|<....full size->|
;|              3Bytes             |                                                      22Bytes                                                                                               |3*sequence(22)  |
;|								   |																																	                        |+header(3)=69   |
;
;----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;ADRESSING-SUBs
;----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;
;A)
;READ SEQUENCE TO KEY PARAMETER 
;-> anzahl der vordefinierten befehlssequenzen zu eingang0...4
;input:   yl = eingang0...4 (key_segment)
;output:  yh 
_RDkeySEQ:
	ldi		zl,keySEQ_full			;69
	mul		yl,zl					;*0..4
	movw	zh:zl,r1:r0
	subi	zl,low(-keyTBLstart)	;+ sram-tabellen start
	sbci	zh,high(-keyTBLstart)
	ld		yh,z
	ret

;B)
;READ WAIT PARAMETER 
;-> anzahl der vordefinierten befehlssequenzen zu eingang0...4
;input:   yl = eingang0...4 (key_segment)
;output:  yh 
_RDkeyWAIT:
	ldi		zl,keySEQ_full			;69
	mul		yl,zl					;*0..4
	movw	zh:zl,r1:r0
	subi	zl,low(-keyTBLstart)	;+ sram-tabellen start
	sbci	zh,high(-keyTBLstart)
	adiw	zh:zl,1					;+1
	ld		yh,z
	ret

;C)
;READ TOGGLE FLAG 
;-> anzahl der vordefinierten befehlssequenzen zu eingang0...4
;input:   yl = eingang0...4 (key_segment)
;output:  yh 
_RDkeyTOG:
	ldi		zl,keySEQ_full			;69
	mul		yl,zl					;*0..4
	movw	zh:zl,r1:r0
	subi	zl,low(-keyTBLstart)	;+ sram-tabellen start
	sbci	zh,high(-keyTBLstart)
	adiw	zh:zl,2					;+2
	ld		yh,z
	ret	


;D)
;READ SEQUENCE EVENT 
;-> anzahl der vordefinierten befehlssequenzen zu eingang0...4
;input:   yl = eingang0...4 (key_segment)
;		  yh = sequence 1..3
;output:  yh 
_RDkeyEVE:
	subi	yh,1					;sequence-1
	ldi		zh,keySEQ_size			;22
	mul		yh,zh					;* sequence
	mov		yh,r0					;=yh
	;
	ldi		zl,keySEQ_full			;69
	mul		yl,zl					;*0..4
	movw	zh:zl,r1:r0
	subi	zl,low(-keyTBLstart)	;+ sram-tabellen start
	sbci	zh,high(-keyTBLstart)
	adiw	zh:zl,3					;+3
	clr		yl
	add		zl,yh
	adc		zh,yl
	ld		yh,z
	ret	

;D)
;READ SEQUENCE TOGGLED-EVENT 
;-> anzahl der vordefinierten befehlssequenzen zu eingang0...4
;input:   yl = eingang0...4 (key_segment)
;		  yh = sequence 1..3
;output:  yh 
_RDkeyEVEtog:
	subi	yh,1					;sequence-1
	ldi		zh,keySEQ_size			;22
	mul		yh,zh					;* sequence
	mov		yh,r0					;=yh
	;
	ldi		zl,keySEQ_full			;69
	mul		yl,zl					;*0..4
	movw	zh:zl,r1:r0
	subi	zl,low(-keyTBLstart)	;+ sram-tabellen start
	sbci	zh,high(-keyTBLstart)
	adiw	zh:zl,3					;+3
	clr		yl
	add		zl,yh					;+yh
	adc		zh,yl
	ldi		yh,keyTOG_size
	add		zl,yh					;+keTOG_size
	adc		zh,yl
	ld		yh,z
	ret	


;--------------------------------------------------------------------------------------
;...back
_key999SEQ:
	;edge status sichern
	sts		keyEDGE,r16
	ret
