;-------------------------------------------------------------------------------------
;PHYSICAL LAYER / _physLAYER.asm
;-------------------------------------------------------------------------------------

;bitte beachten - interruptsteuerung nun integriert.
;...d.h. die jeweiligen flag's mssen durch die software (de-,)aktiviert
;werden!!!!

;KURZBESCHREIBUNG:
;routine wird einmal pro timeslot(55,5s) aufgerufen um folgende aktionen
;auszufhren:---> - serial bus scannen bis daten eintreffen oder sendeanforderung eingeht
;		  	 ---> - senden nur wenn Interface not busy condition gegeben ist!!! 
;	      	      - gepufferte daten senden(max.framesize 255byte)
;	      	      - daten aus fifo asynchron empfangen und puffern (max.framesize 255byte)
;			 ---> - aktiver empfang wenn host-adressen identisch, oder broadcast!
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
;->SOFTWARE-FLAG-VARIABLEN
;--------------------------------------------------------------------------------
;--------------------------------------------------------------------------------
;SEflagA <=> serial data communication flag variable
;********Flag's in SEflagA: 
;.equ	SEbusy  	=	7	;<=> activ data communication
;.equ	RXframe 	=	4	;<=> new received data frame in buffer
;.equ	TXframe 	=	3	;<=> new data frame to transmit
;.equ	SEdir   	=	2	;<=> rx/tx ?
;.equ	SEerror 	=	1	;<=> frame error
;.equ	SEenabled	=	0	;<=> serial IF (dis-)enabled


;SEflagB <=> serial data communication flag variable
;********Flag's in SEflagB: 
;.equ	newTX		=	2	;<=> signaliere new-data-frame-TX 
;.equ	reqTX		=	1	;<=> "       "  request-data-tx
;.equ	BRcast		=	0	;<=> broadcast frame received

;SEflagC <=> serial data communication flag variable
;********Flag's in SEflagC:
;.equ	FRbuild		=	6	;<=> framebuilder modifiert gerade rxbuffer
;.equ	SERcomACT	=	5	;<=> allgemeine sercom aktivitt die den rxbuffer belegt...
;---------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
;->TRANSFER-VARIABLEN (SRAM)
;----------------------------------------------------------------------------------
;----------------------------------------------------------------------------------
;
;RXbufferST 	        <=> buffer start adress high 16-bit
;CNTbyte     	        <=> byte counter adress high 8-bit
;net_sou_adress     	<=> host adress              8-bit
;-------------------------------------------------------------------------------------------
;->ENTRY
;-------------------------------------------------------------------------------------------

_uart_transfer_entry:
	lds	r20,CNTbyte
	lds	r19,SEflagVARa		;hole SEflag Variable A
	lds	r22,SEflagVARb		;"  " " " " " " "	  B
	lds	r21,SEflagVARc		;  "            "     C
	;	
	;
	;zuerst prfen ob serielle if enabled (...via program - mode )
	sbrs	r19,SEenabled
	rjmp		GO_HOMEx


	;prfe ob sercom gerade rxbuffer belegt !
	sbrc	r21,SERcomACT	
	rjmp		GO_HOMEx
	;
	;verarbeitung luft - also momentan taub !!!
	sbrc	r19,RXframe
	rjmp		GO_HOMEx
	

;------------------------------------
;- luft da eine bertragung ?
;- oder neue Daten im RX-buffer  ?
;------------------------------------
	sbrc	r19,SEbusy			;<1> datacommunication activ
	rjmp	_uart_direction_check
	lds		r16,fifoVAR			;<0> inaktiv !
	sbrs	r16,fifoNEW
	rjmp	_serNOTbsy			;da luft noch nichts - ODER vielleicht doch eine Sendeanforderung?
	;daten eingetroffen == timeout sequenz resetten 
	sbr	r19,1<<SEbusy
	cbr	r19,1<<SEdir
	clr	r20
	rjmp	_uart_direction_check


;-------------------------------------------------------------------------------------------
;-> AKTIVER DATENTRANSFER ANGEFORDERT ?
;-------------------------------------------------------------------------------------------
GO_HOMEz:
	jmp	GO_HOME
_serNOTbsy:
	sbrs	r19,TXframe
	rjmp		GO_HOMEz					
;--------------------------
;Interface bereit - senden!
;--------------------------
_serTXchkGO:
	sbr	r19,1<<SEbusy		;heit das ein neues frame gesendet
	sbr	r19,1<<SEdir		;werden soll und die flag's und byte
	clr	r20				;counter dementsprechend prpariert werden mssen
	;----------------------
	;serCOM hat's entschieden
	;==tx
	;alles bisher empfangene 
	;lschen!
	sts	fifoIN,r20
	sts	fifoOUT,r20
	lds	r16,fifoVAR
	cbr	r16,1<<fifoERR
	cbr	r16,1<<fifoNEW
	sts	fifoVAR,r16
;----------------------------------------------------------------------------------
;RX/TX
;----------------------------------------------------------------------------------
	;-> receive dor0 transmit
_uart_direction_check:
	sbrc	r19,SEdir		;<1> data tx / <0> data rx		
	rjmp	_serACTIVtx	;..... transmit
	rjmp	_STreceive	;..... receive	












;-------------------------------------------------------------------------------------------
;-> RECEIVING
;-> check receive conditions
;NEW VERSION !!!
;interrupt based
;a) check fifo state
;b) clear fifoNEW flag !
;-------------------------------------------------------------------------------------------
_STreceive:
    ;TEST
    cli
_STreceiveLOOP:

	lds		r16,fifoVAR
	cbr	r16,1<<fifoNEW
	sts	fifoVAR,r16
	lds	zh,fifoIN		;fifo in counter
	lds	zl,fifoOUT		;fifo out counter
	mov	r17,zl
	cp	zh,zl
	breq	_serMISSINGrx	;same...countdown to timeout

;----------------------------------
;fifo-counter vor berlauf schtzen
;bereichsende erreicht dann wieder
;bei 0 beginnen...
;----------------------------------
	;fifo counters max.254
	cpi	zh,254	;fifoIN am limit
	brlo	_fifoRANGEok ;ne...
	tst	zl		;fifoOUT am unteren limit
	breq	_fifoRANGEok ; nix zu berechnen !
	mov	r10,zl	;fifoOUT(old)
	mov	r11,zh	;fifoIN(old)
	sub 	r11,r10	;fifoIN-fifoOUT = fifoDIFF	
	;nun fifoOUT auf 0
	clr	r10
	sts	fifoOUT,r10	;fifoOUT(new)=0
	;nun fifoIN == fifoDIFF
	sts	fifoIN,r11	;fifoIN(new)

;----------------------------------
;fifoIN-daten kopieren
;----------------------------------
_fifoCOPY:
	ldi	zh,high(fifoSEG)
	ldi	zl,low(fifoSEG)
	mov	yl,r17;fifoOUT(old)
	clr	yh
	subi	yl,low(-fifoSEG)
	sbci	yh,high(-fifoSEG)
_fifoCOPYloop:
	ld	r16,y+	;daten aus alter fifoOUT-position holen
	st	z+,r16	;daten nach neuer position
	inc	r10		;fifoOUT(new)+1
	cp	r11,r10	;fifoIN(new) == fifoOUT(new)
	brne	_fifoCOPYloop
	rjmp	_STreceiveLOOP  ;TEST

;---------------------------------
;FIFO RANGE OK!
;---------------------------------
_fifoRANGEok:
   ;TEST 
    sei
	;take fifoOUT adress
	inc	r17
	sts	fifoOUT,r17		;store fifo out counter
	clr	zh
	subi	zl,low(-fifoSEG)
	sbci	zh,high(-fifoSEG)
	ld	r17,z			;stored data to r17
	rjmp	_uartRXpreCK


;---------------------------------------------------------
;!!! --> hier timeout integrieren !!!
;unvollstndiges frame (fehlende bytes) 
;knnten alles blockieren !!!
;a)timebase   = (1/baud)*9*2
;b)routine reset !!!
;---------------------------------------------------------
_serMISSINGrx:	
    ;TEST
    sei
	lds	r16,WToutRX		;timeoutvariable holen
	tst	r16
	brne	_rxBYTEno
	;RX-ABBRUCH !!!
	ldi	r16,WToutVAR
	sts	WToutRX,r16		;timeout preset !!!
	rjmp	_errRX_DATA		;timeout -> keine brauchbaren daten mehr zu erwarten - also break !
_rxBYTEno:
	dec	r16
	sts	WToutRX,r16	
	rjmp	GO_HOME			;....nein!
GO_HOMEx:
	rjmp	GO_HOME




;-------------------------------------------
;-> wenn's flag gesetzt ist
;   kann das byte aus dem
;   udr0 abgeholt werden
;   flag automatisch ge-
;   lscht !	
;...........................................	
_uartRXpreCK:
	ldi	r16,WToutVAR
	sts	WToutRX,r16		;timeout preset !!!	
	lds		r16,fifoVAR
	sbrc		r16,fifoERR		;error...
	rjmp		_errRX_DATA		;da wird irgendwas unbrauchbares empfangen....
	rjmp	_serACTIVrx


;-------------------------------------------------------------------------------------------
;->TIMEOUT / ERROR  RECEIVING
;-------------------------------------------------------------------------------------------
_errRX_DATA:
	;also flags wieder lschen !
	lds	r16,fifoVAR
	cbr	r16,1<<fifoERR
	sts	fifoVAR,r16
	rjmp	_RXerror		;frame nicht komplett oder fehlerhaft - also ,ERROR'




;-------------------------------------------------------------------------------------------
;->ACTIV RECEIVING
;header bytes nach savAREArx & RXbufferST wg. acknowledge !
;rest nur nach RXbufferST !
;-------------------------------------------------------------------------------------------
_serACTIVrx:
	cpi	r20,7
	brsh	_serACTIVrxSTANDARD	
	clr	zh
	mov	zl,r20
	;addiere startadresse buffer 
	subi	zl,low(-savAREArx)	;Add low byte ( x -(-y)) = x + y
	sbci	zh,high(-savAREArx)	;Add high byte with carry	
	st		z,r17
_serACTIVrxSTANDARD:
	clr	zh
	mov	zl,r20
	;addiere startadresse buffer 
	subi	zl,low(-RXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh,high(-RXbufferST)	;Add high byte with carry	
	st		z,r17

;-------------------------------------------------------------------------------------------
;->BYTE COUNTER PRFEN & PRE-SELECTION
;-------------------------------------------------------------------------------------------
	cpi	r20,5	
	breq	loadSIZE
	brlo	gogohome
	rjmp	tstLIM
	;<10 wird der noch nicht geprft
gogohome:
	;byte-cnt +1
	inc	r20
	rjmp	GO_HOME

;------------------------------------------------------
;=5 frame size laden
;------------------------------------------------------
loadSIZE:
	clr	zh			;hole den frame-size wert aus dem sram
	ldi	zl,5
	subi	zl,low(-savAREArx)	;Add low byte ( x -(-y)) = x + y
	sbci	zh,high(-savAREArx)	;Add high byte with carry
	ld	r16,z
	sts	TEframeSIZ,r16
	;byte-cnt +1
	inc	r20
	rjmp	GO_HOME	

;------------------------------------------------------
;>5 limit testen
;------------------------------------------------------
tstLIM:
    ;byte-cnt +1
	inc	r20
	lds	r16,TEframeSIZ
	cp	r20,r16	
	;hier stndig aktuellen byte-counter
	;mit der empfangenen frame-size ver-
	;gleichen
	brsh	_READYtransfer	
	rjmp	GO_HOME	

;......................................
;alle bytes empfangen - LAB Adresse 
;auf bereinstimmung prfen !
;......................................
_READYtransfer:
	lds	r16,net_sou_adress
	;host-systemadresse
	clr	zh	
	ldi	zl,1				;empfangene host-adresse		
	subi	zl,low(-savAREArx)	
	sbci	zh,high(-savAREArx)	
	ld	r17,z
	cp	r17,r16
	brne	_ADRnotEQU
;-----------------------------
;adresse OK / oder broadcast !
;-----------------------------
_FRAMEok:
	sbr	r19,1<<RXframe		;setzen da frameinhalt noch geprft werden soll
	cbr	r19,1<<SEerror		;egal ob hardware-error da frameinhalt ja noch geprft werden soll
	rjmp	_RXhome
	
;-----------------------------
;adressse falsch !
;broadcast ?
;-----------------------------
_ADRnotEQU:
	;empfangener mi
	ldi	zl,low(savAREArx)	
	ldi	zh,high(savAREArx)	
	ld	r17,z
	sbrs	r17,b_cast		;0=broadcast...	
	rjmp	_FRAMEok		;broadcast !
	rjmp	_RXerror

;-----------------------------
;frame defekt
;-----------------------------
_RXerror:
	sbr	r19,1<<RXframe		;wird von sercom gelscht - keine weiterverarbeitung
	sbr	r19,1<<SEerror		;egal ob fehler da keine verarbeitung im sercom!
	rjmp	_RXhome




;-----------------------------
;exit.....
;-----------------------------

_RXhome:
	clr	r20
	cbr	r19,1<<SEbusy		;SEbbusy lschen da hardwareseitig komplett fertig					
	;fifo clear
;	clr		r17
;	sts		fifoIN,r17
;	sts		fifoOUT,r17
	rjmp	GO_HOME		














;-------------------------------------------------------------------------------------------
;->TRANSMITTING 
;NEW: Communication by SPI
;-> start transmitting if spi isn't busy 
;_SS = high
;_nSEL = high
;-> tell the system active NET-DATA transmiting
;_ifSEL = low
;
;-------------------------------------------------------------------------------------------

_serACTIVtx:
  	tst	r20
	breq   _serACTIVtxSTART 
    ;.................
    ;reset counter
    clr yh
    sts spiDEMcnt,yh
    ;................

    jmp _serACTIVtxGO

	;------------------------------------------------------------
	;try to transmit
	;if..
	;CHECK: _ifSEL still low if there's any rfm12 communication
	;	   spiBUSY always set if there's any HIF-data transfer
	;------------------------------------------------------------
_serACTIVtxSTART:
	sbis	pinb,_IFsel		    ;_IFsel =1 
	rjmp	_serACTIVtxWAIT		;=0 spi still busy
    ;start block bus !
    cbi ddrb,_ifSEL
    cbi portb,_ifSEL
    nop
     ;now block
    sbi ddrb,_ifSEL
    cbi portb,_ifSEL
 
    rjmp    _serACTIVtxFREE

;--------------------------------------
;conceptXM1.2 wrde gerne senden
;aber spi noch busy...
;falls dies innerhalb einer angemessenen
;zeit nicht mglich ist - anforderung 
;lschen !
;--------------------------------------
_serACTIVtxWAIT:
	clr	yh
	clr	yl
	lds	zl,WTackCTl
	lds	zh,WTackCTh
	cp	zl,yl
	cpc	zh,yh
	breq	_INTFbusyX		;zeit ist abgelaufen...
	;counter -1
	clr	yh
	ldi	yl,1
	sub	zl,yl
	sbc	zh,yh
	sts	WTackCTl,zl
	sts	WTackCTh,zh
	jmp	GO_HOME			    ;zeit luft  === noch in der schleife !	
_INTFbusyX:
    ;default waittime
    ldi r16,20
	;load counters mit konstante wg.handshake
	ldi	zl,low(TIrxBYTE)
	ldi	zh,high(TIrxBYTE)
	;konst(16bit) * framesize
	clr	yh
	mul	r16,zl	
	movw	r7:r6, r1:r0
	mul	yh, zl		
	add	r7, r0
	mul	zh, yl	
	add	r7, r0
	movw	zh:zl, r7:r6
	;..........................
	;NEW*** + net_sou_adress
	;minimiert kollisionen !
	clr	yh
	lds	yl,net_sou_adress
	add	zl,yl
	adc	zh,yh
	;..........................	
	sts	WTackCTl,zl
	sts	WTackCTh,zh
	;+++++++++++++++++++++++++++++++++++++++++
	clr	r16
	sts	WIREset,r16
	clr	r20
	;standard defaults
	cbr	r19,1<<TXframe	
	cbr	r19,1<<SEdir			;
	cbr	r19,1<<SEbusy
    ;transfer ready
    cbr r19,1<<NETrun
	rjmp	GO_HOME	





;...............................
;bus free - transmit via spi
;...............................
_serACTIVtxFREE:
    ;transfer activ
    sbr r19,1<<NETrun
	;-----------------------------------------------------
	;set SPI to master-mode
	;-----------------------------------------------------
	call	_SPImasterINIT
	;-----------------------------------------------------
	;lade frame-size 
	;-----------------------------------------------------
	clr	zh			;hole den frame-size wert aus dem sram
	ldi	zl,5
	subi	zl,low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh,high(-TXbufferST)	;Add high byte with carry
	ld	r16,z
	;framesize speichern
	sts	TEframeSIZ,r16

;.........................................
;first byte...identifier 0xd4
;.........................................
	ldi		r16,0xd4	
	out	spdr,r16	;zu bertragende daten
	inc	r20
	rjmp	GO_HOME	





;.........................................
;dynamic run - lade daten aus buffer
;.........................................
_serACTIVtxGO:
	clr		zh			;adressiere  sram
	mov		zl,r20			;addiere byte counter ....
    subi    zl,1
	subi	zl,low(-TXbufferST)	;Add low byte ( x -(-y)) = x + y
	sbci	zh,high(-TXbufferST)	;Add high byte with carry
	ld	r16,z			;hole tx-byte aus sram
;****************************************
;daten via spi verschicken
;****************************************
_serACTIVtxlOOP:
		sbis	spsr,spif
		rjmp	_serACTIVtxlOOP	;warte bis daten bertragen
		out	spdr,r16	;zu bertragende daten
	
;.........................................	
;hier stndig aktuellen byte-counter
;mit der definierten frame-size ver-
;gleichen
;.........................................
	lds		r16,TEframeSIZ
	cpi	   	r20,0xff
	breq		_READYtx
	;r20=r16 == ready
	cp		r20,r16	
	brsh		_READYtx
	inc	r20
	rjmp	GO_HOME	
;.........................................
;datenbertragung fertig !
;.........................................
_READYtx:
	;load counters mit konstante wg.handshake
	ldi	zl,low(TIrxBYTE)
	ldi	zh,high(TIrxBYTE)
	;konst(16bit) * framesize
	clr	yh
	mul	r16,zl	
	movw	r7:r6, r1:r0
	mul	yh, zl		
	add	r7, r0
	mul	zh, yl	
	add	r7, r0
	movw	zh:zl, r7:r6
	;..........................
	;NEW*** + net_sou_adress
	;minimiert kollisionen !
	clr	yh
	lds	yl,net_sou_adress
	add	zl,yl
	adc	zh,yh
	;..........................	
	sts	WTackCTl,zl
	sts	WTackCTh,zh
	;+++++++++++++++++++++++++++++++++++++++++
	clr	r16
	sts	WIREset,r16
	clr	r20
	;standard defaults
	cbr	r19,1<<TXframe	
	cbr	r19,1<<SEdir			;
	cbr	r19,1<<SEbusy
    ;transfer ready
    cbr r19,1<<NETrun
;	cbr	r22,1<<newTX
	;---------------------------------------------
	;set SPI to slave-mode 
	;---------------------------------------------
 _SERactivlOOPF:
	sbis	spsr,spif
	rjmp	 _SERactivlOOPF	;warte bis daten bertragen
	call	_SPIslaveINIT
	;---------------------------------------------
	;
	rjmp	GO_HOME

;-------------------------------------------------------------------------------------------
;->GO_HOME
;-------------------------------------------------------------------------------------------
;
GO_HOME:
	sts	SEflagVARa,r19		;speichere SEflag Variable
	sts	SEFLAGVARb,r22		;  "      "      "
	sts	SEflagVARc,r21
	sts	CNTbyte,r20		;speichere CNTbyte variable
	ret


