;--------------------------------------------------------------------------------------
;-------------- powerrail steuerung (soft-on/off, delayed off, current - sense...) ----
;--------------------------------------------------------------------------------------

;------ ebenfalls integriert: 	- softstart powerrail via pwm0 (1-255*1ms)    			---- e2prom 0x2(Pdelay1 sram)
;			      				- softdown  powerrail via pwm0 (1-255*1ms)     			---- e2prom 0x3(Pdelay2 sram)
;			      				- delay current sense mode (1-255*1ms)         			---- e2prom 0x4(Pdelay3 sram)
;			      				- delayed off (1-255*1ms)                      			---- e2prom 0x5(Pdelay4 sram)
;			      				- current switch value                         			---- e2prom 0x27 == adcHYSe2p
;			      				- flag-variable   PWRflag                      			---- sram   PWRflag
;			      				- temp. 1ms backup			     						---- sram   R1msBK
;			      				- 1ms variable (aus timeslot...)	             		---- sram   ms1CNT
;
;	out	ocr1al,rxx	;pwm0 folgt rxx-value
;-------------------------------------------------------------------------------------------
;modes:
;a)softstart (0-255)
;         __|_______________........
;       __  |
;     __    | 
;   __      |
;|__        |
;|          |
;start...  max.  
;0%        100%

;a)soft off (0-255)
;...|__
;   |  __
;   |    __
;   |      __
;   |        __
;   |          __|
;  max...       aus  
;  100%          0%


;a)delayed off (0-255) &
;...|____________|
;   |  			 |
;   |            |
;   |      		 |
;   |            |
;   |            |
;  max...       aus   
;  100%          0%

;* PWRflag (PWRflag)-Tabelle :
; _______________________________________________
;|     |     |     |     |     |     |     |     |
;|bit7 |bit6 |bit5 |bit4 |bit3 |bit2 |bit1 |bit0 |
;|_____|_____|_____|_____|_____|_____|_____|_____|
;|rail |port |rail |rail |rail |rail |cur. |cur. |
;|s-cut|on   |d-off|down |up   |on   |up   |on   |
;|_____|_____|_____|_____|_____|_____|_____|_____|

_PWRrail:
	lds	r19,ms1CNT	;1ms counter lesen
	lds	r17,R1msBK	;1ms backup lesen
	cp	r19,r17
	breq	_noPcountJP	;werte noch gleich, also noch keine 100ms vergangen
	mov	r17,r19		;... wieder angleichen !
	sts	R1msBK,r17
	;
	;nachfolgende abfragen finden alle 100ms statt!
	;
	lds	r16,PWRflag	;power-flag-variable
	;
	sbrc	r16,railON	;powerrail... 
	rjmp	_railONconditions
	sbrc	r16,delOFF	;delayed off aktiv
	rjmp	_delayedOFF
	sbrc	r16,softDW	;soft down aktiv
	rjmp	_softDOWN
	;portabschaltung knnte genau hier erfolgen !!!
	rjmp	_BKfRAIL
	;
_railONconditions:
	sbrc	r16,softUP	;softstart aktiv & delayed current sense
	rjmp	_softSTART
	sbrc	r16,senseON	;current sense
	rjmp	_currentSENSE
	rjmp	_delayedSENSE
_noPcountJP:
	rjmp	_noPcount




;**** rail coming up ***
;--------------------------------------------------------
;------- softstart powerrail ----------------------------
;--------------------------------------------------------
_softSTART:
	lds	r18,Pdelay1	;softstartparameter 
	lds	r17,stSOFT	;hole temp. softstartvariable
	lds	r20,stRAMP	;hole temp. ramp-counter
	cpi	r20,0xff	;maximaler rampwert erreicht?!
	brne	_softCK
	cbr	r16,1<<softUP	;fertig.....
	rjmp	_BKfRAILy


	;hier ports aktivieren
_softCK:
	sbrc	r16,portON	;schon aktiv ?	
	rjmp	_softCKb    ;ja...
	;nein...
	sbr	r16,1<<portON	;ports aktivieren
		;..........................................
		;event
		ldi	zl,9
		sts	eventTE,zl			;port enabled event setzen
		call	_MAKEeventSUB	;submodul !
		;..........................................
        call    _BKraileep


_softCKb:
	tst	r17
	breq	_softCUT	
	dec	r17
	rjmp	_BKfRAILy
	;
_softCUT:
	inc	r20		;ramp-counter +1
	out	ocr1al,r20
	sts	stRAMP,r20	;ramp-counter nach stRAMP sram sichern
	mov	r17,r18		;temp. variable mit SOLL parameter reloaden
_BKfRAILy:
	sts	stSOFT,r17	;temp.variable nach stSOFT sram sichern
	rjmp 	_BKfRAIL	;zurck...
	;INFO: falls delayed sense 
	;unmittelbar danch aktiviert 
	;werden soll mu der letzte
	;sprungbefehl gelscht werden
	;



;--------------------------------------------------------
;------- delayed sense ----------------------------------
;--------------------------------------------------------
_delayedSENSE:
	lds	r17,teCOUNT	;temp.counter 255*1ms=255ms
	inc	r17
	cpi	r17,0x14	;20dec	
	brne	_SNSback
	clr	r17
	;     
	lds	r18,Pdelay3	;sense delay SOLL
	lds	r19,teRAILcnt	;sram-variable lesen
	inc	r19
	cp	r18,r19
	brne	_senseLIMIT	;
	clr	r19		;soll = ist
	sbr	r16,1<<senseON	;nun,kurzschlusicherung aktiv
_senseLIMIT:
	sts	teRAILcnt,r19	;aktueller delay-wert
_SNSback:	
	sts	teCOUNT,r17
	rjmp	_BKfRAIL	;zurck....


;**** rail coming down ****



;--------------------------------------------------------
;------- delayed off powerrail --------------------------
;--------------------------------------------------------
_delayedOFF:
	lds	r17,teCOUNT	;temp.counter
	inc	r17
	cpi	r17,0x14	;20 dec	
	brne	_OFFback
	clr	r17
	;
	lds	r18,Pdelay4	;delayed off SOLL
	lds	r19,teRAILcnt	;sram-variable lesen
	inc	r19		;erhhe variable...
	cp	r18,r19
	brne	_DEoffLIMIT	;
	clr	r19		;soll = ist
	sbr	r16,1<<softDW	;nun, falls gewnscht, soft-down !
	cbr	r16,1<<delOFF
_DEoffLIMIT:
	sts	teRAILcnt,r19	;aktueller delay-off-wert
_OFFback:	
	sts	teCOUNT,r17
	rjmp	_BKfRAIL	;zurck....




;--------------------------------------------------------
;------- softdown powerrail ----------------------------
;--------------------------------------------------------
_softDOWN:
	lds	r18,Pdelay2		;softdownparameter SOLL
	lds	r17,stSOFT	;hole temp. softvariable
	lds	r20,stRAMP	;hole temp. ramp-counter
	tst	r20		;minimaler rampwert erreicht?!
	brne	_softdwCK
	cbr	r16,1<<softDW	;fertig.....
	cbr	r16,1<<portON	;ports DEaktivieren
		;..........................................
		;events neu
		ldi	zl,10
		sts	eventTE,zl			;port disabled event setzen
		call	_MAKEeventSUB	;submodul !
		;..........................................
        call    _BKraileep
	;
	;hier port-abschaltung !
	;...aber zuerst blend-counter stoppen
	;blend device 0...7 soll <=> ist
	ldi		zl,low(CpareA)	;soll
	ldi		zh,high(CpareA)
	ldi		yl,low(CpareI1) ;ist
	ldi		yh,high(CpareI1)
_softDOWNstopLOOP:
	ld		r18,y+
	st		z+,r18
	cpi	zl,low(CpareA+8)
	ldi	r18,high(CpareA+8)	
	cpc	zh,r18	
	brlo	_softDOWNstopLOOP
	clr	r18
	out	porta,r18
	out	portc,r18
	rjmp	_BKgRAILy
	;
	;
_softdwCK:
	tst	r17
	breq	_softdwCUT	
	dec	r17
	rjmp	_BKgRAILy
	;
_softdwCUT:
	dec	r20		;ramp-counter +1
	out	ocr1al,r20
	sts	stRAMP,r20	;ramp-counter nach stRAMP sram sichern
	mov	r17,r18		;temp. variable mit SOLL parameter  reloaden
_BKgRAILy:
	sts	stSOFT,r17	;temp.variable nach stSOFT sram sichern
	rjmp 	_BKfRAIL	;zurck...


;**** running system ****

;--------------------------------------------------------
;------- current sense ----------------------------------
;--------------------------------------------------------
_currentSENSE:
	sbrs	r16,curON	;kurzschluabschaltung aktiv?
	rjmp	_senseOKDO
	;
	ldi	r20,adcHYSe2p		;current-limit-adresse
	rcall 	_READe2DATA	;... limit-daten in r18	
	lds	r19,ADCpA		;adcA-wert lesen
	;preset=0 ... keine auswertung
	tst	r18
	brne	_currentSENSEgo
	rjmp	_senseOKDO	;zurck....
	;prfe...
_currentSENSEgo:
	cp	r19,r18
	brlo	_senseOKDO	;ist < limit
	;
	ldi	r16,0b10000001	;setzt kurzschluflag
	clr	r17
	out	ocr1ah,r17
	out	ocr1al,r17	
		;..........................................
		;events neu
		ldi	zl,19
		sts	eventTE,zl			;shortcut event setzen
		call	_MAKEeventSUB	;submodul !
		;..........................................
	;
_senseOKDO:
	rjmp	_BKfRAIL		;back



;-----------------------------------------------------------------------
;-------------- read e2prom --------------------------------------------
;-----------------------------------------------------------------------

_READe2DATA:
	sbic	eecr,eewe		; EEPE nicht gelscht ?
	rjmp	_READe2DATA		; warte...
	clr	r21
	out	eearh,r21
	out	eearl,r20		; adressierung e2prom
	sbi	eecr,eere		; bereite bertragung aus e2prom vor 
	in	r18,eedr		; kopiere daten aus e2prom nach r18
	ret
;-----------------------------------------------------------------------
;--------------- ... und zurck ----------------------------------------
;-----------------------------------------------------------------------	
_BKfRAIL:
	sts	PWRflag,r16	;flag's sichern
_noPcount:	
	ret



;.........................
;sub backup power flag
;.........................
_BKraileep:
    ldi zl,low(PWRflagadr)
    ldi zh,high(PWRflagadr)
_BKraileepWT:
	sbic	eecr,eewe		; EEPE nicht gelscht ?
	rjmp	_BKraileepWT			; warte...
	out	eearh,zh
	out	eearl,zl		; adressierung e2prom
	out	eedr,r16          ; kopiere ins'e2prom-datenregister
	sbi	eecr,eemwe	
	sbi	eecr,eewe		;set EEPROM Write strobe
					; nach 4 maschinenzyklen ist der schreibprozess abgeschlossen 		
    ret


